"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Axis", function () {
    var horizontalOrientations = ["top", "bottom"];
    var verticalOrientations = ["left", "right"];
    var orientations = horizontalOrientations.concat(verticalOrientations);
    var isHorizOrient = function (orientation) { return horizontalOrientations.indexOf(orientation) > -1; };
    var numAttr = TestMethods.numAttr;
    horizontalOrientations.forEach(function (horizontalOrientation) {
        var alignment = horizontalOrientation === "top" ? "bottom" : "top";
        it("defaults to a " + alignment + " y alignment with orientation " + horizontalOrientation, function () {
            var scale = new Plottable.Scales.Linear();
            var axis = new Plottable.Axis(scale, horizontalOrientation);
            chai_1.assert.strictEqual(axis.yAlignment(), alignment, "y alignment for " + horizontalOrientation + " is default " + alignment);
            axis.destroy();
        });
    });
    verticalOrientations.forEach(function (verticalOrientation) {
        var alignment = verticalOrientation === "left" ? "right" : "left";
        it("defaults to a " + alignment + " x alignment with orientation " + verticalOrientation, function () {
            var scale = new Plottable.Scales.Linear();
            var axis = new Plottable.Axis(scale, verticalOrientation);
            chai_1.assert.strictEqual(axis.xAlignment(), alignment, "x alignment for " + verticalOrientation + " is default " + alignment);
            axis.destroy();
        });
    });
    it("throws an error when setting an invalid orientation", function () {
        var scale = new Plottable.Scales.Linear();
        chai_1.assert.throws(function () { return new Plottable.Axis(scale, "blargh"); }, "unsupported");
    });
    it("throws an error when setting a negative tickLabelPadding", function () {
        var scale = new Plottable.Scales.Linear();
        var axis = new Plottable.Axis(scale, "bottom");
        chai_1.assert.throws(function () { return axis.tickLabelPadding(-1); }, "must be positive");
        axis.destroy();
    });
    it("throws an error when setting a negative margin", function () {
        var scale = new Plottable.Scales.Linear();
        var axis = new Plottable.Axis(scale, "right");
        chai_1.assert.throws(function () { return axis.margin(-1); }, "must be positive");
        axis.destroy();
    });
    orientations.forEach(function (orientation) {
        it("updates the layout when updating the margin after rendering for orientation " + orientation, function () {
            var div = TestMethods.generateDiv();
            var scale = new Plottable.Scales.Linear();
            var axis = new Plottable.Axis(scale, orientation);
            axis.renderTo(div);
            var relevantDimensionF = function () { return isHorizOrient(orientation) ? axis.height() : axis.width(); };
            var axisSize = axis.innerTickLength() + axis.margin();
            chai_1.assert.strictEqual(relevantDimensionF(), axisSize, "axis size is the tick length and the margin");
            var newMargin = 20;
            axis.margin(newMargin);
            axisSize = axis.innerTickLength() + axis.margin();
            chai_1.assert.strictEqual(relevantDimensionF(), axisSize, "changing the margin size updates the size");
            axis.destroy();
            div.remove();
        });
    });
    orientations.forEach(function (orientation) {
        it("draws the baseline at the correct edge for " + orientation + " orientation", function () {
            var div = TestMethods.generateDiv();
            var scale = new Plottable.Scales.Linear();
            var axis = new Plottable.Axis(scale, orientation);
            axis.renderTo(div);
            var baseline = axis.content().select(".baseline");
            chai_1.assert.isFalse(baseline.empty(), "baseline exists");
            chai_1.assert.strictEqual(baseline.attr("x1"), orientation !== "left" ? "0" : String(axis.width()), "x1");
            chai_1.assert.strictEqual(baseline.attr("x2"), orientation !== "right" ? String(axis.width()) : "0", "x2");
            chai_1.assert.strictEqual(baseline.attr("y1"), orientation !== "top" ? "0" : String(axis.height()), "y1");
            chai_1.assert.strictEqual(baseline.attr("y2"), orientation !== "bottom" ? String(axis.height()) : "0", "y2");
            axis.destroy();
            div.remove();
        });
    });
    describe("setting the length of the tick marks", function () {
        it("can set for inner", function () {
            var scale = new Plottable.Scales.Linear();
            var axis = new Plottable.Axis(scale, "bottom");
            var tickLength = 10;
            chai_1.assert.strictEqual(axis.innerTickLength(tickLength), axis, "setting returns calling object");
            chai_1.assert.strictEqual(axis.innerTickLength(), tickLength, "retrieves set tick length");
            chai_1.assert.throws(function () { return axis.innerTickLength(-1); }, "must be positive");
            axis.destroy();
        });
        it("can set for ends", function () {
            var scale = new Plottable.Scales.Linear();
            var axis = new Plottable.Axis(scale, "bottom");
            var tickLength = 10;
            chai_1.assert.strictEqual(axis.endTickLength(tickLength), axis, "setting returns calling object");
            chai_1.assert.strictEqual(axis.endTickLength(), tickLength, "retrieves set tick length");
            chai_1.assert.throws(function () { return axis.endTickLength(-1); }, "must be positive");
            axis.destroy();
        });
        it("adjusts the height to the greater of innerTickLength and endTickLength", function () {
            var div = TestMethods.generateDiv();
            var scale = new Plottable.Scales.Linear();
            var axis = new Plottable.Axis(scale, "bottom");
            axis.showEndTickLabels(true);
            axis.renderTo(div);
            var expectedHeight = Math.max(axis.innerTickLength(), axis.endTickLength()) + axis.margin();
            chai_1.assert.strictEqual(axis.height(), expectedHeight, "height should be equal to the maximum of the two");
            var increasingInnerTickLength = axis.endTickLength() + 10;
            axis.innerTickLength(increasingInnerTickLength);
            chai_1.assert.strictEqual(axis.height(), increasingInnerTickLength + axis.margin(), "height should increase to inner tick length");
            var increasingEndTickLength = axis.innerTickLength() + 10;
            axis.endTickLength(increasingEndTickLength);
            chai_1.assert.strictEqual(axis.height(), increasingEndTickLength + axis.margin(), "height should increase to end tick length");
            var decreasingInnerTickLength = axis.endTickLength() - 10;
            axis.innerTickLength(decreasingInnerTickLength);
            chai_1.assert.strictEqual(axis.height(), increasingEndTickLength + axis.margin(), "height should not decrease");
            axis.destroy();
            div.remove();
        });
    });
    describe("enabling axis annotations", function () {
        var axis;
        var scale;
        beforeEach(function () {
            scale = new Plottable.Scales.Time();
            axis = new Plottable.Axis(scale, "bottom");
        });
        it("has annotations disabled by default", function () {
            chai_1.assert.isFalse(axis.annotationsEnabled(), "annotations are disabled by default");
        });
        it("can set if annotations are enabled", function () {
            var annotationsEnabled = true;
            chai_1.assert.strictEqual(axis.annotationsEnabled(annotationsEnabled), axis, "enabling/disabling annotations returns calling axis");
            chai_1.assert.strictEqual(axis.annotationsEnabled(), annotationsEnabled, "can set if annotations are enabled");
            var annotatedTicks = [new Date((scale.domain()[0].valueOf() + scale.domain()[1].valueOf()) / 2)];
            axis.annotatedTicks(annotatedTicks);
            var div = TestMethods.generateDiv();
            axis.renderTo(div);
            chai_1.assert.isFalse(axis.content().selectAll("." + Plottable.Axis.ANNOTATION_CIRCLE_CLASS).empty(), "annotations render");
            axis.destroy();
            div.remove();
        });
        it("can enable annotations after render", function () {
            var div = TestMethods.generateDiv();
            var domainValues = [scale.domain()[0].valueOf(), scale.domain()[1].valueOf()];
            var annotatedTicks = Plottable.Utils.Math.range(domainValues[0], domainValues[1], (domainValues[1] - domainValues[0]) / 10)
                .map(function (d) { return new Date(d); });
            axis.annotatedTicks(annotatedTicks);
            axis.renderTo(div);
            chai_1.assert.strictEqual(axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS).size(), 0, "no annotations by default");
            axis.annotationsEnabled(true);
            chai_1.assert.strictEqual(axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS).size(), annotatedTicks.length, "annotations render");
            div.remove();
        });
        it("can disable annotations after render", function () {
            var div = TestMethods.generateDiv();
            var domainValues = [scale.domain()[0].valueOf(), scale.domain()[1].valueOf()];
            var annotatedTicks = Plottable.Utils.Math.range(domainValues[0], domainValues[1], (domainValues[1] - domainValues[0]) / 10)
                .map(function (d) { return new Date(d); });
            axis.annotationsEnabled(true);
            axis.annotatedTicks(annotatedTicks);
            axis.renderTo(div);
            chai_1.assert.strictEqual(axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS).size(), annotatedTicks.length, "annotations render");
            axis.annotationsEnabled(false);
            chai_1.assert.strictEqual(axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS).size(), 0, "no annotations by default");
            div.remove();
        });
    });
    describe("annotating ticks", function () {
        var axis;
        var scale;
        beforeEach(function () {
            scale = new Plottable.Scales.Linear();
            axis = new Plottable.Axis(scale, "bottom");
        });
        it("has no annotated ticks by default", function () {
            chai_1.assert.deepEqual(axis.annotatedTicks(), [], "no annotated ticks by default");
            axis.destroy();
        });
        it("can set the annotated ticks", function () {
            var domainValues = [scale.domain()[0].valueOf(), scale.domain()[1].valueOf()];
            var annotatedTicks = Plottable.Utils.Math.range(domainValues[0], domainValues[1], 10);
            chai_1.assert.strictEqual(axis.annotatedTicks(annotatedTicks), axis, "setting annotated ticks returns calling axis");
            chai_1.assert.deepEqual(axis.annotatedTicks(), annotatedTicks, "can set the annotated ticks");
            axis.annotationsEnabled(true);
            var div = TestMethods.generateDiv();
            axis.renderTo(div);
            chai_1.assert.deepEqual(axis.content().selectAll(".annotation-label").data(), annotatedTicks, "annotated ticks set");
            axis.destroy();
            div.remove();
        });
        it("re-renders when annotated ticks are set after render", function () {
            var domainValues = [scale.domain()[0].valueOf(), scale.domain()[1].valueOf()];
            var annotatedTicks = Plottable.Utils.Math.range(domainValues[0], domainValues[1], 10);
            var div = TestMethods.generateDiv();
            axis.annotationsEnabled(true);
            axis.renderTo(div);
            chai_1.assert.strictEqual(axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS).size(), 0, "no annotations by default");
            axis.annotatedTicks(annotatedTicks);
            chai_1.assert.strictEqual(axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS).size(), annotatedTicks.length, "annotated ticks annotated");
            var annotationFormatter = axis.annotationFormatter();
            var annotationLabels = axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS);
            annotationLabels.each(function (d, i) {
                chai_1.assert.strictEqual(d3.select(this).text(), annotationFormatter(annotatedTicks[i]), "annotated tick " + i + " has been formatted");
            });
            div.remove();
        });
    });
    describe("formatting annotation ticks", function () {
        var axis;
        var scale;
        beforeEach(function () {
            scale = new Plottable.Scales.Linear();
            axis = new Plottable.Axis(scale, "bottom");
        });
        it("uses the identity formatter by default", function () {
            var annotationFormatter = axis.annotationFormatter();
            var identityFormatter = Plottable.Formatters.identity();
            var testString = "testString";
            chai_1.assert.strictEqual(annotationFormatter(testString), identityFormatter(testString), "uses the identity formatter by default");
            axis.destroy();
        });
        it("can set the annotation formatter", function () {
            var annotationFormatter = function (d) { return d + "foo"; };
            chai_1.assert.strictEqual(axis.annotationFormatter(annotationFormatter), axis, "setting annotation formatter returns calling axis");
            var testString = "testString";
            chai_1.assert.strictEqual(annotationFormatter(testString), axis.annotationFormatter()(testString), "can set the annotated ticks");
            var annotatedTicks = Plottable.Utils.Math.range(scale.domain()[0], scale.domain()[1], 10);
            axis.annotatedTicks(annotatedTicks);
            axis.annotationsEnabled(true);
            var div = TestMethods.generateDiv();
            axis.renderTo(div);
            var annotationLabels = axis.content().selectAll(".annotation-label");
            chai_1.assert.strictEqual(annotationLabels.size(), annotatedTicks.length, "same number of annotation labels as annotated ticks");
            annotationLabels.each(function (d, i) {
                chai_1.assert.strictEqual(d3.select(this).text(), annotationFormatter(d), "formats tick " + i);
            });
            axis.destroy();
            div.remove();
        });
        it("re-renders when the annotation formatter is changed", function () {
            var div = TestMethods.generateDiv();
            var annotatedTicks = Plottable.Utils.Math.range(scale.domain()[0], scale.domain()[1], 10);
            axis.annotationsEnabled(true);
            axis.annotatedTicks(annotatedTicks);
            axis.renderTo(div);
            chai_1.assert.strictEqual(axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS).size(), annotatedTicks.length, "annotations have rendered");
            var bazFormatter = function (d) { return d + "baz"; };
            axis.annotationFormatter(bazFormatter);
            var annotationLabels = axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS);
            chai_1.assert.strictEqual(annotationLabels.size(), annotatedTicks.length, "annotations have rendered");
            annotationLabels.each(function (d, i) {
                chai_1.assert.strictEqual(d3.select(this).text(), bazFormatter(annotatedTicks[i]), "new formatter used for annotation label " + i);
            });
            axis.destroy();
            div.remove();
        });
    });
    describe("setting the number of annotation tiers", function () {
        var axis;
        beforeEach(function () {
            axis = new Plottable.Axis(new Plottable.Scales.Linear(), "bottom");
        });
        it("one annotation tier by default", function () {
            chai_1.assert.deepEqual(axis.annotationTierCount(), 1, "one annotation tier by default");
            axis.destroy();
        });
        it("can set the annotation tier count", function () {
            var annotationTierCount = 5;
            chai_1.assert.strictEqual(axis.annotationTierCount(annotationTierCount), axis, "setting annotation tier count returns calling axis");
            chai_1.assert.deepEqual(axis.annotationTierCount(), annotationTierCount, "can set the annotation tier count");
            axis.annotationsEnabled(true);
            var div = TestMethods.generateDiv();
            axis.renderTo(div);
            var oldAxisHeight = axis.height();
            var increaseAmount = 2;
            axis.annotationTierCount(axis.annotationTierCount() + increaseAmount);
            chai_1.assert.operator(axis.height(), ">", oldAxisHeight, "axis takes number of tiers into account");
            axis.destroy();
            div.remove();
        });
        it("throws an error when annotation tier count is not valid", function () {
            var invalidAnnotationTierCounts = [-1, -100];
            invalidAnnotationTierCounts.forEach(function (annotationTierCount) {
                var currentAnnotationTierCount = axis.annotationTierCount();
                chai_1.assert.throws(function () { return axis.annotationTierCount(annotationTierCount); }, "annotationTierCount cannot be negative");
                chai_1.assert.strictEqual(axis.annotationTierCount(), currentAnnotationTierCount, "annotationTierCount should not be changed to invalid values");
            });
            axis.destroy();
        });
    });
    describe("drawing annotation elements", function () {
        orientations.forEach(function (orientation) {
            var scale = new Plottable.Scales.ModifiedLog();
            scale.domain([0, 300]);
            var annotatedTicks = [3, 100, 250];
            it("renders annotation lines, circles, and text extending out from baseline to rect with orientation " + orientation, function () {
                var div = TestMethods.generateDiv();
                var axis = new Plottable.Axis(scale, orientation);
                axis.annotatedTicks(annotatedTicks);
                axis.annotationsEnabled(true);
                axis.renderTo(div);
                var annotationRects = axis.content().selectAll("." + Plottable.Axis.ANNOTATION_RECT_CLASS);
                var annotationCircles = axis.content().selectAll("." + Plottable.Axis.ANNOTATION_CIRCLE_CLASS);
                chai_1.assert.strictEqual(annotationCircles.size(), annotatedTicks.length, "circle for each annotated tick");
                var axisPointX = function (index) {
                    var correspRect = d3.select(annotationRects.nodes()[index]);
                    return isHorizOrient(orientation) ? numAttr(correspRect, "x") : (orientation === "left" ? axis.width() : 0);
                };
                var axisPointY = function (index) {
                    var correspRect = d3.select(annotationRects.nodes()[index]);
                    return isHorizOrient(orientation) ? (orientation === "top" ? axis.height() : 0) : numAttr(correspRect, "y");
                };
                annotationCircles.each(function (d, i) {
                    var annotationCircle = d3.select(this);
                    chai_1.assert.closeTo(numAttr(annotationCircle, "cx"), axisPointX(i), window.Pixel_CloseTo_Requirement, "circle " + i + " at start");
                    chai_1.assert.closeTo(numAttr(annotationCircle, "cy"), axisPointY(i), window.Pixel_CloseTo_Requirement, "circle " + i + " at the top");
                });
                var annotationLines = axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LINE_CLASS);
                chai_1.assert.strictEqual(annotationLines.size(), annotatedTicks.length, "line for each annotated tick");
                annotationLines.each(function (d, i) {
                    var annotationLine = d3.select(this);
                    var correspondingRect = d3.select(annotationRects.nodes()[i]);
                    var x2 = numAttr(correspondingRect, "x") + (orientation === "left" ? numAttr(correspondingRect, "width") : 0);
                    var y2 = numAttr(correspondingRect, "y") + (orientation === "top" ? numAttr(correspondingRect, "height") : 0);
                    chai_1.assert.closeTo(numAttr(annotationLine, "x1"), axisPointX(i), window.Pixel_CloseTo_Requirement, "line " + i + " x1");
                    chai_1.assert.closeTo(numAttr(annotationLine, "x2"), x2, window.Pixel_CloseTo_Requirement, "line " + i + " x2");
                    chai_1.assert.closeTo(numAttr(annotationLine, "y1"), axisPointY(i), window.Pixel_CloseTo_Requirement, "line " + i + " y1");
                    chai_1.assert.closeTo(numAttr(annotationLine, "y2"), y2, window.Pixel_CloseTo_Requirement, "line " + i + " y2");
                });
                var annotationLabelTexts = axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS + " text");
                chai_1.assert.strictEqual(annotationLabelTexts.size(), annotatedTicks.length, "label text for each annotated tick");
                annotationLabelTexts.each(function (d, i) {
                    var annotationLabel = d3.select(this);
                    var surroundingRect = d3.select(annotationRects.nodes()[i]);
                    TestMethods.assertBBoxInclusion(surroundingRect, annotationLabel);
                });
                axis.destroy();
                div.remove();
            });
        });
        orientations.forEach(function (orientation) {
            var scale = new Plottable.Scales.ModifiedLog();
            scale.domain([0, 300]);
            var annotatedTicks = [3, 100, 250];
            it("places the rectangle at the scaled x position for " + orientation + " orientation", function () {
                var div = TestMethods.generateDiv();
                var axis = new Plottable.Axis(scale, orientation);
                axis.annotatedTicks(annotatedTicks);
                axis.annotationsEnabled(true);
                axis.renderTo(div);
                var annotationRects = axis.content().selectAll("." + Plottable.Axis.ANNOTATION_RECT_CLASS);
                chai_1.assert.strictEqual(annotationRects.size(), annotatedTicks.length, "same number of annotation rects as ticks");
                var scaleAttribute = isHorizOrient(orientation) ? "x" : "y";
                annotationRects.each(function (d, i) {
                    var annotationRect = d3.select(this);
                    chai_1.assert.closeTo(numAttr(annotationRect, scaleAttribute), scale.scale(d), window.Pixel_CloseTo_Requirement, "rectangle " + i + " positioned at scaled position");
                });
                axis.destroy();
                div.remove();
            });
        });
        orientations.forEach(function (orientation) {
            var scale = new Plottable.Scales.ModifiedLog();
            scale.domain([0, 300]);
            var annotatedTicks = [3, 100, 250];
            it("places the first row right at the beginning of the annotation area for " + orientation + " orientation", function () {
                var div = TestMethods.generateDiv();
                var axis = new Plottable.Axis(scale, orientation);
                axis.annotatedTicks(annotatedTicks);
                axis.annotationsEnabled(true);
                axis.renderTo(div);
                var firstAnnotationRect = axis.content().select("." + Plottable.Axis.ANNOTATION_RECT_CLASS);
                var rectangleRowPos;
                var expectedRowPos;
                switch (orientation) {
                    case "bottom":
                        rectangleRowPos = numAttr(firstAnnotationRect, "y") + numAttr(firstAnnotationRect, "height");
                        expectedRowPos = axis.height() - axis.margin();
                        break;
                    case "top":
                        rectangleRowPos = numAttr(firstAnnotationRect, "y");
                        expectedRowPos = axis.margin();
                        break;
                    case "left":
                        rectangleRowPos = numAttr(firstAnnotationRect, "x");
                        expectedRowPos = axis.margin();
                        break;
                    case "right":
                        rectangleRowPos = numAttr(firstAnnotationRect, "x") + numAttr(firstAnnotationRect, "width");
                        expectedRowPos = axis.width() - axis.margin();
                        break;
                }
                chai_1.assert.closeTo(rectangleRowPos, expectedRowPos, window.Pixel_CloseTo_Requirement, "rectangle positioned right above margin");
                axis.destroy();
                div.remove();
            });
        });
        orientations.forEach(function (orientation) {
            var scale = new Plottable.Scales.ModifiedLog();
            scale.domain([0, 300]);
            var annotatedTicks = [50, 51];
            it("moves rectangles that would overlap to different rows for " + orientation + " orientation", function () {
                var div = TestMethods.generateDiv();
                var axis = new Plottable.Axis(scale, orientation);
                axis.annotatedTicks(annotatedTicks);
                axis.annotationsEnabled(true);
                axis.renderTo(div);
                var annotationRects = axis.content().selectAll("." + Plottable.Axis.ANNOTATION_RECT_CLASS);
                var firstAnnotationRect = d3.select(annotationRects.nodes()[0]);
                var secondAnnotationRect = d3.select(annotationRects.nodes()[1]);
                var positionAttr = isHorizOrient(orientation) ? "y" : "x";
                var offsetAttr = isHorizOrient(orientation) ? "height" : "width";
                chai_1.assert.strictEqual(Math.abs(numAttr(secondAnnotationRect, positionAttr) - numAttr(firstAnnotationRect, positionAttr)), numAttr(firstAnnotationRect, offsetAttr), "rectangle offset by previous rectangle");
                axis.destroy();
                div.remove();
            });
        });
    });
    it("hides annotations if rectangles are outside the annotation area", function () {
        var scale = new Plottable.Scales.Linear();
        var annotatedTicks = [50, 51, 150];
        scale.domain([0, 300]);
        var axis = new Plottable.Axis(scale, "bottom");
        axis.annotatedTicks(annotatedTicks);
        axis.annotationsEnabled(true);
        var div = TestMethods.generateDiv(300, 300);
        axis.renderTo(div);
        var annotationRects = axis.content().selectAll("." + Plottable.Axis.ANNOTATION_RECT_CLASS);
        chai_1.assert.strictEqual(annotationRects.size(), annotatedTicks.length, "same number of annotation rects as ticks");
        annotationRects.each(function (d, i) {
            var annotationRect = d3.select(this);
            var bbox = this.getBBox();
            var insideAnnotationArea = bbox.x >= 0 && bbox.x + bbox.width <= axis.width() &&
                bbox.y >= 0 && bbox.y + bbox.height <= axis.height() - axis.margin();
            if (insideAnnotationArea) {
                chai_1.assert.strictEqual(annotationRect.attr("visibility"), "visible", "rect " + i + " inside margin area should be visible");
            }
            else {
                chai_1.assert.strictEqual(annotationRect.attr("visibility"), "hidden", "rect " + i + " outside margin area should be not visible");
            }
        });
        axis.destroy();
        div.remove();
    });
    it("shows annotation circles regardless if rectangles are hidden", function () {
        var scale = new Plottable.Scales.Linear();
        var annotatedTicks = [50, 51, 150];
        scale.domain([0, 300]);
        var axis = new Plottable.Axis(scale, "bottom");
        axis.annotatedTicks(annotatedTicks);
        axis.annotationsEnabled(true);
        var div = TestMethods.generateDiv(300, 300);
        axis.renderTo(div);
        var annotationCircles = axis.content().selectAll("." + Plottable.Axis.ANNOTATION_CIRCLE_CLASS);
        chai_1.assert.strictEqual(annotationCircles.size(), annotatedTicks.length, "same number of annotation circles as ticks");
        annotationCircles.each(function (d, i) {
            var annotationCircle = d3.select(this);
            chai_1.assert.notStrictEqual(annotationCircle.attr("visibility"), "hidden", "circle " + i + " inside margin area should be visible");
        });
        axis.destroy();
        div.remove();
    });
    it("does not render the null as an annotation", function () {
        var annotatedTicks = [null];
        var scale = new Plottable.Scales.Time();
        scale.domain([new Date(1994, 11, 17), new Date(1995, 11, 17)]);
        var axis = new Plottable.Axis(scale, "bottom");
        axis.annotationsEnabled(true);
        var div = TestMethods.generateDiv(300, 300);
        axis.renderTo(div);
        axis.annotatedTicks(annotatedTicks);
        chai_1.assert.strictEqual(axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS).size(), 0, "no annotated ticks");
        axis.destroy();
        div.remove();
    });
    it("only renders unique annotated ticks", function () {
        var annotatedTicks = [150, 150, 200];
        var scale = new Plottable.Scales.Linear();
        scale.domain([100, 200]);
        var axis = new Plottable.Axis(scale, "bottom");
        axis.annotationsEnabled(true);
        var div = TestMethods.generateDiv(300, 300);
        axis.renderTo(div);
        axis.annotatedTicks(annotatedTicks);
        var annotatedTickSet = new Plottable.Utils.Set();
        annotatedTicks.forEach(function (annotatedTick) {
            annotatedTickSet.add(annotatedTick);
        });
        chai_1.assert.strictEqual(axis.content().selectAll("." + Plottable.Axis.ANNOTATION_LABEL_CLASS).size(), annotatedTickSet.size, "only unique annotations rendered");
        axis.destroy();
        div.remove();
    });
    it("places the tick at the top most row even if it is a duplicated tick", function () {
        var annotatedTicks = [150, 150];
        var scale = new Plottable.Scales.Linear();
        scale.domain([100, 200]);
        var axis = new Plottable.Axis(scale, "bottom");
        axis.annotationsEnabled(true);
        var div = TestMethods.generateDiv(300, 300);
        axis.renderTo(div);
        axis.annotatedTicks(annotatedTicks);
        var annotationRects = axis.content().selectAll("." + Plottable.Axis.ANNOTATION_RECT_CLASS);
        chai_1.assert.strictEqual(annotationRects.size(), 1, "only one annotation rendered");
        var annotationRect = d3.select(annotationRects.nodes()[0]);
        chai_1.assert.closeTo(numAttr(annotationRect, "y") + numAttr(annotationRect, "height"), axis.height() - axis.margin(), window.Pixel_CloseTo_Requirement, "rectangle positioned correctly");
        axis.destroy();
        div.remove();
    });
});
