"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
var domUtils_1 = require("../../src/utils/domUtils");
describe("Category Axes", function () {
    describe("rendering the tick labels", function () {
        it("handles newlines", function () {
            var div = TestMethods.generateDiv();
            var domain = ["Johannes\nGensfleisch\nGutenberg"];
            var scale = new Plottable.Scales.Category().domain(domain);
            var axis = new Plottable.Axes.Category(scale, "bottom");
            axis.renderTo(div);
            var ticks = axis.content().selectAll("text");
            var texts = ticks.nodes().map(function (tick) { return d3.select(tick).text(); });
            chai_1.assert.deepEqual(texts, domain[0].split("\n"), "newlines are supported in domains");
            div.remove();
        });
        it("renders short words fully", function () {
            var div = TestMethods.generateDiv();
            var domain = ["2000", "2001", "2002", "2003"];
            var scale = new Plottable.Scales.Category().domain(domain);
            var axis = new Plottable.Axes.Category(scale, "bottom");
            axis.renderTo(div);
            var ticks = axis.content().selectAll("text");
            var texts = ticks.nodes().map(function (tick) { return d3.select(tick).text(); });
            chai_1.assert.deepEqual(texts, domain, "text displayed correctly when horizontal");
            axis.tickLabelAngle(90);
            ticks = axis.content().selectAll("text");
            texts = ticks.nodes().map(function (d) { return d3.select(d).text(); });
            chai_1.assert.deepEqual(texts, domain, "text displayed correctly when horizontal");
            chai_1.assert.closeTo(domUtils_1.getRotate(axis.content().selectAll(".text-area")), 90, window.Pixel_CloseTo_Requirement, "the ticks were rotated right");
            axis.tickLabelAngle(0);
            ticks = axis.content().selectAll("text");
            texts = ticks.nodes().map(function (d) { return d3.select(d).text(); });
            chai_1.assert.deepEqual(texts, domain, "text displayed correctly when horizontal");
            chai_1.assert.closeTo(domUtils_1.getRotate(axis.content().selectAll(".text-area")), 0, window.Pixel_CloseTo_Requirement, "the ticks were rotated right");
            axis.tickLabelAngle(-90);
            ticks = axis.content().selectAll("text");
            texts = ticks.nodes().map(function (d) { return d3.select(d).text(); });
            chai_1.assert.deepEqual(texts, domain, "text displayed correctly when horizontal");
            chai_1.assert.closeTo(domUtils_1.getRotate(axis.content().selectAll(".text-area")), -90, window.Pixel_CloseTo_Requirement, "the ticks were rotated left");
            div.remove();
        });
        it("truncates longer labels when tickLabelMaxWidth is set", function () {
            var div = TestMethods.generateDiv();
            var domain = ["albatross long long long long long long long long long long long long title", "short"];
            var scale = new Plottable.Scales.Category().domain(domain);
            var axis = new Plottable.Axes.Category(scale, "left");
            var TICK_LABEL_MAX_WIDTH = 60;
            axis.tickLabelMaxWidth(TICK_LABEL_MAX_WIDTH);
            axis.renderTo(div);
            var tickLabelContainer = axis.content().select(".tick-label-container").node();
            // add 8px padding to account for https://github.com/palantir/svg-typewriter/issues/40
            chai_1.assert.isBelow(tickLabelContainer.getBBox().width, TICK_LABEL_MAX_WIDTH + 8, "tick width was capped");
            div.remove();
        });
        it("has a maximum number of lines when ticklabelMaxLines is set", function () {
            var div = TestMethods.generateDiv();
            var domain = ["albatross long long long long long long long long long long long long title", "short"];
            var scale = new Plottable.Scales.Category().domain(domain);
            var axis = new Plottable.Axes.Category(scale, "left");
            axis.tickLabelMaxWidth(60);
            axis.tickLabelMaxLines(2);
            axis.renderTo(div);
            var tickLabels = axis.content().selectAll(".tick-label");
            chai_1.assert.strictEqual(tickLabels.size(), 2, "only renders two labels");
            var _a = tickLabels.nodes(), longLabel = _a[0], shortLabel = _a[1];
            chai_1.assert.strictEqual(d3.select(longLabel).selectAll("text").size(), 2, "first label is only two lines long");
            chai_1.assert.strictEqual(d3.select(shortLabel).selectAll("text").size(), 1, "second label is only one line long");
            div.remove();
        });
        it("downsamples the domain if there are too many ticks to fit", function () {
            var height = 30;
            var div = TestMethods.generateDiv(400, height);
            var domain = ["one", "two", "three", "four", "five", "six", "seven"];
            var scale = new Plottable.Scales.Category().domain(domain);
            var axis = new Plottable.Axes.Category(scale, "left");
            axis.renderTo(div);
            var _a = axis.getDownsampleInfo(), downsampledDomain = _a.domain, stepWidth = _a.stepWidth;
            chai_1.assert.strictEqual(stepWidth, 4 * scale.stepWidth(), "computes new stepWidth correctly");
            chai_1.assert.deepEqual(downsampledDomain, ["one", "five"], "downsamples domain correctly");
            var tickLabels = axis.content().selectAll(".tick-label");
            chai_1.assert.strictEqual(tickLabels.size(), 2, "renders downsampled labels");
            div.remove();
        });
        it("re-renders with the new domain when the category scale's domain changes", function () {
            var div = TestMethods.generateDiv();
            var domain = ["foo", "bar", "baz"];
            var scale = new Plottable.Scales.Category().domain(domain);
            var axis = new Plottable.Axes.Category(scale, "left");
            axis.renderTo(div);
            var tickLabels = axis.content().selectAll(".tick-label");
            chai_1.assert.strictEqual(tickLabels.size(), domain.length, "same number of tick labels as domain entries");
            tickLabels.each(function (d, i) {
                var tickLabel = d3.select(this);
                chai_1.assert.strictEqual(tickLabel.text(), domain[i], "tick labels render domain");
            });
            var changedDomain = ["bar", "baz", "bam"];
            scale.domain(changedDomain);
            tickLabels = axis.content().selectAll(".tick-label");
            chai_1.assert.strictEqual(tickLabels.size(), changedDomain.length, "same number of tick labels as changed domain entries");
            tickLabels.each(function (d, i) {
                var tickLabel = d3.select(this);
                chai_1.assert.strictEqual(tickLabel.text(), changedDomain[i], "tick label " + i + " renders after changing domain");
            });
            div.remove();
        });
        it("does not overlap labels with tick marks", function () {
            function verifyTickLabelOverlaps(tickLabels, tickMarks) {
                for (var i = 0; i < tickLabels.nodes().length; i++) {
                    var tickLabelRect = tickLabels.nodes()[i].getBoundingClientRect();
                    var tickMarkRect = tickMarks.nodes()[i].getBoundingClientRect();
                    chai_1.assert.isFalse(Plottable.Utils.DOM.clientRectsOverlap(tickLabelRect, tickMarkRect), "tick label and rect do not overlap");
                }
            }
            var div = TestMethods.generateDiv();
            var scale = new Plottable.Scales.Category();
            var axis = new Plottable.Axes.Category(scale, "left");
            scale.domain(["A", "B", "C"]);
            axis.renderTo(div);
            var tickLabels = axis.content().selectAll(".tick-label");
            var tickMarks = axis.content().selectAll(".tick-mark");
            verifyTickLabelOverlaps(tickLabels, tickMarks);
            axis.orientation("right");
            verifyTickLabelOverlaps(tickLabels, tickMarks);
            div.remove();
        });
        it("renders the domain from top to bottom on a vertical axis", function () {
            var div = TestMethods.generateDiv();
            var domain = ["label1", "label2", "label100"];
            var scale = new Plottable.Scales.Category().domain(domain);
            var axis = new Plottable.Axes.Category(scale, "left");
            axis.renderTo(div);
            var tickLabels = axis.content().selectAll(".tick-label");
            chai_1.assert.deepEqual(tickLabels.data(), domain, "tick label per datum in given order");
            var getYTransform = function (selection) {
                return domUtils_1.getTranslateValues(selection)[1];
            };
            tickLabels.each(function (d, i) {
                if (i === tickLabels.size() - 1) {
                    return;
                }
                var tickLabel = d3.select(this);
                var nextTickLabel = d3.select(tickLabels.nodes()[i + 1]);
                chai_1.assert.operator(getYTransform(tickLabel), "<", getYTransform(nextTickLabel), "labels render from top to bottom");
            });
            axis.destroy();
            div.remove();
        });
        it("renders the domain from left to right on a horizontal axis", function () {
            var div = TestMethods.generateDiv();
            var domain = ["label1", "label2", "label100"];
            var scale = new Plottable.Scales.Category().domain(domain);
            var axis = new Plottable.Axes.Category(scale, "bottom");
            axis.renderTo(div);
            var tickLabels = axis.content().selectAll(".tick-label");
            chai_1.assert.deepEqual(tickLabels.data(), domain, "tick label per datum in given order");
            var getXTransform = function (selection) {
                return domUtils_1.getTranslateValues(selection)[0];
            };
            tickLabels.each(function (d, i) {
                if (i === tickLabels.size() - 1) {
                    return;
                }
                var tickLabel = d3.select(this);
                var nextTickLabel = d3.select(tickLabels.nodes()[i + 1]);
                chai_1.assert.operator(getXTransform(tickLabel), "<", getXTransform(nextTickLabel), "labels render from left to right");
            });
            axis.destroy();
            div.remove();
        });
    });
    describe("requesting space when bottom oriented", function () {
        var div;
        var axis;
        var scale;
        beforeEach(function () {
            div = TestMethods.generateDiv();
            scale = new Plottable.Scales.Category();
            axis = new Plottable.Axes.Category(scale, "bottom");
        });
        afterEach(function () {
            axis.destroy();
        });
        it("requests no space when the scale has no domain", function () {
            axis.anchor(div);
            var space = axis.requestedSpace(Plottable.Utils.DOM.elementWidth(div), Plottable.Utils.DOM.elementHeight(div));
            chai_1.assert.strictEqual(space.minWidth, 0, "requested no width");
            chai_1.assert.strictEqual(space.minHeight, 0, "requested no height");
            div.remove();
        });
        it("requests more space if not enough space to fit the text", function () {
            var domain = ["2000", "2001", "2002", "2003"];
            scale.domain(domain);
            axis.renderTo(div);
            var smallDimension = 10;
            var spaceRequest = axis.requestedSpace(300, smallDimension);
            chai_1.assert.operator(spaceRequest.minHeight, ">", smallDimension, "horizontal axis requested more height if constrained");
            axis.orientation("left");
            spaceRequest = axis.requestedSpace(smallDimension, 300);
            chai_1.assert.operator(spaceRequest.minWidth, ">", smallDimension, "vertical axis requested more width if constrained");
            div.remove();
        });
        it("requests more space for rotated text", function () {
            var domain = ["label1", "label2", "label100"];
            scale.domain(domain);
            axis.renderTo(div);
            var requestedSpace = axis.requestedSpace(Plottable.Utils.DOM.elementWidth(div), 50);
            var flatHeight = requestedSpace.minHeight;
            axis.tickLabelAngle(-90);
            requestedSpace = axis.requestedSpace(Plottable.Utils.DOM.elementWidth(div), 50);
            chai_1.assert.operator(flatHeight, "<", requestedSpace.minHeight, "axis should request more height when tick labels are rotated");
            div.remove();
        });
        it("accounts for margin, innerTickLength, and padding when calculating for height", function () {
            scale.domain(["foo", "bar", "baz"]);
            axis.anchor(div);
            var divWidth = Plottable.Utils.DOM.elementWidth(div);
            var divHeight = Plottable.Utils.DOM.elementHeight(div);
            var axisRequestedHeight = function () { return axis.requestedSpace(divWidth, divHeight).minHeight; };
            var oldHeight = axisRequestedHeight();
            var increaseAmount = 5;
            axis.tickLabelPadding(axis.tickLabelPadding() + increaseAmount);
            chai_1.assert.strictEqual(axisRequestedHeight(), oldHeight + increaseAmount, "increasing tickLabelPadding increases height");
            oldHeight = axisRequestedHeight();
            axis.margin(axis.margin() + increaseAmount);
            chai_1.assert.strictEqual(axisRequestedHeight(), oldHeight + increaseAmount, "increasing margin increases height");
            oldHeight = axisRequestedHeight();
            axis.innerTickLength(axis.innerTickLength() + increaseAmount);
            chai_1.assert.strictEqual(axisRequestedHeight(), oldHeight + increaseAmount, "increasing innerTickLength increases height");
            axis.destroy();
            div.remove();
        });
    });
    describe("requesting space on left oriented axes", function () {
        it("requests space only for the ticks it'll actually show", function () {
            var div = TestMethods.generateDiv(400, 400);
            var scale = new Plottable.Scales.Category().domain(["a", "b", "c", "long long long long long long long long long long"]);
            var axis = new Plottable.Axes.Category(scale, "left");
            axis.anchor(div);
            var originalRequestedWidth = axis.requestedSpace(400, 400).minWidth;
            // zoom towards the start of the domain such that the long entry shouldn't be shown
            scale.zoom(0.5, 0);
            var newRequestedWidth = axis.requestedSpace(400, 400).minWidth;
            chai_1.assert.isBelow(newRequestedWidth, originalRequestedWidth, "new wanted width is less than old");
            axis.destroy();
            div.remove();
        });
        it("accounts for margin, innerTickLength, and padding when calculating for width", function () {
            var div = TestMethods.generateDiv();
            var scale = new Plottable.Scales.Category().domain(["foo", "bar", "baz"]);
            var axis = new Plottable.Axes.Category(scale, "left");
            axis.anchor(div);
            var divWidth = Plottable.Utils.DOM.elementWidth(div);
            var divHeight = Plottable.Utils.DOM.elementHeight(div);
            var axisRequestedWidth = function () { return axis.requestedSpace(divWidth, divHeight).minWidth; };
            var oldWidth = axisRequestedWidth();
            var increaseAmount = 5;
            axis.tickLabelPadding(axis.tickLabelPadding() + increaseAmount);
            chai_1.assert.strictEqual(axisRequestedWidth(), oldWidth + increaseAmount, "increasing tickLabelPadding increases width");
            oldWidth = axisRequestedWidth();
            axis.margin(axis.margin() + increaseAmount);
            chai_1.assert.strictEqual(axisRequestedWidth(), oldWidth + increaseAmount, "increasing margin increases width");
            oldWidth = axisRequestedWidth();
            axis.innerTickLength(axis.innerTickLength() + increaseAmount);
            chai_1.assert.strictEqual(axisRequestedWidth(), oldWidth + increaseAmount, "increasing innerTickLength increases width");
            axis.destroy();
            div.remove();
        });
    });
    describe("coercing", function () {
        it("does not blow up for non-string data", function () {
            var div = TestMethods.generateDiv();
            var domain = [null, undefined, true, 2, "foo"];
            var scale = new Plottable.Scales.Category().domain(domain);
            var axis = new Plottable.Axes.Category(scale, "bottom");
            axis.renderTo(div);
            var texts = div.selectAll("text").nodes().map(function (s) { return d3.select(s).text(); });
            chai_1.assert.deepEqual(texts, domain.map(function (d) { return String(d); }));
            axis.destroy();
            div.remove();
        });
    });
    describe("formatting the text", function () {
        it("uses the formatter if supplied", function () {
            var div = TestMethods.generateDiv();
            var domain = ["Air", "Bi", "Sea"];
            var scale = new Plottable.Scales.Category().domain(domain);
            var axis = new Plottable.Axes.Category(scale, "bottom");
            var addPlane = function (l) { return l + "plane"; };
            axis.formatter(addPlane);
            axis.renderTo(div);
            var expectedTexts = domain.map(addPlane);
            axis.content().selectAll("text").each(function (d, i) {
                var actualText = d3.select(this).text();
                chai_1.assert.strictEqual(actualText, expectedTexts[i], "formatter was applied");
            });
            axis.destroy();
            div.remove();
        });
    });
    describe("setting the tick lengths", function () {
        it("draws inner ticks with the specified length", function () {
            var div = TestMethods.generateDiv();
            var scale = new Plottable.Scales.Category().domain(["foo", "bar", "baz", "blue", "red"]);
            var axis = new Plottable.Axes.Category(scale, "bottom");
            var innerTickLength = 20;
            axis.innerTickLength(innerTickLength);
            axis.renderTo(div);
            var innerTickMarks = axis.content().selectAll("." + Plottable.Axis.TICK_MARK_CLASS + ":not(." + Plottable.Axis.END_TICK_MARK_CLASS + ")");
            chai_1.assert.strictEqual(innerTickMarks.size(), scale.domain().length - 2, "same number of inner ticks as domain entries minus 2");
            innerTickMarks.each(function (d, i) {
                var innerTickMark = d3.select(this);
                var innerTickMarkLength = Math.abs(TestMethods.numAttr(innerTickMark, "y1") - TestMethods.numAttr(innerTickMark, "y2"));
                chai_1.assert.closeTo(innerTickMarkLength, innerTickLength, window.Pixel_CloseTo_Requirement, "tick mark " + i + " of specified length");
            });
            axis.destroy();
            div.remove();
        });
        it("draws end ticks with the specified length", function () {
            var div = TestMethods.generateDiv();
            var scale = new Plottable.Scales.Category().domain(["foo", "bar", "baz", "blue", "red"]);
            var axis = new Plottable.Axes.Category(scale, "bottom");
            var endTickLength = 20;
            axis.endTickLength(endTickLength);
            axis.renderTo(div);
            var endTickMarks = axis.content().selectAll("." + Plottable.Axis.END_TICK_MARK_CLASS);
            chai_1.assert.strictEqual(endTickMarks.size(), 2, "2 end ticks");
            endTickMarks.each(function (d, i) {
                var endTickMark = d3.select(this);
                var endTickMarkLength = Math.abs(TestMethods.numAttr(endTickMark, "y1") - TestMethods.numAttr(endTickMark, "y2"));
                chai_1.assert.closeTo(endTickMarkLength, endTickLength, window.Pixel_CloseTo_Requirement, "tick mark " + i + " of specified length");
            });
            axis.destroy();
            div.remove();
        });
    });
});
