"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("TimeAxis", function () {
    var orientations = ["top", "bottom"];
    describe("setting the orientation", function () {
        it("throws an error when setting a vertical orientation", function () {
            var scale = new Plottable.Scales.Time();
            chai_1.assert.throws(function () { return new Plottable.Axes.Time(scale, "left"); }, "horizontal");
            chai_1.assert.throws(function () { return new Plottable.Axes.Time(scale, "right"); }, "horizontal");
        });
        it("cannot change to a vertical orientation", function () {
            var scale = new Plottable.Scales.Time();
            var originalOrientation = "bottom";
            var axis = new Plottable.Axes.Time(scale, originalOrientation);
            chai_1.assert.throws(function () { return axis.orientation("left"); }, "horizontal");
            chai_1.assert.throws(function () { return axis.orientation("right"); }, "horizontal");
            chai_1.assert.strictEqual(axis.orientation(), originalOrientation, "orientation unchanged");
            axis.destroy();
        });
    });
    describe("rendering in edge case scenarios", function () {
        it("does not error when setting the domain to a large span", function () {
            var scale = new Plottable.Scales.Time();
            var axis = new Plottable.Axes.Time(scale, "bottom");
            var div = TestMethods.generateDiv();
            axis.renderTo(div);
            // very large time span
            chai_1.assert.doesNotThrow(function () { return scale.domain([new Date(0, 0, 1, 0, 0, 0, 0), new Date(5000, 0, 1, 0, 0, 0, 0)]); });
            axis.destroy();
            div.remove();
        });
        it("does not error when setting the domain to a small span", function () {
            var scale = new Plottable.Scales.Time();
            var axis = new Plottable.Axes.Time(scale, "bottom");
            var div = TestMethods.generateDiv();
            axis.renderTo(div);
            // very small time span
            chai_1.assert.doesNotThrow(function () { return scale.domain([new Date(0, 0, 1, 0, 0, 0, 0), new Date(0, 0, 1, 0, 0, 0, 100)]); });
            axis.destroy();
            div.remove();
        });
    });
    function assertVisibleLabelsDoNotOverlap(axis) {
        axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS + "-container").each(function (d, i) {
            var container = d3.select(this);
            var visibleTickLabels = container
                .selectAll("." + Plottable.Axis.TICK_LABEL_CLASS)
                .filter(function () {
                return d3.select(this).style("visibility") === "visible";
            });
            visibleTickLabels.each(function (d2, j) {
                var clientRect1 = this.getBoundingClientRect();
                visibleTickLabels.filter(function (d3, k) { return k > j; }).each(function (d3, k) {
                    var clientRect2 = this.getBoundingClientRect();
                    chai_1.assert.isFalse(Plottable.Utils.DOM.clientRectsOverlap(clientRect1, clientRect2), "tick labels don't overlap");
                });
            });
        });
    }
    orientations.forEach(function (orientation) {
        var domains = [
            // 100 year span
            [new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2100, 0, 1, 0, 0, 0, 0)],
            // 1 year span
            [new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2000, 11, 31, 0, 0, 0, 0)],
            // 1 month span
            [new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2000, 1, 1, 0, 0, 0, 0)],
            // 1 day span
            [new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2000, 0, 1, 23, 0, 0, 0)],
            // 1 hour span
            [new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2000, 0, 1, 1, 0, 0, 0)],
            // 1 minute span
            [new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2000, 0, 1, 0, 1, 0, 0)],
            // 1 second span
            [new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2000, 0, 1, 0, 0, 1, 0)],
        ];
        it("does not overlap visible tick labels with orientation " + orientation, function () {
            var div = TestMethods.generateDiv();
            var scale = new Plottable.Scales.Time();
            var axis = new Plottable.Axes.Time(scale, orientation);
            axis.renderTo(div);
            domains.forEach(function (domain) {
                scale.domain(domain);
                assertVisibleLabelsDoNotOverlap(axis);
            });
            axis.destroy();
            div.remove();
        });
    });
    function assertTickMarksAndLabelsDoNotOverlap(axis) {
        var tickMarks = axis.content().selectAll("." + Plottable.Axis.TICK_MARK_CLASS + ":not(." + Plottable.Axis.END_TICK_MARK_CLASS + ")");
        chai_1.assert.operator(tickMarks.size(), ">=", 1, "There is at least one tick mark in the test");
        var tickLabels = axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS).filter(function (d, i) {
            return d3.select(this).style("visibility") !== "hidden";
        });
        chai_1.assert.operator(tickLabels.size(), ">=", 1, "There is at least one tick label in the test");
        tickMarks.each(function (tickMark, i) {
            var tickMarkRect = this.getBoundingClientRect();
            tickLabels.each(function (tickLabel, j) {
                var tickLabelRect = this.getBoundingClientRect();
                var isOverlap = Plottable.Utils.DOM.clientRectsOverlap(tickMarkRect, tickLabelRect);
                chai_1.assert.isFalse(isOverlap, "Tick mark " + i + " should not overlap with tick label " + j);
            });
        });
    }
    var tierLabelPositions = ["center", "between"];
    orientations.forEach(function (orientation) {
        tierLabelPositions.forEach(function (tierLabelPosition) {
            it("does not overlap labels with tick marks when label position is " + tierLabelPosition + " and orientation " + orientation, function () {
                var scale = new Plottable.Scales.Time();
                var axis = new Plottable.Axes.Time(scale, orientation);
                var div = TestMethods.generateDiv();
                axis.tierLabelPositions([tierLabelPosition, tierLabelPosition]);
                axis.renderTo(div);
                assertTickMarksAndLabelsDoNotOverlap(axis);
                div.remove();
            });
        });
    });
    describe("drawing tick marks", function () {
        var axis;
        var div;
        beforeEach(function () {
            var scale = new Plottable.Scales.Time();
            axis = new Plottable.Axes.Time(scale, "bottom");
            div = TestMethods.generateDiv();
        });
        afterEach(function () {
            axis.destroy();
        });
        it("renders end ticks at the two edges", function () {
            axis.renderTo(div);
            var firstTick = axis.content().select("." + Plottable.Axis.TICK_MARK_CLASS);
            chai_1.assert.strictEqual(firstTick.attr("x1"), "0", "first tick mark at beginning of axis");
            chai_1.assert.strictEqual(firstTick.attr("x2"), "0", "first tick mark at beginning of axis");
            var lastTick = axis.content().select("." + Plottable.Axis.TICK_MARK_CLASS + ":last-child");
            chai_1.assert.strictEqual(lastTick.attr("x1") + "px", div.style("width"), "last end tick mark at end of axis");
            chai_1.assert.strictEqual(lastTick.attr("x2") + "px", div.style("width"), "last end tick mark at end of axis");
            div.remove();
        });
        it("adds the end-tick class to the first and last ticks", function () {
            axis.renderTo(div);
            var firstTick = axis.content().select("." + Plottable.Axis.TICK_MARK_CLASS);
            chai_1.assert.isTrue(firstTick.classed(Plottable.Axis.END_TICK_MARK_CLASS), "first end tick has end-tick-mark class");
            var lastTick = axis.content().select("." + Plottable.Axis.TICK_MARK_CLASS + ":last-child");
            chai_1.assert.isTrue(lastTick.classed(Plottable.Axis.END_TICK_MARK_CLASS), "last end tick has end-tick-mark class");
            div.remove();
        });
        it("sets the length of the end ticks to the specified value when tierLabelPosition is set to center", function () {
            axis.tierLabelPositions(["center", "center"]);
            axis.renderTo(div);
            var endTicks = axis.content().selectAll("." + Plottable.Axis.END_TICK_MARK_CLASS);
            chai_1.assert.operator(endTicks.size(), ">=", 1, "At least one end tick mark is selected in the test");
            endTicks.each(function (d, i) {
                var endTick = d3.select(this);
                var tickLength = Math.abs(TestMethods.numAttr(endTick, "y1") - TestMethods.numAttr(endTick, "y2"));
                chai_1.assert.closeTo(tickLength, axis.endTickLength(), window.Pixel_CloseTo_Requirement, "end tick mark " + i + " length should equal the specified amount");
            });
            div.remove();
        });
    });
    describe("formatting annotation ticks", function () {
        it("formats the dates to '{{abbreviated weekday}} {{abbreviated month}} {{day of month}}, {{year}}' by default", function () {
            var div = TestMethods.generateDiv();
            var scale = new Plottable.Scales.Time();
            var axis = new Plottable.Axes.Time(scale, "bottom");
            axis.annotationsEnabled(true);
            var testDate = new Date((scale.domain()[0].valueOf() + scale.domain()[1].valueOf()) / 2);
            axis.annotatedTicks([testDate]);
            var defaultFormatter = Plottable.Formatters.time("%a %b %d, %Y");
            var annotationFormatter = axis.annotationFormatter();
            chai_1.assert.strictEqual(annotationFormatter(testDate), defaultFormatter(testDate), "formats to a default customized time formatter");
            axis.renderTo(div);
            var annotationLabels = axis.content().selectAll(".annotation-label");
            annotationLabels.each(function (d, i) {
                var annotationLabel = d3.select(this);
                chai_1.assert.strictEqual(annotationLabel.text(), defaultFormatter(d), "formats to a default customized time formatter");
            });
            div.remove();
        });
    });
    describe("calculating space", function () {
        var axis;
        var div;
        beforeEach(function () {
            var scale = new Plottable.Scales.Time();
            axis = new Plottable.Axes.Time(scale, "bottom");
            div = TestMethods.generateDiv();
        });
        afterEach(function () {
            axis.destroy();
        });
        it("grows in height for each added tier", function () {
            axis.axisConfigurations([
                [
                    { interval: Plottable.TimeInterval.day, step: 2, formatter: Plottable.Formatters.time("%a %e") },
                ],
            ]);
            axis.renderTo(div);
            var oneTierHeight = axis.height();
            axis.axisConfigurations([
                [
                    { interval: Plottable.TimeInterval.day, step: 2, formatter: Plottable.Formatters.time("%a %e") },
                    { interval: Plottable.TimeInterval.day, step: 2, formatter: Plottable.Formatters.time("%a %e") },
                ],
            ]);
            var twoTierHeight = axis.height();
            chai_1.assert.operator(twoTierHeight, ">", oneTierHeight, "two-tiered a taller than one-tiered");
            axis.axisConfigurations([
                [
                    { interval: Plottable.TimeInterval.day, step: 2, formatter: Plottable.Formatters.time("%a %e") },
                    { interval: Plottable.TimeInterval.day, step: 2, formatter: Plottable.Formatters.time("%a %e") },
                    { interval: Plottable.TimeInterval.day, step: 2, formatter: Plottable.Formatters.time("%a %e") },
                ],
            ]);
            var threeTierHeight = axis.height();
            chai_1.assert.operator(threeTierHeight, ">", twoTierHeight, "three-tiered is taller than the two-tiered");
            div.remove();
        });
        it("shrinks in height if a newly set configuration provides less tiers", function () {
            axis.axisConfigurations([
                [
                    { interval: Plottable.TimeInterval.day, step: 2, formatter: Plottable.Formatters.time("%a %e") },
                    { interval: Plottable.TimeInterval.day, step: 2, formatter: Plottable.Formatters.time("%a %e") },
                ],
            ]);
            axis.renderTo(div);
            var twoTierHeight = axis.height();
            axis.axisConfigurations([
                [
                    { interval: Plottable.TimeInterval.day, step: 2, formatter: Plottable.Formatters.time("%a %e") },
                ],
            ]);
            var newHeight = axis.height();
            chai_1.assert.operator(newHeight, "<", twoTierHeight, "two-tier time axis shrinks when changed to 1 tier");
            div.remove();
        });
        it("includes the margin, label padding, and tick length in the height", function () {
            axis.margin(100);
            axis.anchor(div);
            axis.computeLayout({ x: 0, y: 0 }, 400, 400);
            var minimumHeight = axis.tickLabelPadding() + axis.margin() + axis.innerTickLength();
            chai_1.assert.operator(axis.height(), ">=", minimumHeight, "height includes all relevant pieces");
            div.remove();
        });
        it("includes the annotation space in the height", function () {
            axis.annotationsEnabled(true);
            axis.annotationTierCount(2);
            axis.anchor(div);
            axis.computeLayout({ x: 0, y: 0 }, Plottable.Utils.DOM.elementWidth(div), Plottable.Utils.DOM.elementHeight(div));
            var twoAnnotationTierHeight = axis.height();
            axis.annotationTierCount(3);
            axis.computeLayout({ x: 0, y: 0 }, Plottable.Utils.DOM.elementWidth(div), Plottable.Utils.DOM.elementHeight(div));
            chai_1.assert.operator(axis.height(), ">", twoAnnotationTierHeight, "height includes all relevant pieces");
            axis.destroy();
            div.remove();
        });
    });
    describe("drawing tiers", function () {
        it("draws all of tiers within the component space", function () {
            var div = TestMethods.generateDiv();
            var constrainedHeight = 50;
            div.style("height", constrainedHeight + "px");
            var scale = new Plottable.Scales.Time();
            var axis = new Plottable.Axes.Time(scale, "bottom");
            var tiersToCreate = 15;
            var configuration = Array.apply(null, Array(tiersToCreate)).map(function () {
                return { interval: Plottable.TimeInterval.day, step: 2, formatter: Plottable.Formatters.time("%a %e") };
            });
            axis.axisConfigurations([configuration]);
            axis.renderTo(div);
            var axisBoundingRect = axis.element().node().getBoundingClientRect();
            var isInsideAxisBoundingRect = function (innerRect) {
                return Math.floor(innerRect.bottom) <= Math.ceil(axisBoundingRect.bottom) + window.Pixel_CloseTo_Requirement &&
                    Math.floor(axisBoundingRect.top) <= Math.ceil(innerRect.top) + window.Pixel_CloseTo_Requirement;
            };
            axis.content()
                .selectAll("." + Plottable.Axes.Time.TIME_AXIS_TIER_CLASS)
                .each(function (d, i) {
                var tier = d3.select(this);
                var visibility = tier.style("visibility");
                // HACKHACK window.getComputedStyle() is behaving weirdly in IE9. Further investigation required
                if (visibility === "inherit") {
                    visibility = getStyleInIE9(tier.node());
                }
                if (isInsideAxisBoundingRect(tier.node().getBoundingClientRect())) {
                    chai_1.assert.strictEqual(visibility, "visible", "tier " + i + " inside axis should be visible");
                }
                else {
                    chai_1.assert.strictEqual(visibility, "hidden", "tier " + i + " outside axis should not be visible");
                }
            });
            div.remove();
            axis.destroy();
            function getStyleInIE9(element) {
                while (element) {
                    var visibility = window.getComputedStyle(element).visibility;
                    if (visibility !== "inherit") {
                        return visibility;
                    }
                    element = element.parentNode;
                }
                return "visible";
            }
        });
    });
    describe("configuring the label presentation", function () {
        it("allows usage of a custom configuration list", function () {
            var div = TestMethods.generateDiv();
            var scale = new Plottable.Scales.Time();
            var axis = new Plottable.Axes.Time(scale, "bottom");
            var formatter = Plottable.Formatters.time("%a %e");
            axis.axisConfigurations([
                [
                    { interval: Plottable.TimeInterval.day, step: 2, formatter: formatter },
                ],
            ]);
            axis.renderTo(div);
            var tickLabels = axis.content().selectAll(".tick-label");
            tickLabels.each(function (d, i) {
                var tickLabel = d3.select(this);
                chai_1.assert.strictEqual(tickLabel.text(), formatter(d), "formats to a customized time formatter");
            });
            axis.destroy();
            div.remove();
        });
    });
    describe("limiting timeinterval precision", function () {
        it("uses default axis config when maxTimeIntervalPrecision not set", function () {
            var div = TestMethods.generateDiv();
            var axis = new Plottable.Axes.Time(new Plottable.Scales.Time(), "bottom");
            axis.renderTo(div);
            var config = axis.currentAxisConfiguration();
            chai_1.assert.strictEqual(config.length, 2, "2 tiers");
            chai_1.assert.strictEqual(config[0].interval, "hour");
            chai_1.assert.strictEqual(config[1].interval, "day");
            axis.destroy();
            div.remove();
        });
        it("still works when maxTimeIntervalPrecision is set high", function () {
            var div = TestMethods.generateDiv();
            var axis = new Plottable.Axes.Time(new Plottable.Scales.Time(), "bottom");
            axis.maxTimeIntervalPrecision("minute");
            axis.renderTo(div);
            var config = axis.currentAxisConfiguration();
            chai_1.assert.strictEqual(config.length, 2, "2 tiers");
            chai_1.assert.strictEqual(config[0].interval, "hour");
            chai_1.assert.strictEqual(config[1].interval, "day");
            axis.destroy();
            div.remove();
        });
        it("limits axis config when maxTimeIntervalPrecision is set", function () {
            var div = TestMethods.generateDiv();
            var axis = new Plottable.Axes.Time(new Plottable.Scales.Time(), "bottom");
            axis.maxTimeIntervalPrecision("day");
            axis.renderTo(div);
            var config = axis.currentAxisConfiguration();
            // day/month is most precise valid config after hour/day
            chai_1.assert.strictEqual(config.length, 2, "2 tiers");
            chai_1.assert.strictEqual(config[0].interval, "day");
            chai_1.assert.strictEqual(config[1].interval, "month");
            axis.destroy();
            div.remove();
        });
    });
});
