"use strict";
var _this = this;
var d3 = require("d3");
var chai_1 = require("chai");
var sinon = require("sinon");
var Plottable = require("../../src");
var Mocks = require("../mocks");
var TestMethods = require("../testMethods");
describe("Component", function () {
    var DIV_WIDTH = 400;
    var DIV_HEIGHT = 300;
    describe("anchoring", function () {
        var c;
        var div;
        beforeEach(function () {
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        });
        it("adds itself as a child element of the input selection", function () {
            chai_1.assert.strictEqual(c.anchor(div), c, "setter returns calling object");
            chai_1.assert.strictEqual(c.rootElement().node(), div.node(), "component DOM elements are children of div element");
            c.destroy();
            div.remove();
        });
        it("adds default selections in the correct order", function () {
            c.anchor(div);
            chai_1.assert.isFalse(c.foreground().empty(), "foreground exists in the DOM");
            chai_1.assert.isFalse(c.background().empty(), "background exists in the DOM");
            chai_1.assert.isFalse(c.content().empty(), "content exists in the DOM");
            var componentElement = div.select(".component");
            var containerNodes = componentElement.selectAll("svg").nodes();
            chai_1.assert.strictEqual(containerNodes[0], c.background().node(), "background at the back");
            chai_1.assert.strictEqual(containerNodes[1], c.content().node(), "content at the middle");
            chai_1.assert.strictEqual(containerNodes[2], c.foreground().node(), "foreground at the front");
            c.destroy();
            div.remove();
        });
        it("sets the foreground-container pointer-events to none", function () {
            c.anchor(div);
            var foreground = c.foreground().node();
            var pointerEventForeground = window.getComputedStyle(foreground).pointerEvents;
            chai_1.assert.strictEqual(pointerEventForeground, "none", "foreground-container's pointer-event is set to none");
            c.destroy();
            div.remove();
        });
        it("classes the input with 'plottable' if it is a div", function () {
            c.anchor(div);
            chai_1.assert.isTrue(div.classed("plottable"), "<div> was given \"plottable\" CSS class");
            c.destroy();
            div.remove();
        });
        it("allows mouse events to operate if the component is visible", function () {
            c.anchor(div);
            var computedStyle = window.getComputedStyle(div.node());
            chai_1.assert.strictEqual(computedStyle.pointerEvents.toLowerCase(), "visiblefill", "\"pointer-events\" style set to \"visiblefill\"");
            c.destroy();
            div.remove();
        });
        it("can switch which element it is anchored to", function () {
            c.anchor(div);
            var div2 = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            c.anchor(div2);
            chai_1.assert.notStrictEqual(c.rootElement().node(), div.node(), "component DOM elements are not children of div element");
            chai_1.assert.strictEqual(c.rootElement().node(), div2.node(), "component DOM elements are children of second div element");
            c.destroy();
            div2.remove();
            div.remove();
        });
        it("removes DOM elements in previous div when anchoring to a different div", function () {
            c.anchor(div);
            var div2 = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            c.anchor(div2);
            chai_1.assert.isTrue(div.select("svg").empty(), "previous div element should not have any group child nodes");
            chai_1.assert.isFalse(div2.select("svg").empty(), "new div element should have group child nodes");
            c.destroy();
            div.remove();
            div2.remove();
        });
        it("can undergo set behavior upon anchoring", function () {
            var callbackCalled = false;
            var passedComponent;
            var callback = function (component) {
                callbackCalled = true;
                passedComponent = component;
            };
            chai_1.assert.strictEqual(c.onAnchor(callback), c, "setter returns calling object");
            c.anchor(div);
            chai_1.assert.isTrue(callbackCalled, "callback was called on anchoring");
            chai_1.assert.strictEqual(passedComponent, c, "callback was passed anchored Component");
            c.destroy();
            div.remove();
        });
        it("undergoes on-anchor behavior if already anchored", function () {
            var callbackCalled = false;
            var passedComponent;
            var callback = function (component) {
                callbackCalled = true;
                passedComponent = component;
            };
            c.anchor(div);
            c.onAnchor(callback);
            chai_1.assert.isTrue(callbackCalled, "callback was immediately if Component was already anchored");
            chai_1.assert.strictEqual(passedComponent, c, "callback was passed the Component that anchored");
            c.destroy();
            div.remove();
        });
        it("calls callbacks upon anchoring to different div", function () {
            var callbackCalled = false;
            var passedComponent;
            var callback = function (component) {
                callbackCalled = true;
                passedComponent = component;
            };
            c.onAnchor(callback);
            c.anchor(div);
            var div2 = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            callbackCalled = false;
            c.anchor(div2);
            chai_1.assert.isTrue(callbackCalled, "callback was called on anchoring to a new <div>");
            chai_1.assert.strictEqual(passedComponent, c, "callback was passed anchored Component");
            c.destroy();
            div.remove();
            div2.remove();
        });
        it("can remove set behavior the component would have underwent upon anchoring", function () {
            var callbackCalled = false;
            var callback = function (component) {
                callbackCalled = true;
            };
            c.onAnchor(callback);
            chai_1.assert.strictEqual(c.offAnchor(callback), c, "setter returns calling object");
            c.anchor(div);
            chai_1.assert.isFalse(callbackCalled, "removed callback is not called");
            c.destroy();
            div.remove();
        });
        it("overflow: visible by default", function () {
            c.anchor(div);
            var styles = window.getComputedStyle(c.content().node());
            chai_1.assert.strictEqual(styles.getPropertyValue("overflow"), "visible", "content has overflow: visible");
            c.destroy();
            div.remove();
        });
        it("sets overflow: hidden when _overflowHidden is set", function () {
            c._overflowHidden = true;
            c.anchor(div);
            var styles = window.getComputedStyle(c.content().node());
            chai_1.assert.strictEqual(styles.getPropertyValue("overflow"), "hidden", "content has overflow: hidden");
            c.destroy();
            div.remove();
        });
    });
    describe("detaching", function () {
        var c;
        var div;
        beforeEach(function () {
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        });
        it("can remove its own DOM elements", function () {
            c.renderTo(div);
            chai_1.assert.isTrue(div.node().hasChildNodes(), "the div has children");
            c.detach();
            chai_1.assert.isFalse(div.node().hasChildNodes(), "the div has no children");
            c.destroy();
            div.remove();
        });
        it("does not error if not already anchored", function () {
            chai_1.assert.doesNotThrow(function () { return c.detach(); }, Error, "does not throw error if detach() called before anchor()");
            c.destroy();
            div.remove();
        });
        it("sets its parent to null when detaching", function () {
            var parent = new Plottable.Components.Group([c]);
            c.detach();
            chai_1.assert.isNull(c.parent(), "parent removed upon detaching");
            parent.destroy();
            c.destroy();
            div.remove();
        });
        it("can undergo set behavior upon detaching", function () {
            c.renderTo(div);
            var callbackCalled = false;
            var passedComponent;
            var callback = function (component) {
                callbackCalled = true;
                passedComponent = component;
            };
            chai_1.assert.strictEqual(c.onDetach(callback), c, "setter returns calling object");
            c.detach();
            chai_1.assert.isTrue(callbackCalled, "callback was called when the Component was detached");
            chai_1.assert.strictEqual(passedComponent, c, "callback was passed the Component that detached");
            c.destroy();
            div.remove();
        });
        it("calls callbacks upon detaching even if not anchored", function () {
            var callbackCalled = false;
            var passedComponent;
            var callback = function (component) {
                callbackCalled = true;
                passedComponent = component;
            };
            c.onDetach(callback);
            c.detach();
            chai_1.assert.isTrue(callbackCalled, "callback still called");
            chai_1.assert.strictEqual(passedComponent, c, "callback passed the Component that detached");
            c.destroy();
            div.remove();
        });
        it("can remove callbacks that would have been called upon detaching", function () {
            var callbackCalled = false;
            var callback = function (component) {
                callbackCalled = true;
            };
            c.onDetach(callback);
            chai_1.assert.strictEqual(c.offDetach(callback), c, "setter calls calling object");
            c.renderTo(div);
            c.detach();
            chai_1.assert.isFalse(callbackCalled, "removed callback is not called");
            c.destroy();
            div.remove();
        });
    });
    describe("parent container", function () {
        var c;
        var div;
        beforeEach(function () {
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        });
        it("can set its parent to a container that contains this component", function () {
            var acceptingContainer = {
                has: function () { return true; },
            };
            chai_1.assert.strictEqual(c.parent(acceptingContainer), c, "setter returns calling object");
            chai_1.assert.strictEqual(c.parent(), acceptingContainer, "parent set if parent contains component");
            c.destroy();
            div.remove();
        });
        it("throws an error when the input parent does not contain this component", function () {
            var rejectingContainer = {
                has: function (component) { return false; },
            };
            // HACKHACK: https://github.com/palantir/plottable/issues/2661 Cannot assert errors being thrown with description
            chai_1.assert.throws(function () { return c.parent(rejectingContainer); }, Error, "invalid parent", "error thrown for parent not containing child");
            c.destroy();
            div.remove();
        });
    });
    describe("css classes", function () {
        var c;
        var div;
        beforeEach(function () {
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        });
        it("can add css classes", function () {
            var className = "foo";
            chai_1.assert.strictEqual(c.addClass(className), c, "setter returns calling object");
            chai_1.assert.isTrue(c.hasClass(className), "component has added css class");
            c.renderTo(div);
            chai_1.assert.isFalse(div.select("." + className).empty(), "css class added to DOM element");
            c.destroy();
            div.remove();
        });
        it("can remove css classes", function () {
            var className = "foo";
            c.addClass(className);
            chai_1.assert.isTrue(c.hasClass(className), "component has added css class");
            c.renderTo(div);
            chai_1.assert.strictEqual(c.removeClass(className), c, "setter returns calling object");
            chai_1.assert.isFalse(c.hasClass(className), "component no longer has css class");
            chai_1.assert.isTrue(div.select("." + className).empty(), "css class removed from DOM element");
            c.destroy();
            div.remove();
        });
    });
    describe("computing the layout", function () {
        var c;
        var div;
        beforeEach(function () {
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        });
        it("defaults to take up all space", function () {
            chai_1.assert.isFalse(c.fixedWidth(), "will take all available width");
            chai_1.assert.isFalse(c.fixedHeight(), "will take all available height");
            c.anchor(div);
            c.computeLayout();
            chai_1.assert.strictEqual(c.width(), DIV_WIDTH, "takes all available width");
            chai_1.assert.strictEqual(c.height(), DIV_HEIGHT, "takes all available height");
            var origin = c.origin();
            chai_1.assert.strictEqual(origin.x, 0, "x origin defaults to leftmost");
            chai_1.assert.strictEqual(origin.y, 0, "y origin defaults to topmost");
            c.destroy();
            div.remove();
        });
        it("does not allow the origin object to be modified", function () {
            c.renderTo(div);
            var receivedOrigin = c.origin();
            var delta = 10;
            receivedOrigin.x += delta;
            receivedOrigin.y += delta;
            chai_1.assert.notDeepEqual(c.origin(), receivedOrigin, "underlying origin object cannot be modified");
            c.destroy();
            div.remove();
        });
        it("recomputes the layout if the environment changes", function () {
            c.anchor(div);
            c.computeLayout();
            chai_1.assert.strictEqual(c.width(), DIV_WIDTH, "takes all available width");
            chai_1.assert.strictEqual(c.height(), DIV_HEIGHT, "takes all available height");
            div.style("width", 2 * DIV_WIDTH + "px").style("height", 2 * DIV_HEIGHT + "px");
            c.computeLayout();
            chai_1.assert.strictEqual(c.width(), 2 * DIV_WIDTH, "updated to take new available width");
            chai_1.assert.strictEqual(c.height(), 2 * DIV_HEIGHT, "updated to take new available height");
            c.destroy();
            div.remove();
        });
        it("can compute the layout based on CSS", function () {
            var parentWidth = 400;
            var parentHeight = 200;
            // Manually size parent
            var parent = d3.select(div.node().parentNode);
            parent.style("width", parentWidth + "px");
            parent.style("height", parentHeight + "px");
            // Remove width/height on div directly
            div.style("width", null).style("height", null);
            c.anchor(div);
            c.computeLayout();
            chai_1.assert.strictEqual(c.width(), parentWidth, "defaults to width of parent");
            chai_1.assert.strictEqual(c.height(), parentHeight, "defaults to height of parent");
            var origin = c.origin();
            chai_1.assert.strictEqual(origin.x, 0, "xOrigin defaulted to 0");
            chai_1.assert.strictEqual(origin.y, 0, "yOrigin defaulted to 0");
            var divWidthPercentage = 50;
            var divHeightPercentage = 50;
            div.style("width", divWidthPercentage + "%").style("height", divHeightPercentage + "%");
            c.computeLayout();
            chai_1.assert.strictEqual(c.width(), parentWidth * divWidthPercentage / 100, "width computed to be percentage of div width");
            chai_1.assert.strictEqual(c.height(), parentHeight * divHeightPercentage / 100, "height computed to be percentage of div height");
            origin = c.origin();
            chai_1.assert.strictEqual(origin.x, 0, "xOrigin defaulted to 0");
            chai_1.assert.strictEqual(origin.y, 0, "yOrigin defaulted to 0");
            // reset test page DOM
            parent.style("width", "auto");
            parent.style("height", "auto");
            div.remove();
        });
        it("throws an error when computing the layout on an unanchored component", function () {
            // HACKHACK: https://github.com/palantir/plottable/issues/2661 Cannot assert errors being thrown with description
            chai_1.assert.throws(function () { return c.computeLayout(); }, Error, "anchor() must be called before", "cannot compute layout on an unanchored component");
            div.remove();
        });
        it("computes the layout of the component based on input", function () {
            var origin = {
                x: 10,
                y: 20,
            };
            var width = 100;
            var height = 200;
            c.anchor(div);
            c.computeLayout(origin, width, height);
            chai_1.assert.strictEqual(c.origin().x, origin.x, "x origin set");
            chai_1.assert.strictEqual(c.origin().y, origin.y, "y origin set");
            chai_1.assert.strictEqual(c.width(), width, "width set");
            chai_1.assert.strictEqual(c.height(), height, "height set");
            var componentElement = c.element();
            var translate = [parseFloat(componentElement.style("left")), parseFloat(componentElement.style("top"))];
            chai_1.assert.deepEqual(translate, [origin.x, origin.y], "the element translated appropriately");
            c.destroy();
            div.remove();
        });
        it("allows for recomputing the layout and rendering", function () {
            c.renderTo(div);
            c.computeLayout({ x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 }, DIV_WIDTH / 4, DIV_HEIGHT / 4);
            c.redraw();
            var origin = c.origin();
            chai_1.assert.deepEqual(origin, { x: 0, y: 0 }, "origin reset");
            var componentElement = div.select(".component");
            var translate = [parseFloat(componentElement.style("left")), parseFloat(componentElement.style("top"))];
            chai_1.assert.deepEqual(translate, [origin.x, origin.y], "DOM element rendered at new origin");
            c.destroy();
            div.remove();
        });
        it("calls onResize callback if callback is registered", function (done) {
            var origin = {
                x: 10,
                y: 20,
            };
            var width = 100;
            var height = 200;
            c.anchor(div);
            c.onResize(function (size) {
                chai_1.assert.deepEqual(size, { width: width, height: height });
                c.destroy();
                div.remove();
                done();
            });
            c.computeLayout(origin, width, height);
        });
    });
    describe("computing the layout when of fixed size", function () {
        var c;
        var div;
        var fixedWidth = 100;
        var fixedHeight = 100;
        beforeEach(function () {
            c = new Mocks.FixedSizeComponent(fixedWidth, fixedHeight);
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            c.anchor(div);
        });
        it("gives the component the fixed space if offered more space", function () {
            c.computeLayout();
            chai_1.assert.strictEqual(c.width(), fixedWidth, "width bounded to fixed width");
            chai_1.assert.strictEqual(c.height(), fixedHeight, "height bounded to fixed height");
            c.destroy();
            div.remove();
        });
        it("gives the component the offered space if it is less than the fixed space", function () {
            var restrictedWidth = fixedWidth / 2;
            var restrictedHeight = fixedHeight / 2;
            c.computeLayout({ x: 0, y: 0 }, restrictedWidth, restrictedHeight);
            chai_1.assert.strictEqual(c.width(), restrictedWidth, "width bounded to restricted width");
            chai_1.assert.strictEqual(c.height(), restrictedHeight, "height bounded to restricted height");
            c.destroy();
            div.remove();
        });
        it("does not translate if more space was requested than offered", function () {
            var requestedWidth = DIV_WIDTH * 2;
            var requestedHeight = DIV_HEIGHT * 2;
            c.destroy();
            c = new Mocks.FixedSizeComponent(requestedWidth, requestedHeight);
            var t = new Plottable.Components.Table([[c]]);
            t.renderTo(div);
            var componentElement = div.select(".component");
            var translate = [parseFloat(componentElement.style("left")), parseFloat(componentElement.style("top"))];
            chai_1.assert.deepEqual(translate, [0, 0], "the element was not translated");
            div.remove();
        });
    });
    describe("aligning", function () {
        var c;
        var div;
        beforeEach(function () {
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        });
        it("defaults to a top and left alignment", function () {
            chai_1.assert.strictEqual(c.xAlignment(), Plottable.XAlignment.left, "x alignment defaults to \"left\"");
            chai_1.assert.strictEqual(c.yAlignment(), Plottable.YAlignment.top, "y alignment defaults to \"top\"");
            c.destroy();
            div.remove();
        });
        it("can set the alignment", function () {
            var xAlignment = Plottable.XAlignment.right;
            chai_1.assert.strictEqual(c.xAlignment(xAlignment), c, "returns calling object");
            chai_1.assert.strictEqual(c.xAlignment(), xAlignment, "x alignment has been set");
            var yAlignment = Plottable.YAlignment.bottom;
            chai_1.assert.strictEqual(c.yAlignment(yAlignment), c, "returns calling object");
            chai_1.assert.strictEqual(c.yAlignment(), yAlignment, "y alignment has been set");
            c.destroy();
            div.remove();
        });
        it("throws errors on bad alignments", function () {
            var invalidAlignment = "foo";
            // HACKHACK: https://github.com/palantir/plottable/issues/2661 Cannot assert errors being thrown with description
            chai_1.assert.throws(function () { return c.xAlignment(invalidAlignment); }, Error, "Unsupported alignment", "cannot set an invalid x alignment");
            chai_1.assert.throws(function () { return c.yAlignment(invalidAlignment); }, Error, "Unsupported alignment", "cannot set an invalid y alignment");
            c.destroy();
            div.remove();
        });
    });
    describe("aligning when of fixed size", function () {
        var c;
        var div;
        var fixedWidth = 100;
        var fixedHeight = 100;
        beforeEach(function () {
            c = new Mocks.FixedSizeComponent(fixedWidth, fixedHeight);
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            c.anchor(div);
        });
        it("translates the origin in accordance with alignment", function () {
            c.xAlignment(Plottable.XAlignment.left)
                .yAlignment(Plottable.YAlignment.top);
            c.computeLayout();
            var expectedOrigin = { x: 0, y: 0 };
            chai_1.assert.deepEqual(c.origin(), expectedOrigin, "top-left component aligns correctly");
            c.xAlignment(Plottable.XAlignment.center)
                .yAlignment(Plottable.YAlignment.center);
            c.computeLayout();
            expectedOrigin = { x: DIV_WIDTH / 2 - fixedWidth / 2, y: DIV_HEIGHT / 2 - fixedHeight / 2 };
            chai_1.assert.deepEqual(c.origin(), expectedOrigin, "center component aligns correctly");
            c.xAlignment(Plottable.XAlignment.right)
                .yAlignment(Plottable.YAlignment.bottom);
            c.computeLayout();
            expectedOrigin = { x: DIV_WIDTH - fixedWidth, y: DIV_HEIGHT - fixedHeight };
            chai_1.assert.deepEqual(c.origin(), expectedOrigin, "bottom-right component aligns correctly");
            c.destroy();
            div.remove();
        });
    });
    describe("calculating the minimum requested space", function () {
        var c;
        var div;
        beforeEach(function () {
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        });
        it("does not request any space when offered a width and a height", function () {
            var offeredWidth = 1;
            var offeredHeight = 1;
            var layout = c.requestedSpace(offeredWidth, offeredHeight);
            chai_1.assert.strictEqual(layout.minWidth, 0, "requested minWidth defaults to 0");
            chai_1.assert.strictEqual(layout.minHeight, 0, "requested minHeight defaults to 0");
            div.remove();
        });
    });
    describe("destroying", function () {
        var c;
        var div;
        beforeEach(function () {
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        });
        it("cannot reanchor if destroyed", function () {
            c.renderTo(div);
            c.destroy();
            // HACKHACK: https://github.com/palantir/plottable/issues/2661 Cannot assert errors being thrown with description
            chai_1.assert.throws(function () { return c.renderTo(div); }, "reuse", "cannot reanchor a destroyed component");
            div.remove();
        });
        it("performs all of the same operations as detaching", function () {
            var detachCalled = false;
            c.detach = function () {
                detachCalled = true;
                return c;
            };
            c.destroy();
            chai_1.assert.isTrue(detachCalled, "detach called in destroy invocation");
            div.remove();
        });
    });
    describe("rendering on the anchored div", function () {
        var c;
        var div;
        beforeEach(function () {
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        });
        it("performs all of the same operations as renderImmediately()", function () {
            var renderFlag = false;
            c.renderImmediately = function () {
                renderFlag = true;
                return c;
            };
            c.anchor(div);
            c.computeLayout();
            chai_1.assert.strictEqual(c.render(), c, "returns calling object");
            chai_1.assert.isTrue(renderFlag, "renderImmediately() called in render invocation");
            c.destroy();
            div.remove();
        });
        it("does not render unless allocated space", function () {
            var renderFlag = false;
            c.renderImmediately = function () {
                renderFlag = true;
                return c;
            };
            c.anchor(div);
            c.render();
            chai_1.assert.isFalse(renderFlag, "no render until width/height set to nonzero");
            var offeredWidth = 10;
            var offeredHeight = 0;
            c.computeLayout({ x: 0, y: 0 }, offeredWidth, offeredHeight);
            c.render();
            chai_1.assert.isTrue(renderFlag, "render still occurs if one of width/height is zero");
            renderFlag = false;
            offeredHeight = 10;
            c.computeLayout({ x: 0, y: 0 }, offeredWidth, offeredHeight);
            c.render();
            chai_1.assert.isTrue(renderFlag, "render occurs if width and height are positive");
            c.destroy();
            div.remove();
        });
    });
    describe("rendering to a DOM node", function () {
        var c;
        var div;
        var renderFlag;
        beforeEach(function () {
            renderFlag = false;
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            c.renderImmediately = function () {
                renderFlag = true;
                return _this;
            };
        });
        afterEach(function () {
            Plottable.RenderController.renderPolicy(Plottable.RenderController.Policy.immediate);
        });
        it("renders to a DOM node involves anchoring, layout computing, and actual rendering", function () {
            chai_1.assert.strictEqual(c.renderTo(div), c, "returns calling object");
            chai_1.assert.isTrue(div.classed("plottable"), "anchored to div");
            chai_1.assert.strictEqual(c.width(), DIV_WIDTH, "component takes up div width");
            chai_1.assert.strictEqual(c.height(), DIV_HEIGHT, "component takes up div height");
            chai_1.assert.isTrue(renderFlag, "component has rendered");
            c.destroy();
            div.remove();
        });
        it("only computes layout once", function (done) {
            // use async rendering so flushing doesn't happen immediately
            Plottable.RenderController.renderPolicy(Plottable.RenderController.Policy.animationFrame);
            // set up a bar plot with a color scale
            var colorScale = new Plottable.Scales.Color();
            var xScale = new Plottable.Scales.Linear();
            var yScale = new Plottable.Scales.Linear();
            colorScale.autoDomain();
            var barPlot = new Plottable.Plots.Bar();
            barPlot.addDataset(new Plottable.Dataset([{ x: 1, y: 1 }]));
            barPlot.x(function (d) { return d.x; }, xScale);
            barPlot.y(function (d) { return d.y; }, yScale);
            // hook up the colorScale to look at data from the bar plot
            barPlot.attr("fill", function (d) { return d.x; }, colorScale);
            // set up a legend to based on the colorScale. Legend will trigger an internal redraw() no the Table
            // a second time; this test ensures that second redraw doesn't compute layout twice
            var legend = new Plottable.Components.Legend(colorScale);
            var table = new Plottable.Components.Table([[barPlot, legend]]);
            var computeLayoutSpy = sinon.spy(table, "computeLayout");
            table.renderTo(div);
            chai_1.assert.strictEqual(computeLayoutSpy.callCount, 1, "component only computes layout once");
            table.destroy();
            div.remove();
            done();
        });
        it("renders to a node chosen through D3 selection", function () {
            c.renderTo(div);
            chai_1.assert.isTrue(div.classed("plottable"), "anchored to div");
            chai_1.assert.isTrue(renderFlag, "component has rendered");
            c.destroy();
            div.remove();
        });
        it("renders to a node chosen through a selector string", function () {
            var divId = "foo";
            div.attr("id", divId);
            c.renderTo("#" + divId);
            chai_1.assert.isTrue(div.classed("plottable"), "correct div chosen");
            chai_1.assert.isTrue(renderFlag, "component has rendered");
            c.destroy();
            div.remove();
        });
        it("renders to a node chosen through DOM element", function () {
            c.renderTo(div.node());
            chai_1.assert.isTrue(div.classed("plottable"), "correct div chosen");
            chai_1.assert.isTrue(renderFlag, "component has rendered");
            c.destroy();
            div.remove();
        });
        it("errors on inputs that do not evaluate to an Element", function () {
            chai_1.assert.throws(function () { return c.renderTo("#not-an-element"); }, Error, "Plottable requires a valid Element to renderTo", "rejects strings that don't correspond to DOM elements");
            chai_1.assert.throws(function () { return c.renderTo(d3.select(null)); }, Error, "Plottable requires a valid Element to renderTo", "rejects empty d3 selections");
            c.destroy();
            div.remove();
        });
        it("detaches the component if rendering to a new div", function () {
            var divHeight2 = 50;
            var div2 = TestMethods.generateDiv(DIV_WIDTH, divHeight2);
            c.renderTo(div);
            chai_1.assert.isTrue(div.node().hasChildNodes(), "anchored onto div");
            chai_1.assert.strictEqual(c.height(), DIV_HEIGHT, "occupies entire space of div");
            c.renderTo(div2);
            chai_1.assert.isFalse(div.node().hasChildNodes(), "removed from div");
            chai_1.assert.isTrue(div2.node().hasChildNodes(), "anchored onto second div");
            chai_1.assert.strictEqual(c.height(), divHeight2, "occupies entire space of second div");
            c.destroy();
            div2.remove();
            div.remove();
        });
    });
    describe("calculating the origin in relation to the div", function () {
        var c;
        var div;
        beforeEach(function () {
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        });
        it("returns origin without a parent", function () {
            chai_1.assert.deepEqual(c.originToRoot(), c.origin(), "same as origin with no parent");
            c.destroy();
            div.remove();
        });
        it("is offset by the parent's origin", function () {
            var parent = new Plottable.Components.Group([c]);
            parent.anchor(div);
            c.anchor(div);
            parent.computeLayout({ x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 }, DIV_WIDTH / 2, DIV_HEIGHT / 2);
            c.computeLayout({ x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 }, DIV_WIDTH / 4, DIV_HEIGHT / 4);
            var originToRoot = {
                x: parent.origin().x + c.origin().x,
                y: parent.origin().y + c.origin().y,
            };
            chai_1.assert.deepEqual(c.originToRoot(), originToRoot, "origin offsetted by parents");
            parent.destroy();
            c.destroy();
            div.remove();
        });
    });
    describe("calculating the bounds", function () {
        var c;
        var div;
        beforeEach(function () {
            c = new Plottable.Component();
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            c.anchor(div);
            c.computeLayout();
        });
        it("calculates the bounds relative to the origin", function () {
            chai_1.assert.deepEqual(c.bounds(), {
                topLeft: c.origin(),
                bottomRight: { x: DIV_WIDTH, y: DIV_HEIGHT }
            });
            c.destroy();
            div.remove();
        });
    });
});
