"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("GuideLineLayer", function () {
    describe("DragLineLayer", function () {
        describe("Basic Usage", function () {
            var dll;
            var div;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                dll = new Plottable.Components.DragLineLayer("vertical");
            });
            it("can get and set detectionRadius property", function () {
                chai_1.assert.strictEqual(dll.detectionRadius(), 3, "defaults to 3");
                chai_1.assert.doesNotThrow(function () { return dll.detectionRadius(4); }, Error, "can set detection radius before anchoring");
                dll.renderTo(div);
                chai_1.assert.strictEqual(dll.detectionRadius(), 4, "detection radius did not change upon rendering");
                chai_1.assert.strictEqual(dll.detectionRadius(6), dll, "returns the calling DragLineLayer");
                chai_1.assert.strictEqual(dll.detectionRadius(), 6, "getter returns the set value");
                // HACKHACK #2661: Cannot assert errors being thrown with description
                chai_1.assert.throws(function () { return dll.detectionRadius(-1); }, Error, "detection radius cannot be negative.", "rejects negative values");
                div.remove();
            });
            it("can get and set enabled property", function () {
                chai_1.assert.isTrue(dll.enabled(), "defaults to true");
                chai_1.assert.strictEqual(dll.enabled(false), dll, "setter mode returns the calling DragLineLayer");
                chai_1.assert.isFalse(dll.enabled(), "successfully set to false");
                chai_1.assert.strictEqual(dll.enabled(true), dll, "setter mode returns the calling DragLineLayer");
                chai_1.assert.isTrue(dll.enabled(), "successfully set back to true");
                div.remove();
            });
            it("removes all callbacks on the DragLineLayer on destroy", function () {
                var callback = function () { return "foo"; };
                dll.onDragStart(callback);
                dll.onDrag(callback);
                dll.onDragEnd(callback);
                dll.destroy();
                var dragStartCallbacks = dll._dragStartCallbacks;
                var dragCallbacks = dll._dragCallbacks;
                var dragEndCallbacks = dll._dragEndCallbacks;
                chai_1.assert.strictEqual(dragStartCallbacks.size, 0, "dragStart callbacks removed on destroy()");
                chai_1.assert.strictEqual(dragCallbacks.size, 0, "drag callbacks removed on destroy()");
                chai_1.assert.strictEqual(dragEndCallbacks.size, 0, "drag end callbacks removed on destroy()");
                div.remove();
            });
            it("correctly disconnects from the internal Drag Interaction on destroy", function () {
                var dragInteraction = dll._dragInteraction;
                dll.destroy();
                var interactionStartCallbacks = dragInteraction._dragStartCallbacks;
                var interactionCallbacks = dragInteraction._dragCallbacks;
                var interactionEndCallbacks = dragInteraction._dragEndCallbacks;
                chai_1.assert.strictEqual(interactionStartCallbacks.size, 0, "Interaction dragStart callbacks removed on destroy()");
                chai_1.assert.strictEqual(interactionCallbacks.size, 0, "Interaction drag callbacks removed on destroy()");
                chai_1.assert.strictEqual(interactionEndCallbacks.size, 0, "Interaction drag end callbacks removed on destroy()");
                chai_1.assert.notStrictEqual(dragInteraction._componentAttachedTo, dll, "Interaction was detached");
                div.remove();
            });
        });
        ["vertical", "horizontal"].forEach(function (orientation) {
            describe("Dragging orientation for " + orientation + " DragLineLayer", function () {
                var DIV_WIDTH = orientation === "vertical" ? 400 : 300;
                var DIV_HEIGHT = orientation === "vertical" ? 300 : 400;
                var DRAG_EDGE_CSSCLASS = ".drag-edge";
                var GUIDE_LINE_CSSCLASS = ".guide-line";
                var quarterPoint = {
                    x: DIV_WIDTH / 4,
                    y: DIV_HEIGHT / 4,
                };
                var halfPoint = {
                    x: DIV_WIDTH / 2,
                    y: DIV_HEIGHT / 2,
                };
                var dll;
                var div;
                beforeEach(function () {
                    div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                    dll = new Plottable.Components.DragLineLayer(orientation);
                });
                function getExpectedAttr(value) {
                    return {
                        x1: orientation === "vertical" ? value : 0,
                        x2: orientation === "vertical" ? value : DIV_WIDTH,
                        y1: orientation === "vertical" ? 0 : value,
                        y2: orientation === "vertical" ? DIV_HEIGHT : value,
                    };
                }
                ;
                function getDragPoint(value) {
                    return {
                        x: orientation === "vertical" ? value : halfPoint.x,
                        y: orientation === "vertical" ? halfPoint.y : value,
                    };
                }
                ;
                function getRelventAttrFromPoint(point) {
                    return orientation === "vertical" ? point.x : point.y;
                }
                ;
                it("shows the correct cursor", function () {
                    dll.renderTo(div);
                    var dragEdge = dll.content().select(DRAG_EDGE_CSSCLASS);
                    var computedStyles = window.getComputedStyle(dragEdge.node());
                    chai_1.assert.notStrictEqual(computedStyles.stroke, "none", "drag-edge has a non-\"none\" stroke");
                    var expectCursor = orientation === "vertical" ? "ew-resize" : "ns-resize";
                    chai_1.assert.strictEqual(computedStyles.cursor, expectCursor, "shows the resize cursor if enabled");
                    dll.enabled(false);
                    computedStyles = window.getComputedStyle(dragEdge.node());
                    chai_1.assert.strictEqual(computedStyles.cursor, "auto", "cursor set to \"auto\" if not enabled");
                    div.remove();
                });
                it("moves the line on drag", function () {
                    var startPosition = getRelventAttrFromPoint(halfPoint);
                    var endPosition = getRelventAttrFromPoint(quarterPoint);
                    dll.pixelPosition(startPosition);
                    dll.renderTo(div);
                    TestMethods.triggerFakeDragSequence(dll.background(), getDragPoint(startPosition), getDragPoint(endPosition));
                    TestMethods.assertLineAttrs(dll.content().select(GUIDE_LINE_CSSCLASS), getExpectedAttr(endPosition), "line was dragged to the final location");
                    chai_1.assert.strictEqual(dll.pixelPosition(), endPosition, "pixelPosition was updated correctly");
                    div.remove();
                });
                it("drags line if drag starts within detectionRadius", function () {
                    var startPosition = getRelventAttrFromPoint(halfPoint);
                    var endPosition = getRelventAttrFromPoint(quarterPoint);
                    dll.pixelPosition(startPosition);
                    dll.renderTo(div);
                    TestMethods.triggerFakeDragSequence(dll.background(), getDragPoint(startPosition + dll.detectionRadius()), getDragPoint(endPosition));
                    TestMethods.assertLineAttrs(dll.content().select(GUIDE_LINE_CSSCLASS), getExpectedAttr(endPosition), "line was dragged to the final location");
                    chai_1.assert.strictEqual(dll.pixelPosition(), endPosition, "pixelPosition was updated correctly");
                    div.remove();
                });
                it("does nothing if drag starts outside the detection radius ", function () {
                    var startPosition = getRelventAttrFromPoint(halfPoint);
                    var endPosition = getRelventAttrFromPoint(quarterPoint);
                    dll.pixelPosition(startPosition);
                    dll.renderTo(div);
                    TestMethods.triggerFakeDragSequence(dll.background(), getDragPoint(startPosition + 2 * dll.detectionRadius()), getDragPoint(endPosition));
                    TestMethods.assertLineAttrs(dll.content().select(GUIDE_LINE_CSSCLASS), getExpectedAttr(startPosition), "line did not move");
                    chai_1.assert.strictEqual(dll.pixelPosition(), startPosition, "pixelPosition was not changed");
                    div.remove();
                });
                it("does nothing if not enabled", function () {
                    var startPosition = getRelventAttrFromPoint(halfPoint);
                    var endPosition = getRelventAttrFromPoint(quarterPoint);
                    dll.enabled(false);
                    dll.pixelPosition(startPosition);
                    dll.renderTo(div);
                    TestMethods.triggerFakeDragSequence(dll.background(), getDragPoint(startPosition), getDragPoint(endPosition));
                    TestMethods.assertLineAttrs(dll.content().select(GUIDE_LINE_CSSCLASS), getExpectedAttr(startPosition), "line did not move");
                    chai_1.assert.strictEqual(dll.pixelPosition(), startPosition, "pixelPosition was not changed");
                    div.remove();
                });
                it("does not affect the mode dragging in value mode", function () {
                    var scale = new Plottable.Scales.Linear();
                    scale.domain([0, 1]);
                    dll.scale(scale);
                    var startValue = 0.5;
                    dll.value(startValue);
                    dll.renderTo(div);
                    var startPosition = scale.scale(startValue);
                    var endPosition = getRelventAttrFromPoint(quarterPoint);
                    TestMethods.triggerFakeDragSequence(dll.background(), getDragPoint(startPosition), getDragPoint(endPosition));
                    var endValue = scale.invert(endPosition);
                    chai_1.assert.strictEqual(dll.value(), endValue, "value was updated correctly");
                    scale.domain([0, 2]);
                    chai_1.assert.strictEqual(dll.value(), endValue, "remained in \"value\" mode: value did not change when scale updated");
                    div.remove();
                });
                it("does not affect the mode dragging in pixel mode", function () {
                    var scale = new Plottable.Scales.Linear();
                    scale.domain([0, 1]);
                    dll.scale(scale);
                    var startPosition = getRelventAttrFromPoint(halfPoint);
                    dll.pixelPosition(startPosition);
                    dll.renderTo(div);
                    var endPosition = getRelventAttrFromPoint(quarterPoint);
                    TestMethods.triggerFakeDragSequence(dll.background(), getDragPoint(startPosition), getDragPoint(endPosition));
                    var endValue = scale.invert(endPosition);
                    chai_1.assert.strictEqual(dll.value(), endValue, "value was updated correctly");
                    scale.domain([0, 2]);
                    chai_1.assert.strictEqual(dll.pixelPosition(), endPosition, "remained in \"position\" mode: position did not change when scale updated");
                    div.remove();
                });
            });
        });
        describe("Interactions", function () {
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 300;
            var startPosition = DIV_WIDTH / 2;
            var dll;
            var callbackCalled = false;
            var pixelPositionOnCalling;
            var callback;
            var div;
            beforeEach(function () {
                dll = new Plottable.Components.DragLineLayer("vertical");
                callback = function (layer) {
                    chai_1.assert.strictEqual(layer, dll, "was passed the calling DragLineLayer");
                    pixelPositionOnCalling = dll.pixelPosition();
                    callbackCalled = true;
                };
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                dll.pixelPosition(startPosition);
                dll.renderTo(div);
            });
            it("calls callback onDragStart", function () {
                chai_1.assert.strictEqual(dll.onDragStart(callback), dll, "onDragStart() returns the calling DragLineLayer");
                dll.renderTo(div);
                TestMethods.triggerFakeMouseEvent("mousedown", dll.background(), DIV_WIDTH / 2, DIV_HEIGHT / 2);
                chai_1.assert.isTrue(callbackCalled, "callback was called on drag start");
                chai_1.assert.strictEqual(pixelPositionOnCalling, DIV_WIDTH / 2, "DragLineLayer's state was updated before calling callbacks");
                TestMethods.triggerFakeMouseEvent("mouseup", dll.background(), DIV_WIDTH / 2, DIV_HEIGHT / 2);
                callbackCalled = false;
                TestMethods.triggerFakeMouseEvent("mousedown", dll.background(), 0, DIV_HEIGHT / 2);
                chai_1.assert.isFalse(callbackCalled, "callback not called if line was not grabbed");
                TestMethods.triggerFakeMouseEvent("mouseup", dll.background(), 0, DIV_HEIGHT / 2);
                chai_1.assert.strictEqual(dll.offDragStart(callback), dll, "offDragStart() returns the calling DragLineLayer");
                TestMethods.triggerFakeMouseEvent("mousedown", dll.background(), DIV_WIDTH / 2, DIV_HEIGHT / 2);
                chai_1.assert.isFalse(callbackCalled, "callback was deregistered successfully");
                TestMethods.triggerFakeMouseEvent("mouseup", dll.background(), DIV_WIDTH / 2, DIV_HEIGHT / 2);
                div.remove();
            });
            it("calls callback onDrag", function () {
                chai_1.assert.strictEqual(dll.onDrag(callback), dll, "onDrag() returns the calling DragLineLayer");
                var midX = startPosition * 3 / 8;
                var endX = DIV_WIDTH / 4;
                TestMethods.triggerFakeMouseEvent("mousedown", dll.background(), startPosition, DIV_HEIGHT / 2);
                TestMethods.triggerFakeMouseEvent("mousemove", dll.background(), midX, DIV_HEIGHT / 2);
                chai_1.assert.isTrue(callbackCalled, "callback was called on drag");
                chai_1.assert.strictEqual(pixelPositionOnCalling, midX, "DragLineLayer's state was updated before calling callbacks");
                callbackCalled = false;
                TestMethods.triggerFakeMouseEvent("mousemove", dll.background(), endX, DIV_HEIGHT / 2);
                chai_1.assert.isTrue(callbackCalled, "callback was called again on continued dragging");
                chai_1.assert.strictEqual(pixelPositionOnCalling, endX, "DragLineLayer's state was updated again");
                TestMethods.triggerFakeMouseEvent("mouseup", dll.background(), endX, DIV_HEIGHT / 2);
                callbackCalled = false;
                dll.pixelPosition(startPosition);
                TestMethods.triggerFakeMouseEvent("mousedown", dll.background(), 0, DIV_HEIGHT / 2);
                TestMethods.triggerFakeMouseEvent("mousemove", dll.background(), endX, DIV_HEIGHT / 2);
                chai_1.assert.isFalse(callbackCalled, "callback not called if line was not grabbed");
                TestMethods.triggerFakeMouseEvent("mouseup", dll.background(), endX, DIV_HEIGHT / 2);
                chai_1.assert.strictEqual(dll.offDrag(callback), dll, "offDrag() returns the calling DragLineLayer");
                TestMethods.triggerFakeMouseEvent("mousedown", dll.background(), startPosition, DIV_HEIGHT / 2);
                TestMethods.triggerFakeMouseEvent("mousemove", dll.background(), endX, DIV_HEIGHT / 2);
                chai_1.assert.isFalse(callbackCalled, "callback was deregistered successfully");
                TestMethods.triggerFakeMouseEvent("mouseup", dll.background(), endX, DIV_HEIGHT / 2);
                div.remove();
            });
            it("calls callback onDragEnd", function () {
                chai_1.assert.strictEqual(dll.onDragEnd(callback), dll, "onDragEnd() returns the calling DragLineLayer");
                var endPosition = DIV_WIDTH / 4;
                TestMethods.triggerFakeDragSequence(dll.background(), { x: startPosition, y: DIV_HEIGHT / 2 }, { x: endPosition, y: DIV_HEIGHT / 2 });
                chai_1.assert.isTrue(callbackCalled, "callback was called on drag end");
                chai_1.assert.strictEqual(pixelPositionOnCalling, endPosition, "DragLineLayer's state was updated before calling callbacks");
                callbackCalled = false;
                dll.pixelPosition(startPosition);
                TestMethods.triggerFakeDragSequence(dll.background(), { x: 0, y: DIV_HEIGHT / 2 }, { x: endPosition, y: DIV_HEIGHT / 2 });
                chai_1.assert.isFalse(callbackCalled, "callback not called if line was not grabbed");
                chai_1.assert.strictEqual(dll.offDragEnd(callback), dll, "offDragEnd() returns the calling DragLineLayer");
                TestMethods.triggerFakeDragSequence(dll.background(), { x: startPosition, y: DIV_HEIGHT / 2 }, { x: endPosition, y: DIV_HEIGHT / 2 });
                chai_1.assert.isFalse(callbackCalled, "callback was deregistered successfully");
                div.remove();
            });
        });
    });
});
