"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var Mocks = require("../mocks");
var TestMethods = require("../testMethods");
describe("Group", function () {
    describe("Basic add/remove/has functionality", function () {
        it("appends Components with append()", function () {
            var componentGroup = new Plottable.Components.Group();
            var c1 = new Plottable.Component();
            componentGroup.append(c1);
            chai_1.assert.deepEqual(componentGroup.components(), [c1], "Component 1 was added to the Group");
            var c2 = new Plottable.Component();
            componentGroup.append(c2);
            chai_1.assert.deepEqual(componentGroup.components(), [c1, c2], "appended Component 2 to the Group");
            componentGroup.append(c1);
            chai_1.assert.deepEqual(componentGroup.components(), [c1, c2], "adding an already-added Component does nothing");
            var div = TestMethods.generateDiv();
            componentGroup.renderTo(div);
            var c3 = new Plottable.Component();
            componentGroup.append(c3);
            chai_1.assert.deepEqual(componentGroup.components(), [c1, c2, c3], "Components can be append()ed after rendering");
            div.remove();
        });
        it("can append() null to a Group without failing", function () {
            var group = new Plottable.Components.Group();
            var component = new Plottable.Component;
            group.append(component);
            chai_1.assert.strictEqual(group.components().length, 1, "there should first be 1 element in the group");
            chai_1.assert.doesNotThrow(function () { return group.append(null); });
            chai_1.assert.strictEqual(group.components().length, 1, "adding null to a group should have no effect on the group");
        });
        it("removes Components using remove()", function () {
            var c1 = new Plottable.Component();
            var c2 = new Plottable.Component();
            var c3 = new Plottable.Component();
            var componentGroup = new Plottable.Components.Group([c1, c2, c3]);
            chai_1.assert.deepEqual(componentGroup.components(), [c1, c2, c3], "Group initialized correctly");
            componentGroup.remove(c2);
            chai_1.assert.deepEqual(componentGroup.components(), [c1, c3], "removing a Component respects the order of the remaining Components");
            var div = TestMethods.generateDiv();
            c2.renderTo(div);
            componentGroup.remove(c2);
            chai_1.assert.deepEqual(componentGroup.components(), [c1, c3], "removing a Component not in the Group does not remove Components from the Group");
            chai_1.assert.strictEqual(c2.element().node().parentNode, div.node(), "The Component not in the Group is still anchored");
            div.remove();
        });
        it("checks for Components using has()", function () {
            var component = new Plottable.Component();
            var group = new Plottable.Components.Group([component]);
            chai_1.assert.isTrue(group.has(component), "correctly checks that Component is in the Group");
            group.remove(component);
            chai_1.assert.isFalse(group.has(component), "correctly checks that Component is no longer in the Group");
            group.append(component);
            chai_1.assert.isTrue(group.has(component), "correctly checks that Component is in the Group again");
        });
    });
    describe("Detaching constituent Components", function () {
        it("takes its constutuent Components with it when detach()ed or anchor()ed", function () {
            var c1 = new Plottable.Component();
            var c2 = new Plottable.Component();
            var group = new Plottable.Components.Group([c1, c2]);
            var div = TestMethods.generateDiv();
            group.renderTo(div);
            chai_1.assert.isTrue(TestMethods.isInDOM(group), "Group was added to the DOM");
            chai_1.assert.isTrue(TestMethods.isInDOM(c1), "Component 1 was added to the DOM");
            chai_1.assert.isTrue(TestMethods.isInDOM(c2), "Component 2 was added to the DOM");
            group.detach();
            chai_1.assert.isFalse(TestMethods.isInDOM(group), "Group was removed from the DOM");
            chai_1.assert.isFalse(TestMethods.isInDOM(c1), "Component 1 was also removed from the DOM");
            chai_1.assert.isFalse(TestMethods.isInDOM(c2), "Component 2 was also removed from the DOM");
            group.renderTo(div);
            chai_1.assert.isTrue(TestMethods.isInDOM(group), "Group was added back to the DOM");
            chai_1.assert.isTrue(TestMethods.isInDOM(c1), "Component 1 was also added back to the DOM");
            chai_1.assert.isTrue(TestMethods.isInDOM(c2), "Component 2 was also added back to the DOM");
            div.remove();
        });
        it("detach()es Components from their previous location when they are append()ed", function () {
            var component = new Plottable.Component;
            var div = TestMethods.generateDiv();
            component.renderTo(div);
            var group = new Plottable.Components.Group();
            group.append(component);
            chai_1.assert.isFalse(div.node().hasChildNodes(), "Component was detach()ed");
            div.remove();
        });
        it("removes Components if detach() is called on them (before rendering)", function () {
            var component = new Plottable.Component();
            var group = new Plottable.Components.Group([component]);
            component.detach();
            chai_1.assert.lengthOf(group.components(), 0, "Component is no longer in the Group");
            chai_1.assert.isNull(component.parent(), "Component disconnected from Group");
        });
        it("removes Components if detach() is called on them (after rendering)", function () {
            var c1 = new Plottable.Component();
            var c2 = new Plottable.Component();
            var group = new Plottable.Components.Group([c1, c2]);
            var div = TestMethods.generateDiv();
            group.renderTo(div);
            chai_1.assert.isTrue(TestMethods.isInDOM(c1), "Component 1 was added to the DOM");
            chai_1.assert.isTrue(TestMethods.isInDOM(c2), "Component 2 was added to the DOM");
            c2.detach();
            chai_1.assert.isTrue(TestMethods.isInDOM(c1), "Component 1 is still in the DOM");
            chai_1.assert.isFalse(TestMethods.isInDOM(c2), "Component 2 was removed from the DOM");
            chai_1.assert.isFalse(group.has(c2), "Component 2 was removed from the Group");
            div.remove();
        });
        it("can move Components to other Groups after anchoring", function () {
            var div = TestMethods.generateDiv();
            var group1 = new Plottable.Components.Group();
            var group2 = new Plottable.Components.Group();
            var component = new Plottable.Component();
            group1.append(component);
            group1.renderTo(div);
            group2.renderTo(div);
            chai_1.assert.strictEqual(group2.components().length, 0, "second group should have no component before movement");
            chai_1.assert.strictEqual(group1.components().length, 1, "first group should have 1 component before movement");
            chai_1.assert.strictEqual(component.parent(), group1, "component's parent before moving should be the group 1");
            chai_1.assert.doesNotThrow(function () { return group2.append(component); }, Error, "should be able to move components between groups after anchoring");
            chai_1.assert.strictEqual(group2.components().length, 1, "second group should have 1 component after movement");
            chai_1.assert.strictEqual(group1.components().length, 0, "first group should have no components after movement");
            chai_1.assert.strictEqual(component.parent(), group2, "component's parent after movement should be the group 2");
            div.remove();
        });
        it("destroy()s its Components when destroy()ed", function () {
            var c1 = new Plottable.Component();
            var c2 = new Plottable.Component();
            var group = new Plottable.Components.Group([c1, c2]);
            var div = TestMethods.generateDiv();
            group.renderTo(div);
            group.destroy();
            // HACKHACK #2614: chai-assert.d.ts has the wrong signature
            chai_1.assert.throws(function () { return c1.renderTo(div); }, Error, "Can't reuse destroy()-ed Components!", "Component 1 was destroyed");
            chai_1.assert.throws(function () { return c2.renderTo(div); }, Error, "Can't reuse destroy()-ed Components!", "Component 2 was destroyed");
            div.remove();
        });
    });
    describe("Layout", function () {
        var DIV_WIDTH = 400;
        var DIV_HEIGHT = 400;
        var div;
        beforeEach(function () {
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        });
        it("requests no space when empty, but occupies all offered space", function () {
            var group = new Plottable.Components.Group([]);
            var request = group.requestedSpace(DIV_WIDTH, DIV_HEIGHT);
            TestMethods.verifySpaceRequest(request, 0, 0, "empty Group doesn't request any space");
            group.renderTo(div);
            chai_1.assert.strictEqual(group.width(), DIV_WIDTH, "occupies all offered width");
            chai_1.assert.strictEqual(group.height(), DIV_HEIGHT, "occupies all offered height");
            div.remove();
        });
        it("requests space correctly when it contains a non-fixed-size Component", function () {
            var c1 = new Plottable.Component();
            var c2 = new Plottable.Component();
            var group = new Plottable.Components.Group([c1, c2]);
            var groupRequest = group.requestedSpace(DIV_WIDTH, DIV_HEIGHT);
            var c1Request = c1.requestedSpace(DIV_WIDTH, DIV_HEIGHT);
            chai_1.assert.deepEqual(groupRequest, c1Request, "request reflects request of sub-component");
            chai_1.assert.isFalse(group.fixedWidth(), "width is not fixed if subcomponents are not fixed width");
            chai_1.assert.isFalse(group.fixedHeight(), "height is not fixed if subcomponents are not fixed height");
            group.renderTo(div);
            chai_1.assert.strictEqual(group.width(), DIV_WIDTH, "occupies all offered width");
            chai_1.assert.strictEqual(group.height(), DIV_HEIGHT, "occupies all offered height");
            div.remove();
        });
        it("requests space correctly when it contains fixed-size Components", function () {
            var tallComponent = new Mocks.FixedSizeComponent(DIV_WIDTH / 4, DIV_WIDTH / 2);
            var wideComponent = new Mocks.FixedSizeComponent(DIV_WIDTH / 2, DIV_WIDTH / 4);
            var group = new Plottable.Components.Group([tallComponent, wideComponent]);
            var request = group.requestedSpace(DIV_WIDTH, DIV_HEIGHT);
            chai_1.assert.strictEqual(request.minWidth, DIV_WIDTH / 2, "requested enough space for widest Component");
            chai_1.assert.strictEqual(request.minHeight, DIV_HEIGHT / 2, "requested enough space for tallest Component");
            var constrainedRequest = group.requestedSpace(DIV_WIDTH / 10, DIV_HEIGHT / 10);
            chai_1.assert.strictEqual(constrainedRequest.minWidth, DIV_WIDTH / 2, "requested enough space for widest Component");
            chai_1.assert.strictEqual(constrainedRequest.minHeight, DIV_HEIGHT / 2, "requested enough space for tallest Component");
            group.renderTo(div);
            chai_1.assert.strictEqual(group.width(), DIV_WIDTH, "occupies all offered width");
            chai_1.assert.strictEqual(group.height(), DIV_HEIGHT, "occupies all offered height");
            div.remove();
        });
        it("allocates space to its Components correctly", function () {
            var FIXED_COMPONENT_SIZE = DIV_WIDTH / 4;
            var fixedComponent = new Mocks.FixedSizeComponent(FIXED_COMPONENT_SIZE, FIXED_COMPONENT_SIZE);
            fixedComponent.xAlignment("right").yAlignment("bottom");
            var unfixedComponent = new Plottable.Component();
            var group = new Plottable.Components.Group([fixedComponent, unfixedComponent]);
            group.renderTo(div);
            chai_1.assert.strictEqual(fixedComponent.width(), FIXED_COMPONENT_SIZE, "fixed-size Component has correct width");
            chai_1.assert.strictEqual(fixedComponent.height(), FIXED_COMPONENT_SIZE, "fixed-size Component has correct height");
            var expectedFixedOrigin = {
                x: DIV_WIDTH - FIXED_COMPONENT_SIZE,
                y: DIV_HEIGHT - FIXED_COMPONENT_SIZE,
            };
            TestMethods.assertPointsClose(expectedFixedOrigin, fixedComponent.origin(), 1, "fixed-size Component has correct origin");
            chai_1.assert.strictEqual(unfixedComponent.width(), DIV_WIDTH, "non-fixed-size Component has correct width");
            chai_1.assert.strictEqual(unfixedComponent.height(), DIV_HEIGHT, "non-fixed-size Component has correct height");
            var expectedUnfixedOrigin = {
                x: 0,
                y: 0,
            };
            TestMethods.assertPointsClose(expectedUnfixedOrigin, unfixedComponent.origin(), 1, "non-fixed-size Component has correct origin");
            div.remove();
        });
    });
});
