"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("GuideLineLayer", function () {
    describe("Basic Usage", function () {
        it("has the \"vertical\" class if vertical, and the \"horizontal\" class if horizontal", function () {
            var verticalGLL = new Plottable.Components.GuideLineLayer("vertical");
            chai_1.assert.isTrue(verticalGLL.hasClass("vertical"), "vertical GuideLineLayer has \"vertical\" class");
            var horizontalGLL = new Plottable.Components.GuideLineLayer("horizontal");
            chai_1.assert.isTrue(horizontalGLL.hasClass("horizontal"), "horizontal GuideLineLayer has \"horizontal\" class");
        });
        it("can get and set the scale property", function () {
            var gll = new Plottable.Components.GuideLineLayer("vertical");
            var timeScale = new Plottable.Scales.Time();
            chai_1.assert.isUndefined(gll.scale(), "there is no default scale");
            chai_1.assert.strictEqual(gll.scale(timeScale), gll, "setter returns the calling GuideLineLayer");
            chai_1.assert.strictEqual(gll.scale(), timeScale, "getter returns the set Scale");
        });
        it("can get and set the value property", function () {
            var gll = new Plottable.Components.GuideLineLayer("vertical");
            var expectedValue = 5;
            chai_1.assert.isUndefined(gll.value(), "returns undefined before any value is set");
            chai_1.assert.strictEqual(gll.value(expectedValue), gll, "setter returns the calling GuideLineLayer");
            chai_1.assert.strictEqual(gll.value(), expectedValue, "getter returns the set value");
        });
        it("can get and set the pixelPosition property", function () {
            var gll = new Plottable.Components.GuideLineLayer("vertical");
            var expectedPosition = 5;
            chai_1.assert.isUndefined(gll.pixelPosition(), "returns undefined before any pixel position is set");
            chai_1.assert.strictEqual(gll.pixelPosition(expectedPosition), gll, "setter returns the calling GuideLineLayer");
            chai_1.assert.strictEqual(gll.pixelPosition(), expectedPosition, "getter returns the set pixel position");
            // HACKHACK #2614: chai-assert.d.ts has the wrong signature
            chai_1.assert.throws(function () { return gll.pixelPosition(NaN); }, Error, "pixelPosition must be a finite number", "Rejects NaN");
            chai_1.assert.throws(function () { return gll.pixelPosition(Infinity); }, Error, "pixelPosition must be a finite number", "Rejects Infinity");
            chai_1.assert.throws(function () { return gll.pixelPosition(-Infinity); }, Error, "pixelPosition must be a finite number", "Rejects -Infinity");
            chai_1.assert.throws(function () { return gll.pixelPosition("5"); }, Error, "pixelPosition must be a finite number", "Rejects stringy numbers");
        });
        it("disconnects scales safely on destroy", function () {
            var scaleLessGLL = new Plottable.Components.GuideLineLayer("vertical");
            chai_1.assert.doesNotThrow(function () { return scaleLessGLL.destroy(); }, Error, "destroy() does not error if no scale was set");
            var timeScaleGLL = new Plottable.Components.GuideLineLayer("vertical");
            var timeScale = new Plottable.Scales.Time();
            timeScaleGLL.scale(timeScale);
            chai_1.assert.doesNotThrow(function () { return timeScaleGLL.destroy(); }, Error, "destroy() does not error if a scale was set");
            chai_1.assert.strictEqual(timeScale._callbacks.size, 0, "callback was removed from Scale");
        });
        it("rejects invalid orientations", function () {
            chai_1.assert.doesNotThrow(function () { return new Plottable.Components.GuideLineLayer("vertical"); }, Error, "accepts \"vertical\"");
            chai_1.assert.doesNotThrow(function () { return new Plottable.Components.GuideLineLayer("horizontal"); }, Error, "accepts \"horizontal\"");
            chai_1.assert.throws(function () { return new Plottable.Components.GuideLineLayer("blargh"); }, Error, "blargh is not a valid orientation for GuideLineLayer", "throws error on invalid orientation");
        });
    });
    describe("Coordination between scale, value, and pixelPosition", function () {
        var linearScale;
        var gll;
        beforeEach(function () {
            linearScale = new Plottable.Scales.Linear();
            linearScale.domain([0, 1]);
            linearScale.range([0, 100]);
            gll = new Plottable.Components.GuideLineLayer("vertical");
        });
        it("updates pixelPosition when changing value if scale is set", function () {
            gll.scale(linearScale);
            var value = 0.5;
            var expectedPosition = linearScale.scale(value);
            gll.value(value);
            chai_1.assert.strictEqual(gll.pixelPosition(), expectedPosition, "pixel position was updated to match the set value");
            var valueB = 0.8;
            var expectedPositionB = linearScale.scale(valueB);
            gll.value(valueB);
            chai_1.assert.strictEqual(gll.pixelPosition(), expectedPositionB, "pixel position was updated when value was changed again");
        });
        it("updates value when changing pixelPosition if scale is set", function () {
            gll.scale(linearScale);
            var position = 50;
            var expectedValue = linearScale.invert(position);
            gll.pixelPosition(position);
            chai_1.assert.strictEqual(gll.value(), expectedValue, "value was updated to match the set pixel position");
            var positionB = 75;
            var expectedValueB = linearScale.invert(positionB);
            gll.pixelPosition(positionB);
            chai_1.assert.strictEqual(gll.value(), expectedValueB, "value was updated when the position was changed again");
        });
        it("updates pixelPosition when the scale's domain changes if value was the last property set", function () {
            gll.scale(linearScale);
            var value = 0.5;
            gll.value(value);
            chai_1.assert.strictEqual(gll.pixelPosition(), linearScale.scale(value), "pixel position matches set value");
            linearScale.domain([0, 2]);
            chai_1.assert.strictEqual(gll.pixelPosition(), linearScale.scale(value), "pixel position was updated when scale updated");
        });
        it("updates value when the scale's domain changes if pixelPosition was the last property set", function () {
            gll.scale(linearScale);
            var pixelPosition = 50;
            gll.pixelPosition(pixelPosition);
            chai_1.assert.strictEqual(gll.value(), linearScale.invert(pixelPosition), "value matches set pixel position");
            linearScale.domain([0, 2]);
            chai_1.assert.strictEqual(gll.value(), linearScale.invert(pixelPosition), "value was updated when scale updated");
        });
        it("updates pixelPosition when changing the scale if value was the last property set", function () {
            var setPosition = -100;
            gll.pixelPosition(setPosition);
            var setValue = 0.5;
            gll.value(setValue);
            var expectedPosition = linearScale.scale(setValue);
            gll.scale(linearScale);
            chai_1.assert.strictEqual(gll.pixelPosition(), expectedPosition, "setting the scale updates the pixel position if value() was the last thing set");
            chai_1.assert.strictEqual(gll.value(), setValue, "value is not changed");
            chai_1.assert.notStrictEqual(gll.pixelPosition(), setPosition, "originally-set pixel position was overridden");
            var linearScaleB = new Plottable.Scales.Linear();
            linearScaleB.domain([0, 1]);
            linearScaleB.range([0, 200]);
            var expectedPositionB = linearScaleB.scale(setValue);
            gll.scale(linearScaleB);
            chai_1.assert.strictEqual(gll.pixelPosition(), expectedPositionB, "changing the scale updates the pixel position if value() was the last thing set");
            chai_1.assert.strictEqual(gll.value(), setValue, "value is not changed");
            chai_1.assert.strictEqual(linearScale._callbacks.size, 0, "callback was removed from the previous Scale");
        });
        it("updates value when changing the scale if pixelPosition was the last property set", function () {
            var setValue = 0.5;
            gll.value(setValue);
            var setPosition = -100;
            gll.pixelPosition(setPosition);
            var expectedValue = linearScale.invert(setPosition);
            gll.scale(linearScale);
            chai_1.assert.strictEqual(gll.value(), expectedValue, "setting the scale updates the value if pixelPosition() was the last thing set");
            chai_1.assert.strictEqual(gll.pixelPosition(), setPosition, "pixel position is not changed");
            chai_1.assert.notStrictEqual(gll.value(), setValue, "originally-set value was overridden");
            var linearScaleB = new Plottable.Scales.Linear();
            linearScaleB.domain([0, 1]);
            linearScaleB.range([0, 200]);
            var expectedValueB = linearScaleB.invert(setPosition);
            gll.scale(linearScaleB);
            chai_1.assert.strictEqual(gll.value(), expectedValueB, "changing the scale updates the value if pixelPosition() was the last thing set");
            chai_1.assert.strictEqual(gll.pixelPosition(), setPosition, "pixel position is not changed");
            chai_1.assert.strictEqual(linearScale._callbacks.size, 0, "callback was removed from the previous Scale");
        });
    });
    ["vertical", "horizontal"].forEach(function (orientation) {
        describe("Rendering " + orientation + " guide line", function () {
            var DIV_WIDTH = orientation === "vertical" ? 400 : 300;
            var DIV_HEIGHT = orientation === "vertical" ? 300 : 400;
            var GUIDE_LINE_CLASS = ".guide-line";
            var div;
            beforeEach(function () {
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            });
            function getExpectedAttr(value) {
                return {
                    x1: orientation === "vertical" ? value : 0,
                    x2: orientation === "vertical" ? value : DIV_WIDTH,
                    y1: orientation === "vertical" ? 0 : value,
                    y2: orientation === "vertical" ? DIV_HEIGHT : value,
                };
            }
            it("requests no space, but will occupy all offered space", function () {
                var gll = new Plottable.Components.GuideLineLayer(orientation);
                var request = gll.requestedSpace(DIV_WIDTH, DIV_HEIGHT);
                TestMethods.verifySpaceRequest(request, 0, 0, "does not request any space");
                chai_1.assert.isTrue(gll.fixedWidth(), "fixed width");
                chai_1.assert.isTrue(gll.fixedHeight(), "fixed height");
                gll.anchor(div);
                gll.computeLayout({ x: 0, y: 0 }, DIV_WIDTH, DIV_HEIGHT);
                chai_1.assert.strictEqual(gll.width(), DIV_WIDTH, "accepted all offered width");
                chai_1.assert.strictEqual(gll.height(), DIV_HEIGHT, "accepted all offered height");
                div.remove();
            });
            it("turns _overflowHidden on", function () {
                var gll = new Plottable.Components.GuideLineLayer(orientation);
                chai_1.assert.isTrue(gll._overflowHidden, "overflowHidden is enabled");
                div.remove();
            });
            it("renders correctly given a pixel position", function () {
                var range = (orientation === "vertical" ? DIV_WIDTH : DIV_HEIGHT);
                var expectedPosition1 = range / 2;
                var gll = new Plottable.Components.GuideLineLayer(orientation);
                gll.pixelPosition(expectedPosition1);
                gll.renderTo(div);
                chai_1.assert.strictEqual(gll.content().selectAll(GUIDE_LINE_CLASS).size(), 1, "exactly one line is drawn");
                var line = gll.content().select(GUIDE_LINE_CLASS);
                var expectedAttrs1 = getExpectedAttr(expectedPosition1);
                TestMethods.assertLineAttrs(line, expectedAttrs1, "the line was drawn at the specified position");
                var expectedPosition2 = range * 3 / 4;
                gll.pixelPosition(expectedPosition2);
                chai_1.assert.strictEqual(gll.content().selectAll(GUIDE_LINE_CLASS).size(), 1, "exactly one line is drawn");
                var line2 = gll.content().select(GUIDE_LINE_CLASS);
                var expectedAttrs2 = getExpectedAttr(expectedPosition2);
                TestMethods.assertLineAttrs(line2, expectedAttrs2, "the line was drawn at the updated position");
                div.remove();
            });
            it("renders correctly given a value and scale", function () {
                var gll = new Plottable.Components.GuideLineLayer(orientation);
                gll.renderTo(div);
                var scale = new Plottable.Scales.Linear();
                scale.domain([0, 10]);
                gll.scale(scale);
                var value1 = 5;
                gll.value(value1);
                chai_1.assert.strictEqual(gll.content().selectAll(GUIDE_LINE_CLASS).size(), 1, "exactly one line is drawn");
                var line = gll.content().select(GUIDE_LINE_CLASS);
                var expectedAttrs1 = getExpectedAttr(scale.scale(value1));
                TestMethods.assertLineAttrs(line, expectedAttrs1, "the line was drawn at the correct position");
                var value2 = 8;
                gll.value(value2);
                chai_1.assert.strictEqual(gll.content().selectAll(GUIDE_LINE_CLASS).size(), 1, "exactly one line is drawn");
                var line2 = gll.content().select(GUIDE_LINE_CLASS);
                var expectedAttrs2 = getExpectedAttr(scale.scale(value2));
                TestMethods.assertLineAttrs(line2, expectedAttrs2, "the line was redrawn at the new position when the value was changed");
                div.remove();
            });
            it("re-renders correctly when the scale is updated", function () {
                var gll = new Plottable.Components.GuideLineLayer(orientation);
                var value = 5;
                gll.value(value);
                gll.renderTo(div);
                var scale1 = new Plottable.Scales.Linear();
                scale1.domain([0, 10]);
                gll.scale(scale1);
                var line = gll.content().select(GUIDE_LINE_CLASS);
                var expectedAttrs1 = getExpectedAttr(scale1.scale(value));
                TestMethods.assertLineAttrs(line, expectedAttrs1, "the line was drawn at the correct position");
                scale1.domain([0, 20]);
                var expectedAttrs1b = getExpectedAttr(scale1.scale(value));
                TestMethods.assertLineAttrs(line, expectedAttrs1b, "the line was redrawn at the correct position on domain change");
                var scale2 = new Plottable.Scales.Linear();
                scale2.domain([0, 100]);
                gll.scale(scale2);
                var line2 = gll.content().select(GUIDE_LINE_CLASS);
                var expectedAttrs2 = getExpectedAttr(scale2.scale(value));
                TestMethods.assertLineAttrs(line2, expectedAttrs2, "the line was redrawn at the new position when the scale was changed");
                div.remove();
            });
            it("sets the scale's range based on the allocated " + (orientation === "vertical" ? "width" : "height"), function () {
                var gll = new Plottable.Components.GuideLineLayer(orientation);
                var scale1 = new Plottable.Scales.Linear();
                var expectedRange = orientation === "vertical" ? [0, DIV_WIDTH] : [DIV_HEIGHT, 0];
                gll.scale(scale1);
                gll.renderTo(div);
                chai_1.assert.deepEqual(gll.scale().range(), expectedRange, "range was set based on the allocated space");
                var scale2 = new Plottable.Scales.Linear();
                gll.scale(scale2);
                chai_1.assert.deepEqual(gll.scale().range(), expectedRange, "replacement scale has its range set based on the allocated space");
                div.remove();
            });
        });
    });
});
