"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("InterpolatedColorLegend", function () {
    var SWATCH_SELECTOR = ".swatch";
    var SWATCH_CONTAINER_SELECTOR = ".swatch-container";
    var SWATCH_BBOX_SELECTOR = ".swatch-bounding-box";
    var ORIENTATIONS = ["horizontal", "left", "right"];
    describe("Basic Usage", function () {
        var DIV_HEIGHT = 400;
        var DIV_WIDTH = 400;
        var div;
        var colorScale;
        var legend;
        beforeEach(function () {
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            colorScale = new Plottable.Scales.InterpolatedColor();
            legend = new Plottable.Components.InterpolatedColorLegend(colorScale);
        });
        function assertBasicRendering() {
            var scaleDomain = colorScale.domain();
            var legendContent = legend.content();
            var swatchesNodes = legendContent.selectAll(SWATCH_SELECTOR).nodes();
            chai_1.assert.strictEqual(d3.select(swatchesNodes[0]).attr("fill"), colorScale.scale(scaleDomain[0]), "first swatch's color corresponds with first domain value");
            chai_1.assert.strictEqual(d3.select(swatchesNodes[swatchesNodes.length - 1]).attr("fill"), colorScale.scale(scaleDomain[1]), "last swatch's color corresponds with second domain value");
            var defaultNumSwatches = Plottable.Components.InterpolatedColorLegend._DEFAULT_NUM_SWATCHES;
            chai_1.assert.operator(swatchesNodes.length, ">=", defaultNumSwatches, "there are at least 11 swatches");
            var swatchContainer = legendContent.select(SWATCH_CONTAINER_SELECTOR);
            var swatchContainerBCR = swatchContainer.node().getBoundingClientRect();
            var swatchBoundingBox = legendContent.select(SWATCH_BBOX_SELECTOR);
            var boundingBoxBCR = swatchBoundingBox.node().getBoundingClientRect();
            chai_1.assert.isTrue(Plottable.Utils.DOM.clientRectInside(swatchContainerBCR, boundingBoxBCR), "bounding box contains all swatches");
            var elementBCR = legendContent.node().getBoundingClientRect();
            chai_1.assert.isTrue(Plottable.Utils.DOM.clientRectInside(swatchContainerBCR, elementBCR), "swatches are drawn within the legend's element");
            var formattedDomainValues = scaleDomain.map(legend.formatter());
            var labels = legendContent.selectAll("text");
            var labelTexts = labels.nodes().map(function (textNode) { return textNode.textContent; });
            chai_1.assert.deepEqual(labelTexts, formattedDomainValues, "formatter is used to format label text");
        }
        it("renders correctly when horizontal", function () {
            legend.renderTo(div);
            assertBasicRendering();
            var legendContent = legend.content();
            var labels = legendContent.selectAll("text");
            var swatchContainer = legendContent.select(SWATCH_CONTAINER_SELECTOR);
            var swatchContainerBCR = swatchContainer.node().getBoundingClientRect();
            var lowerLabelBCR = labels.nodes()[0].getBoundingClientRect();
            var upperLabelBCR = labels.nodes()[1].getBoundingClientRect();
            chai_1.assert.operator(lowerLabelBCR.right, "<=", swatchContainerBCR.left, "first label to left of swatches");
            chai_1.assert.operator(swatchContainerBCR.right, "<=", upperLabelBCR.left, "second label to right of swatches");
            div.remove();
        });
        it("renders correctly when oriented right", function () {
            legend.orientation("right");
            legend.renderTo(div);
            assertBasicRendering();
            var legendContent = legend.content();
            var labels = legendContent.selectAll("text");
            var swatchContainer = legendContent.select(SWATCH_CONTAINER_SELECTOR);
            var swatchContainerBCR = swatchContainer.node().getBoundingClientRect();
            var lowerLabelBCR = (labels.nodes()[0]).getBoundingClientRect();
            var upperLabelBCR = (labels.nodes()[1]).getBoundingClientRect();
            chai_1.assert.operator(swatchContainerBCR.right, "<=", lowerLabelBCR.left, "first label to right of swatches");
            chai_1.assert.operator(swatchContainerBCR.right, "<=", upperLabelBCR.left, "second label to right of swatches");
            chai_1.assert.operator(upperLabelBCR.bottom, "<=", lowerLabelBCR.top, "lower label is drawn below upper label");
            div.remove();
        });
        it("renders correctly when oriented left", function () {
            legend.orientation("left");
            legend.renderTo(div);
            assertBasicRendering();
            var legendContent = legend.content();
            var labels = legendContent.selectAll("text");
            var swatchContainer = legendContent.select(SWATCH_CONTAINER_SELECTOR);
            var swatchContainerBCR = swatchContainer.node().getBoundingClientRect();
            var lowerLabelBCR = (labels.nodes()[0]).getBoundingClientRect();
            var upperLabelBCR = (labels.nodes()[1]).getBoundingClientRect();
            chai_1.assert.operator(lowerLabelBCR.left, "<=", swatchContainerBCR.left, "first label to left of swatches");
            chai_1.assert.operator(upperLabelBCR.left, "<=", swatchContainerBCR.left, "second label to left of swatches");
            chai_1.assert.operator(upperLabelBCR.bottom, "<=", lowerLabelBCR.top, "lower label is drawn below upper label");
            div.remove();
        });
        it("does not crash when font-size is 0px", function () {
            legend.renderTo(div);
            var style = legend.content().append("style");
            style.attr("type", "text/css");
            style.text(".plottable .interpolated-color-legend text { font-size: 0px; }" +
                ".plottable .interpolated-color-legend { display: none; }");
            var textHeight = legend._measurer.measure().height;
            chai_1.assert.doesNotThrow(function () { return legend.expands(true); }, Error, "it does not throw error when text height is 0");
            chai_1.assert.strictEqual(textHeight, 0, "text height is set to 0");
            style.remove();
            div.remove();
        });
        it("re-renders when scale domain updates", function () {
            legend.orientation("horizontal");
            legend.renderTo(div);
            colorScale.domain([0, 85]);
            assertBasicRendering();
            div.remove();
        });
        it("checks input of orientation", function () {
            chai_1.assert.doesNotThrow(function () { return legend.orientation("horizontal"); }, "can set orientation to horizontal");
            chai_1.assert.doesNotThrow(function () { return legend.orientation("right"); }, "can set orientation to right");
            chai_1.assert.doesNotThrow(function () { return legend.orientation("left"); }, "can set orientation to left");
            // HACKHACK #2661: Cannot assert errors being thrown with description
            chai_1.assert.throws(function () { return legend.orientation("blargh"); }, Error, "not a valid orientation", "error on invalid orientation");
            div.remove();
        });
        it("triggers layout computation on orientation change", function () {
            legend.renderTo(div);
            var widthBefore = legend.width();
            var heightBefore = legend.height();
            legend.orientation("right");
            chai_1.assert.notEqual(legend.width(), widthBefore, "proportions changed (width)");
            chai_1.assert.notEqual(legend.height(), heightBefore, "proportions changed (height)");
            div.remove();
        });
        ORIENTATIONS.forEach(function (orientation) {
            ["width", "height"].forEach(function (constrain) {
                it("renders correctly when " + constrain + " is constrained in " + orientation + " orientation", function () {
                    var value;
                    if (orientation === "horizontal") {
                        value = constrain === "width" ? 100 : 35;
                    }
                    else {
                        value = constrain === "width" ? 45 : 100;
                    }
                    div.style(constrain, value + "px");
                    legend.orientation(orientation);
                    legend.renderTo(div);
                    assertBasicRendering();
                    div.remove();
                });
            });
        });
        it("has the same number of swatches as its height when vertical", function () {
            legend.renderTo(div);
            legend.orientation("left").expands(false);
            var numSwatches = legend.content().selectAll(SWATCH_SELECTOR).size();
            var swatchContainer = legend.content().select(SWATCH_CONTAINER_SELECTOR);
            var swatchContainerHeight = swatchContainer.node().getBoundingClientRect().height;
            legend.expands(true);
            var newNumSwatches = legend.content().selectAll(SWATCH_SELECTOR).size();
            var swatchContainerExpandedHeight = swatchContainer.node().getBoundingClientRect().height;
            chai_1.assert.closeTo(numSwatches, swatchContainerHeight, 1, "non-expanded left legend has one swatch per pixel");
            chai_1.assert.closeTo(newNumSwatches, swatchContainerExpandedHeight, 1, "expanded left legend has one swatch per pixel");
            div.remove();
        });
        it("has the same number of swatches as its width when horizontal", function () {
            legend.renderTo(div);
            legend.orientation("horizontal").expands(false);
            var numSwatches = legend.content().selectAll(SWATCH_SELECTOR).size();
            var swatchContainer = legend.content().select(SWATCH_CONTAINER_SELECTOR);
            var swatchContainerWidth = swatchContainer.node().getBoundingClientRect().width;
            legend.expands(true);
            var newNumSwatches = legend.content().selectAll(SWATCH_SELECTOR).size();
            var swatchContainerExpandedWidth = swatchContainer.node().getBoundingClientRect().width;
            chai_1.assert.closeTo(numSwatches, swatchContainerWidth, 1, "non-expanded left legend has one swatch per pixel");
            chai_1.assert.closeTo(newNumSwatches, swatchContainerExpandedWidth, 1, "expanded left legend has one swatch per pixel");
            div.remove();
        });
    });
    describe("Expanding", function () {
        var DIV_HEIGHT = 400;
        var DIV_WIDTH = 400;
        var div;
        var colorScale;
        var legend;
        beforeEach(function () {
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            colorScale = new Plottable.Scales.InterpolatedColor();
            legend = new Plottable.Components.InterpolatedColorLegend(colorScale);
        });
        it("fixes height if expand is set to false or orientation is horizontal", function () {
            legend.renderTo(div);
            chai_1.assert.isTrue(legend.fixedHeight(), "height is fixed on default");
            legend.expands(true);
            chai_1.assert.isTrue(legend.fixedHeight(), "height is fixed oriented horizontally");
            legend.orientation("left");
            chai_1.assert.isFalse(legend.fixedHeight(), "height is not fixed oriented vertically");
            legend.orientation("right");
            chai_1.assert.isFalse(legend.fixedHeight(), "height is not fixed oriented vertically");
            legend.expands(false);
            chai_1.assert.isTrue(legend.fixedHeight(), "height is fixed when expand is set to false");
            div.remove();
        });
        it("fixes width if expand is set to false or orientation is vertically", function () {
            legend.renderTo(div);
            chai_1.assert.isTrue(legend.fixedWidth(), "width is fixed on default");
            legend.expands(true);
            chai_1.assert.isFalse(legend.fixedWidth(), "width is not fixed oriented horizontally");
            legend.orientation("left");
            chai_1.assert.isTrue(legend.fixedWidth(), "width is fixed oriented vertically");
            legend.orientation("right");
            chai_1.assert.isTrue(legend.fixedWidth(), "width is fixed oriented vertically");
            legend.expands(false);
            legend.orientation("horizontal");
            chai_1.assert.isTrue(legend.fixedWidth(), "width is fixed when expand is set to false");
            div.remove();
        });
        it("spams the entire height if oriented vertically and expand is set to true", function () {
            legend.orientation("left");
            legend.expands(true);
            legend.renderTo(div);
            chai_1.assert.strictEqual(legend.height(), DIV_HEIGHT, "legend height is the same as div height");
            div.remove();
        });
        it("spams the entire width if oriented horizontally and expand is set to true", function () {
            legend.expands(true);
            legend.renderTo(div);
            chai_1.assert.strictEqual(legend.width(), DIV_WIDTH, "legend width is the same as div width");
            div.remove();
        });
        it("has more swatches than default when expand is true", function () {
            legend.renderTo(div);
            ORIENTATIONS.forEach(function (orientation) {
                legend.orientation(orientation).expands(false);
                var numSwatches = legend.content().selectAll(SWATCH_SELECTOR).size();
                legend.expands(true);
                var newNumSwatches = legend.content().selectAll(SWATCH_SELECTOR).size();
                chai_1.assert.operator(newNumSwatches, ">", numSwatches, "there are more swatches when expanded (orientation: " + orientation + ")");
            });
            div.remove();
        });
    });
    describe("Padding", function () {
        var DIV_HEIGHT = 400;
        var DIV_WIDTH = 400;
        var div;
        var colorScale;
        var legend;
        beforeEach(function () {
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            colorScale = new Plottable.Scales.InterpolatedColor();
            legend = new Plottable.Components.InterpolatedColorLegend(colorScale);
        });
        it("does not have padding on ends of vertical legends", function () {
            legend.renderTo(div);
            var orientations = ["left", "right"];
            orientations.forEach(function (orientation) {
                legend.orientation(orientation).expands(true);
                var height = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect().height;
                chai_1.assert.closeTo(height, DIV_HEIGHT, window.Pixel_CloseTo_Requirement, "actual height is DIV_HEIGHT with orientation: " + orientation);
            });
            div.remove();
        });
        it("pads left-oriented legends correctly", function () {
            legend.orientation("left");
            legend.renderTo(div);
            var swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            var legendBoundingRect = legend.element().node().getBoundingClientRect();
            var swatchWidth = swatchBoundingRect.width;
            var swatchEdge = swatchBoundingRect.right;
            var legendEdge = legendBoundingRect.right;
            var padding = legendEdge - swatchEdge;
            chai_1.assert.closeTo(swatchWidth, padding, window.Pixel_CloseTo_Requirement, "padding is approximately equal to swatch width");
            legend.expands(true);
            swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            legendBoundingRect = legend.element().node().getBoundingClientRect();
            swatchWidth = swatchBoundingRect.width;
            swatchEdge = swatchBoundingRect.right;
            legendEdge = legendBoundingRect.right;
            padding = legendEdge - swatchEdge;
            chai_1.assert.closeTo(swatchWidth, padding, window.Pixel_CloseTo_Requirement, "padding is approximately equal to swatch width when expanded");
            div.remove();
        });
        it("pads right-oriented legends correctly", function () {
            legend.orientation("right");
            legend.renderTo(div);
            var swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            var legendBoundingRect = legend.element().node().getBoundingClientRect();
            var swatchWidth = swatchBoundingRect.width;
            var swatchEdge = swatchBoundingRect.left;
            var legendEdge = legendBoundingRect.left;
            var padding = swatchEdge - legendEdge;
            // HACKHACK #2122: two measurement errors in IE combine, and total error = 2
            chai_1.assert.closeTo(swatchWidth, padding, 2, "padding is approximately equal to swatch width");
            legend.expands(true);
            swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            legendBoundingRect = legend.element().node().getBoundingClientRect();
            swatchWidth = swatchBoundingRect.width;
            swatchEdge = swatchBoundingRect.left;
            legendEdge = legendBoundingRect.left;
            padding = swatchEdge - legendEdge;
            // HACKHACK #2122: two measurement errors in IE combine, and total error = 2
            chai_1.assert.closeTo(swatchWidth, padding, 2, "padding is approximately equal to swatch width when expanded");
            div.remove();
        });
        it("pads horizontal legends correctly", function () {
            legend.orientation("horizontal");
            legend.renderTo(div);
            var swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            var legendBoundingRect = legend.element().node().getBoundingClientRect();
            var swatchHeight = swatchBoundingRect.height;
            var swatchEdge = swatchBoundingRect.bottom;
            var legendEdge = legendBoundingRect.bottom;
            var padding = legendEdge - swatchEdge;
            chai_1.assert.closeTo(swatchHeight, padding, window.Pixel_CloseTo_Requirement, "padding is approximately equal to swatch height");
            legend.expands(true);
            swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            legendBoundingRect = legend.element().node().getBoundingClientRect();
            swatchHeight = swatchBoundingRect.height;
            swatchEdge = swatchBoundingRect.bottom;
            legendEdge = legendBoundingRect.bottom;
            padding = legendEdge - swatchEdge;
            chai_1.assert.closeTo(swatchHeight, padding, window.Pixel_CloseTo_Requirement, "padding is approximately equal to swatch height when expanded");
            div.remove();
        });
    });
    describe("Constrained situations", function () {
        var div;
        var colorScale;
        var legend;
        beforeEach(function () {
            div = TestMethods.generateDiv();
            colorScale = new Plottable.Scales.InterpolatedColor();
            legend = new Plottable.Components.InterpolatedColorLegend(colorScale);
        });
        it("degrades padding when height is constrained in horizontal orientation", function () {
            legend.orientation("horizontal");
            legend.renderTo(div);
            var swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            var unconstrainedSwatchHeight = swatchBoundingRect.height;
            var textHeight = unconstrainedSwatchHeight;
            var constrainedHeight = textHeight;
            div.style("height", constrainedHeight + "px");
            legend.redraw();
            var legendBoundingRect = legend.element().node().getBoundingClientRect();
            swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            var topPadding = swatchBoundingRect.top - legendBoundingRect.top;
            var bottomPadding = legendBoundingRect.bottom - swatchBoundingRect.bottom;
            chai_1.assert.operator(topPadding, "<", textHeight, "top padding degrades to be smaller than textHeight");
            chai_1.assert.operator(bottomPadding, "<", textHeight, "bottom padding degrades to be smaller than textHeight");
            div.remove();
        });
        it("degrades padding when width is constrained in left orientation", function () {
            legend.orientation("left");
            legend.renderTo(div);
            var swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            var unconstrainedSwatchWidth = swatchBoundingRect.width;
            var textHeight = unconstrainedSwatchWidth;
            var constrainedWidth = textHeight * 2;
            div.style("width", constrainedWidth + "px");
            legend.redraw();
            var legendBoundingRect = legend.element().node().getBoundingClientRect();
            swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            var leftPadding = legendBoundingRect.right - swatchBoundingRect.right;
            chai_1.assert.operator(leftPadding, "<", textHeight, "right-side padding degrades to be smaller than textHeight");
            div.remove();
        });
        it("degrades padding when width is constrained in right orientation", function () {
            legend.orientation("right");
            legend.renderTo(div);
            var swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            var unconstrainedSwatchWidth = swatchBoundingRect.width;
            var textHeight = unconstrainedSwatchWidth;
            var constrainedWidth = textHeight * 2;
            div.style("width", constrainedWidth + "px");
            legend.redraw();
            var legendBoundingRect = legend.element().node().getBoundingClientRect();
            swatchBoundingRect = legend.content().select(SWATCH_CONTAINER_SELECTOR).node().getBoundingClientRect();
            var leftPadding = swatchBoundingRect.left - legendBoundingRect.left;
            chai_1.assert.operator(leftPadding, "<", textHeight, "left-side padding degrades to be smaller than textHeight");
            div.remove();
        });
        it("degrades gracefully when width is very constrained", function () {
            legend.orientation("horizontal");
            legend.renderTo(div);
            var constrainedWidth = 30;
            div.style("width", constrainedWidth + "px");
            chai_1.assert.doesNotThrow(function () { return legend.redraw(); }, Error, "rendering in a small space should not error");
            var numSwatches = legend.content().selectAll(SWATCH_SELECTOR).size();
            chai_1.assert.strictEqual(0, numSwatches, "no swatches are drawn");
            div.remove();
        });
    });
    describe("Title Element", function () {
        var div;
        var colorScale;
        var legend;
        beforeEach(function () {
            div = TestMethods.generateDiv();
            colorScale = new Plottable.Scales.InterpolatedColor();
            legend = new Plottable.Components.InterpolatedColorLegend(colorScale);
        });
        it("adds formatted title element to each swatch", function () {
            var formatter = Plottable.Formatters.percentage(2);
            legend.formatter(formatter);
            legend.renderTo(div);
            var entries = legend.content().selectAll("rect.swatch");
            chai_1.assert.operator(entries.size(), ">=", 11, "there is at least 11 swatches");
            entries.each(function (d, i) {
                var swatch = d3.select(this);
                chai_1.assert.strictEqual(swatch.select("title").text(), formatter(d));
            });
            div.remove();
        });
        it("does not create title elements if configuration is set to false", function () {
            var originalSetting = Plottable.Configs.ADD_TITLE_ELEMENTS;
            Plottable.Configs.ADD_TITLE_ELEMENTS = false;
            legend.renderTo(div);
            Plottable.Configs.ADD_TITLE_ELEMENTS = originalSetting;
            var entries = legend.content().selectAll("rect.swatch");
            chai_1.assert.operator(entries.size(), ">=", 11, "there is at least 11 swatches");
            var titles = entries.selectAll("title");
            chai_1.assert.strictEqual(titles.size(), 0, "no titles should be rendered");
            div.remove();
        });
    });
});
