"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
var domUtils_1 = require("../../src/utils/domUtils");
describe("Legend", function () {
    var ENTRY_SELECTOR = "." + Plottable.Components.Legend.LEGEND_ENTRY_CLASS;
    var SYMBOL_SELECTOR = "." + Plottable.Components.Legend.LEGEND_SYMBOL_CLASS;
    var TEXT_LINE_SELECTOR = ".text-line";
    var ROW_SELECTOR = "." + Plottable.Components.Legend.LEGEND_ROW_CLASS;
    describe("Basic Usage", function () {
        var div;
        var color;
        var legend;
        beforeEach(function () {
            div = TestMethods.generateDiv();
            color = new Plottable.Scales.Color();
            legend = new Plottable.Components.Legend(color);
        });
        it("renders rows with correct text, fill, and opacity", function () {
            color.domain(["foo", "bar", "baz"]);
            legend.renderTo(div);
            var rows = legend.content().selectAll(ENTRY_SELECTOR);
            chai_1.assert.strictEqual(rows.size(), color.domain().length, "one entry is created for each item in the domain");
            rows.each(function (d, i) {
                var name = d[0].data.name;
                chai_1.assert.strictEqual(name, color.domain()[i], "the data is set properly");
                var row = d3.select(this);
                var text = row.select("text").text();
                chai_1.assert.strictEqual(text, name, "the text node has correct text");
                var symbol = row.select(SYMBOL_SELECTOR);
                chai_1.assert.strictEqual(symbol.attr("fill"), color.scale(name), "the symbol's fill is set properly");
                chai_1.assert.strictEqual(symbol.attr("opacity"), "1", "the symbol's opacity is set by default to 1");
            });
            div.remove();
        });
        it("updates height when domain is updated", function () {
            legend.renderTo(div);
            legend.colorScale(color);
            var minHeight1 = legend.requestedSpace(200, 200).minHeight;
            chai_1.assert.operator(minHeight1, ">", 0, "there is a padding requested height when domain is empty");
            color.domain(["foo", "bar"]);
            var minHeight2 = legend.requestedSpace(400, 400).minHeight;
            var actualHeight2 = legend.height();
            chai_1.assert.operator(minHeight2, ">", minHeight1, "changing the domain gives a height greater than empty domain");
            color.domain(["foo", "bar", "baz"]);
            var minHeight3 = legend.requestedSpace(400, 400).minHeight;
            chai_1.assert.operator(minHeight3, ">", minHeight2, "adding to the domain increases the height requested");
            var actualHeight3 = legend.height();
            chai_1.assert.operator(actualHeight2, "<", actualHeight3, "Changing the domain caused the legend to re-layout with more height");
            chai_1.assert.strictEqual(legend.content().selectAll(ROW_SELECTOR).size(), 3, "there are 3 rows");
            div.remove();
        });
        it("does not overflow vertically", function () {
            color.domain(["alpha", "beta", "gamma", "delta", "omega", "omicron", "persei", "eight"]);
            legend.renderTo(div);
            var contentBBox = Plottable.Utils.DOM.elementBBox(legend.content());
            var contentBottomEdge = contentBBox.y + contentBBox.height;
            var bboxBBox = legend.element().node().getBoundingClientRect();
            var bboxBottomEdge = bboxBBox.top + bboxBBox.height;
            chai_1.assert.operator(contentBottomEdge, "<=", bboxBottomEdge, "content does not extend past bounding box");
            div.remove();
        });
        it("does not overflow horizontally with long label", function () {
            color.domain(["foooboooloonoogoorooboopoo"]);
            div.style("width", "100px");
            legend.renderTo(div);
            var text = legend.content().select("text").text();
            chai_1.assert.notEqual(text, "foooboooloonoogoorooboopoo", "the text was truncated");
            var rightEdge = legend.content().select("text").node().getBoundingClientRect().right;
            var rightEdgeBBox = legend.element().node().getBoundingClientRect().right;
            chai_1.assert.operator(rightEdge, "<=", rightEdgeBBox, "the long text did not overflow the legend");
            div.remove();
        });
        it("does not add more elements if legend.render is called multiple times", function () {
            color.domain(["foo", "bar", "baz"]);
            legend.renderTo(div);
            chai_1.assert.strictEqual(legend.content().selectAll(ROW_SELECTOR).size(), 3, "there are 3 legend rows initially");
            legend.render();
            chai_1.assert.strictEqual(legend.content().selectAll(ROW_SELECTOR).size(), 3, "there are 3 legend rows after second render");
            div.remove();
        });
        it("updates properties correctly on rerendering after domain is changed", function () {
            color.domain(["foo", "bar", "baz"]);
            legend.renderTo(div);
            var newDomain = ["mushu", "foo", "persei", "baz", "eight"];
            color.domain(newDomain);
            legend.content().selectAll(ENTRY_SELECTOR).each(function (d, i) {
                var name = d[0].data.name;
                chai_1.assert.strictEqual(name, newDomain[i], "the data is set correctly");
                var text = d3.select(this).select("text").text();
                chai_1.assert.strictEqual(text, name, "the text was set properly");
                var fill = d3.select(this).select(SYMBOL_SELECTOR).attr("fill");
                chai_1.assert.strictEqual(fill, color.scale(name), "the fill was set properly");
            });
            chai_1.assert.strictEqual(legend.content().selectAll(ENTRY_SELECTOR).size(), 5, "there are the right number of legend elements");
            div.remove();
        });
        it("updates domain when the scale is replaced", function () {
            color.domain(["foo", "bar", "baz"]);
            legend.renderTo(div);
            var newDomain = ["a", "b", "c"];
            var newColorScale = new Plottable.Scales.Color("20");
            newColorScale.domain(newDomain);
            legend.colorScale(newColorScale);
            legend.content().selectAll(ENTRY_SELECTOR).each(function (d, i) {
                var name = d[0].data.name;
                chai_1.assert.strictEqual(name, newDomain[i], "the data is set correctly");
                var text = d3.select(this).select("text").text();
                chai_1.assert.strictEqual(text, name, "the text was set properly");
                var fill = d3.select(this).select(SYMBOL_SELECTOR).attr("fill");
                chai_1.assert.strictEqual(fill, newColorScale.scale(name), "the fill was set properly");
            });
            div.remove();
        });
        it("reregisters listeners when scale is replaced", function () {
            color.domain(["foo", "bar", "baz"]);
            legend.renderTo(div);
            var tempDomain = ["a", "b", "c"];
            var newColorScale = new Plottable.Scales.Color("20");
            newColorScale.domain(tempDomain);
            legend.colorScale(newColorScale);
            var newDomain = ["a", "foo", "d"];
            newColorScale.domain(newDomain);
            legend.content().selectAll(ENTRY_SELECTOR).each(function (d, i) {
                var name = d[0].data.name;
                chai_1.assert.strictEqual(name, newDomain[i], "the data is set correctly");
                var text = d3.select(this).select("text").text();
                chai_1.assert.strictEqual(text, name, "the text was set properly");
                var fill = d3.select(this).select(SYMBOL_SELECTOR).attr("fill");
                chai_1.assert.strictEqual(fill, newColorScale.scale(name), "the fill was set properly");
            });
            div.remove();
        });
        // Disable this test for now since it flakes all the time on Saucelabs Safari 10
        xit("can set maximum number of lines per entry", function () {
            color.domain(["this is a very very very very very very very long"]);
            legend.renderTo(div);
            legend.maxWidth(100);
            legend.maxLinesPerEntry(2);
            chai_1.assert.strictEqual(2, legend.content().selectAll(ROW_SELECTOR + " " + TEXT_LINE_SELECTOR).size());
            legend.maxLinesPerEntry(4);
            chai_1.assert.strictEqual(4, legend.content().selectAll(ROW_SELECTOR + " " + TEXT_LINE_SELECTOR).size());
            div.remove();
        });
        it("can set maximum width of legend", function () {
            color.domain(["this is a very very very very very very very long"]);
            legend.renderTo(div);
            legend.maxWidth(100);
            chai_1.assert.isTrue(legend.content().node().getBoundingClientRect().width <= 100);
            div.remove();
        });
        // Disable this test for now since it flakes all the time on Saucelabs Safari 10
        xit("can set maximum number of entries per row", function () {
            color.domain(["AA", "BB", "CC", "DD", "EE", "FF"]);
            legend.renderTo(div);
            function verifyMaxEntriesInRow(n) {
                legend.maxEntriesPerRow(n);
                var rows = legend.content().selectAll(ROW_SELECTOR);
                chai_1.assert.strictEqual(rows.size(), (6 / n), "number of rows is correct");
                rows.each(function (d) {
                    var entries = d3.select(this).selectAll(ENTRY_SELECTOR);
                    chai_1.assert.strictEqual(entries.size(), n, "number of entries in row is correct");
                });
            }
            ;
            verifyMaxEntriesInRow(1);
            verifyMaxEntriesInRow(2);
            verifyMaxEntriesInRow(3);
            verifyMaxEntriesInRow(6);
            div.remove();
        });
        it("wraps entries onto extra rows if necessary for horizontal legends", function () {
            color.domain(["George Waaaaaashington", "John Adaaaams", "Thomaaaaas Jefferson"]);
            legend.maxEntriesPerRow(Infinity);
            legend.renderTo(div);
            var rows = legend.content().selectAll(ROW_SELECTOR);
            chai_1.assert.strictEqual(rows.size(), 2, "Wrapped text on to two rows when space is constrained");
            legend.detach();
            div.remove();
            div = TestMethods.generateDiv(100, 100);
            legend.renderTo(div);
            rows = legend.content().selectAll(ROW_SELECTOR);
            chai_1.assert.strictEqual(rows.size(), 3, "Wrapped text on to three rows when further constrained");
            div.remove();
        });
        it("truncates and hides entries if space is constrained for a horizontal legend", function () {
            div.remove();
            div = TestMethods.generateDiv(70, 400);
            legend.maxEntriesPerRow(Infinity);
            legend.renderTo(div);
            var textEls = legend.content().selectAll("text");
            textEls.each(function (d) {
                var textEl = d3.select(this);
                TestMethods.assertBBoxInclusion(legend.content(), textEl);
            });
            legend.detach();
            div.remove();
            div = TestMethods.generateDiv(100, 50);
            legend.renderTo(div);
            textEls = legend.content().selectAll("text");
            textEls.each(function (d) {
                var textEl = d3.select(this);
                TestMethods.assertBBoxInclusion(legend.content(), textEl);
            });
            div.remove();
        });
    });
    describe("Symbols", function () {
        var div;
        var color;
        var legend;
        beforeEach(function () {
            div = TestMethods.generateDiv();
            color = new Plottable.Scales.Color();
            legend = new Plottable.Components.Legend(color);
        });
        it("renders with correct opacity for each symbol when specified", function () {
            var opacity = 0.5;
            color.domain(["foo", "bar", "baz"]);
            legend.symbolOpacity(opacity);
            legend.renderTo(div);
            var rows = legend.content().selectAll(ENTRY_SELECTOR);
            rows.each(function (d, i) {
                var row = d3.select(this);
                var symbol = row.select(SYMBOL_SELECTOR);
                chai_1.assert.strictEqual(TestMethods.numAttr(symbol, "opacity"), opacity, "the symbol's opacity is set to a constant");
            });
            var opacityFunction = function (d, i) {
                return (d === "foo") ? 0.2 : 0.8;
            };
            legend.symbolOpacity(opacityFunction).redraw();
            rows = legend.content().selectAll(ENTRY_SELECTOR);
            rows.each(function (d, i) {
                var name = d[0].data.name;
                var row = d3.select(this);
                var symbol = row.select(SYMBOL_SELECTOR);
                chai_1.assert.strictEqual(symbol.attr("opacity"), String(opacityFunction(name, i)), "the symbol's opacity follows the provided function.");
            });
            div.remove();
        });
        it("scales icon sizes properly with font size", function () {
            color.domain(["foo"]);
            legend.renderTo(div);
            var style = div.append("style");
            style.attr("type", "text/css");
            function verifySymbolHeight() {
                var text = legend.content().select("text");
                var icon = legend.content().select(SYMBOL_SELECTOR);
                var textHeight = Plottable.Utils.DOM.elementBBox(text).height;
                var symbolHeight = icon.node().getBoundingClientRect().height;
                chai_1.assert.operator(symbolHeight, "<", textHeight, "icons too small: symbolHeight < textHeight");
                chai_1.assert.operator(symbolHeight, ">", textHeight / 2, "icons too big: textHeight / 2 > symbolHeight");
            }
            verifySymbolHeight();
            style.text(".plottable .legend text { font-size: 60px; }");
            legend.computeLayout();
            legend.render();
            verifySymbolHeight();
            style.text(".plottable .legend text { font-size: 10px; }");
            legend.computeLayout();
            legend.render();
            verifySymbolHeight();
            div.remove();
        });
        it("passes correct index in symbol", function () {
            var domain = ["AA", "BB", "CC"];
            color.domain(domain);
            var expectedIndex = 0;
            var symbolChecker = function (d, index) {
                chai_1.assert.strictEqual(index, expectedIndex, "index passed in is correct");
                expectedIndex++;
                return function (size) { return ""; };
            };
            legend.symbol(symbolChecker);
            legend.renderTo(div);
            div.remove();
        });
        it("passes correct index in symbolOpacity", function () {
            var domain = ["AA", "BB", "CC"];
            color.domain(domain);
            var expectedIndex = 0;
            var symbolOpacityChecker = function (d, index) {
                chai_1.assert.strictEqual(index, expectedIndex, "index passed in is correct");
                expectedIndex++;
                return 0.5;
            };
            legend.symbolOpacity(symbolOpacityChecker);
            legend.renderTo(div);
            div.remove();
        });
    });
    describe("Title Elements", function () {
        var div;
        var color;
        var legend;
        beforeEach(function () {
            div = TestMethods.generateDiv();
            color = new Plottable.Scales.Color();
            legend = new Plottable.Components.Legend(color);
        });
        it("create title elements by default", function () {
            color.domain(["foo", "bar", "baz"]);
            legend.renderTo(div);
            var entries = legend.content().selectAll(ENTRY_SELECTOR);
            var titles = entries.selectAll("title");
            chai_1.assert.strictEqual(titles.size(), color.domain().length, "same number of title tags as legend entries");
            entries.each(function (d, i) {
                var entry = d3.select(this);
                var text = entry.select("text").text();
                var title = entry.selectAll("title");
                chai_1.assert.strictEqual(title.size(), 1, "only one title node per legend entry should be present");
                chai_1.assert.strictEqual(text, title.text(), "the text and title node have the same text");
            });
            div.remove();
        });
        it("does not create title elements if configuration is set to false", function () {
            color.domain(["foo", "bar", "baz"]);
            var originalSetting = Plottable.Configs.ADD_TITLE_ELEMENTS;
            Plottable.Configs.ADD_TITLE_ELEMENTS = false;
            legend.renderTo(div);
            Plottable.Configs.ADD_TITLE_ELEMENTS = originalSetting;
            var entries = legend.content().selectAll(ENTRY_SELECTOR);
            var titles = entries.selectAll("title");
            chai_1.assert.strictEqual(titles.size(), 0, "no titles should be rendered");
            div.remove();
        });
    });
    describe("Formatting and Sorting", function () {
        var div;
        var color;
        var legend;
        beforeEach(function () {
            div = TestMethods.generateDiv();
            color = new Plottable.Scales.Color();
            legend = new Plottable.Components.Legend(color);
        });
        it("can set formatter to change how entry labels are displayed", function () {
            color.domain(["A", "B", "C"]);
            var formatter = function (id) { return id + "foo"; };
            legend.formatter(formatter);
            legend.renderTo(div);
            var legendRows = div.selectAll(ENTRY_SELECTOR);
            chai_1.assert.operator(legendRows.size(), ">=", 1, "There is at least one entry in the test");
            legendRows.each(function (d, i) {
                var expectedText = formatter(legend.colorScale().domain()[i]);
                chai_1.assert.strictEqual(d3.select(this).select("text").text(), expectedText, "formatter output " + expectedText + " should be displayed");
            });
            div.remove();
        });
        it("can get formatter of the legend", function () {
            var formatter = function (id) { return id + "foo"; };
            legend.formatter(formatter);
            chai_1.assert.strictEqual(legend.formatter(), formatter, "formatter() returns the formatter of legend correctly");
            div.remove();
        });
        it("can sort displayed texts using comparator", function () {
            var colorDomain = ["A", "B", "C"];
            color.domain(colorDomain);
            var expectedTexts = ["Z", "Y", "X"];
            var formatter = function (d) { return expectedTexts[colorDomain.indexOf(d)]; };
            legend.formatter(formatter);
            var comparator = function (a, b) { return a.charCodeAt(0) - b.charCodeAt(0); };
            legend.comparator(comparator);
            legend.renderTo(div);
            var entryTexts = div.selectAll(ENTRY_SELECTOR).nodes().map(function (node) { return d3.select(node).select("text").text(); });
            expectedTexts.sort(comparator);
            chai_1.assert.deepEqual(expectedTexts, entryTexts, "displayed texts should be sorted in alphabetic order");
            div.remove();
        });
        it("does not change the order of legend entries when using default comparator", function () {
            var colorDomain = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k"];
            color.domain(colorDomain);
            legend.renderTo(div);
            var entryTexts = legend.content().selectAll(ENTRY_SELECTOR).data().map(function (datum) { return datum[0].data.name; });
            chai_1.assert.deepEqual(colorDomain, entryTexts, "displayed texts should have the same order as the legend domain");
            div.remove();
        });
        it("sorts entries by comparator", function () {
            var newDomain = ["F", "E", "D", "C", "B", "A"];
            color.domain(newDomain);
            legend.renderTo(div);
            var entries = legend.content().selectAll(ENTRY_SELECTOR);
            var elementTexts = entries.select("text").nodes().map(function (node) { return d3.select(node).text(); });
            chai_1.assert.deepEqual(elementTexts, newDomain, "entry has not been sorted");
            var compareFunction = function (a, b) { return a.localeCompare(b); };
            legend.comparator(compareFunction);
            entries = legend.content().selectAll(ENTRY_SELECTOR);
            elementTexts = entries.select("text").nodes().map(function (node) { return d3.select(node).text(); });
            newDomain.sort(compareFunction);
            chai_1.assert.deepEqual(elementTexts, newDomain, "entry has been sorted alphabetically");
            div.remove();
        });
    });
    describe("Selection", function () {
        var div;
        var color;
        var legend;
        beforeEach(function () {
            div = TestMethods.generateDiv();
            color = new Plottable.Scales.Color();
            legend = new Plottable.Components.Legend(color);
        });
        function computeExpectedSymbolPosition(rowIndex, entryIndexWithinRow) {
            var row = d3.select(legend.content().selectAll(ROW_SELECTOR).nodes()[rowIndex]);
            var entry = d3.select(row.selectAll(ENTRY_SELECTOR).nodes()[entryIndexWithinRow]);
            var symbol = entry.select(SYMBOL_SELECTOR);
            var rowTranslate = domUtils_1.getTranslateValues(row);
            var entryTranslate = domUtils_1.getTranslateValues(entry);
            var symbolTranslate = domUtils_1.getTranslateValues(symbol);
            return {
                x: rowTranslate[0] + entryTranslate[0] + symbolTranslate[0],
                y: rowTranslate[1] + entryTranslate[1] + symbolTranslate[1],
            };
        }
        it("gets Entities representing the entry at a particular point", function () {
            var domain = ["AA", "BB", "CC"];
            color.domain(domain);
            legend.renderTo(div);
            var entities = legend.entitiesAt({ x: 10, y: 10 });
            var entries = legend.content().selectAll(ENTRY_SELECTOR);
            var expectedEntity = {
                datum: "AA",
                position: computeExpectedSymbolPosition(0, 0),
                selection: d3.select(entries.nodes()[0]),
                component: legend,
            };
            TestMethods.assertEntitiesEqual(entities[0], expectedEntity, "returned Entity corresponding to first entry");
            entities = legend.entitiesAt({ x: 10, y: 30 });
            expectedEntity = {
                datum: "BB",
                position: computeExpectedSymbolPosition(1, 0),
                selection: d3.select(entries.nodes()[1]),
                component: legend,
            };
            TestMethods.assertEntitiesEqual(entities[0], expectedEntity, "returned Entity corresponding to second entry");
            legend.detach();
            entities = legend.entitiesAt({ x: 10, y: 10 });
            chai_1.assert.lengthOf(entities, 0, "returns no Entities if not anchored");
            div.remove();
        });
        it("gets Entities representing the entry at a particular point if maxEntriesPerRow is greater than 1", function () {
            var domain = ["AA", "BB", "CC"];
            color.domain(domain);
            legend.maxEntriesPerRow(Infinity);
            legend.renderTo(div);
            var entities = legend.entitiesAt({ x: 10, y: 10 });
            var entries = legend.content().selectAll(ENTRY_SELECTOR);
            var expectedEntity = {
                datum: "AA",
                position: computeExpectedSymbolPosition(0, 0),
                selection: d3.select(entries.nodes()[0]),
                component: legend,
            };
            TestMethods.assertEntitiesEqual(entities[0], expectedEntity, "returned Entity corresponding to first entry");
            entities = legend.entitiesAt({ x: 50, y: 10 });
            expectedEntity = {
                datum: "BB",
                position: computeExpectedSymbolPosition(0, 1),
                selection: d3.select(entries.nodes()[1]),
                component: legend,
            };
            TestMethods.assertEntitiesEqual(entities[0], expectedEntity, "returned Entity corresponding to second entry");
            div.remove();
        });
        it("returns an empty array if no Entitites are present at that point", function () {
            var domain = ["AA", "BB", "CC"];
            color.domain(domain);
            legend.renderTo(div);
            var entities = legend.entitiesAt({ x: -100, y: -100 });
            chai_1.assert.lengthOf(entities, 0, "no Entities returned if there are no entries at that point");
            div.remove();
        });
    });
});
