"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("SelectionBoxLayer", function () {
    describe("XDragBoxLayer", function () {
        describe("Basic Usage", function () {
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 300;
            var quarterTopLeftPoint = {
                x: DIV_WIDTH / 4,
                y: DIV_HEIGHT / 4,
            };
            var middlePoint = {
                x: DIV_WIDTH / 2,
                y: DIV_HEIGHT / 2,
            };
            var quarterBottomRightPoint = {
                x: DIV_WIDTH * 3 / 4,
                y: DIV_HEIGHT * 3 / 4,
            };
            var div;
            var dbl;
            beforeEach(function () {
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                dbl = new Plottable.Components.XDragBoxLayer();
            });
            it("has bounds that extend across full SVG height", function () {
                dbl.boxVisible(true);
                dbl.renderTo(div);
                dbl.bounds({
                    topLeft: quarterTopLeftPoint,
                    bottomRight: middlePoint,
                });
                var actualBounds = dbl.bounds();
                chai_1.assert.strictEqual(actualBounds.topLeft.y, 0, "box starts at top");
                chai_1.assert.strictEqual(actualBounds.topLeft.x, quarterTopLeftPoint.x, "left edge set correctly");
                chai_1.assert.strictEqual(actualBounds.bottomRight.y, DIV_HEIGHT, "box ends at bottom");
                chai_1.assert.strictEqual(actualBounds.bottomRight.x, middlePoint.x, "right edge set correctly");
                div.remove();
            });
            it("resizes only in x", function () {
                dbl.boxVisible(true);
                dbl.resizable(true);
                dbl.renderTo(div);
                var bounds = { topLeft: quarterTopLeftPoint, bottomRight: middlePoint };
                dbl.bounds(bounds);
                TestMethods.triggerFakeDragSequence(dbl.background(), bounds.bottomRight, quarterBottomRightPoint);
                var actualBounds = dbl.bounds();
                chai_1.assert.strictEqual(actualBounds.bottomRight.x, quarterBottomRightPoint.x, "resized in x");
                chai_1.assert.strictEqual(actualBounds.topLeft.y, 0, "box still starts at top");
                chai_1.assert.strictEqual(actualBounds.bottomRight.y, DIV_HEIGHT, "box still ends at bottom");
                div.remove();
            });
            it("adapts to new SVG heights upon redraws", function () {
                dbl.boxVisible(true);
                dbl.resizable(true);
                dbl.renderTo(div);
                dbl.bounds({
                    topLeft: quarterTopLeftPoint,
                    bottomRight: middlePoint,
                });
                var boundsBefore = dbl.bounds();
                var newHeight = 2 * DIV_HEIGHT;
                chai_1.assert.strictEqual(dbl.height(), DIV_HEIGHT, "box has same height as SVG");
                div.style("height", newHeight + "px");
                dbl.redraw();
                chai_1.assert.strictEqual(dbl.height(), newHeight, "box changes height to the new SVG height");
                var boundsAfter = dbl.bounds();
                chai_1.assert.strictEqual(boundsAfter.topLeft.x, boundsBefore.topLeft.x, "box keeps same left edge");
                chai_1.assert.strictEqual(boundsAfter.topLeft.y, 0, "box still starts at top");
                chai_1.assert.strictEqual(boundsAfter.bottomRight.x, boundsBefore.bottomRight.x, "box keeps same right edge");
                chai_1.assert.strictEqual(boundsAfter.bottomRight.y, DIV_HEIGHT * 2, "box still ends at bottom");
                div.remove();
            });
            it("does not error on getting y scale", function () {
                chai_1.assert.doesNotThrow(function () { return dbl.yScale(); }, Error, "getting yScale should not error");
                div.remove();
            });
            it("throws error on setting y scale", function () {
                chai_1.assert.throws(function () { return dbl.yScale(new Plottable.Scales.Linear()); }, Error, "yScales cannot be set", "fails on setting yScale");
                div.remove();
            });
            it("does not error on getting y extent", function () {
                chai_1.assert.doesNotThrow(function () { return dbl.yExtent(); }, Error, "getting yExtent should not error");
                div.remove();
            });
            it("throws error on setting y extent", function () {
                chai_1.assert.throws(function () { return dbl.yExtent([0]); }, Error, "XDragBoxLayer has no yExtent", "fails on setting yExtent");
                div.remove();
            });
            it("moves only in x", function () {
                dbl.boxVisible(true);
                dbl.movable(true);
                dbl.renderTo(div);
                var boundsBefore = { topLeft: quarterTopLeftPoint, bottomRight: quarterBottomRightPoint };
                dbl.bounds(boundsBefore);
                var dragDistance = 10;
                TestMethods.triggerFakeDragSequence(dbl.background(), { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 }, { x: DIV_WIDTH / 2 + dragDistance, y: DIV_HEIGHT / 2 + dragDistance });
                var boundsAfter = dbl.bounds();
                chai_1.assert.strictEqual(boundsAfter.topLeft.x, boundsBefore.topLeft.x + dragDistance, "left edge moved");
                chai_1.assert.strictEqual(boundsAfter.topLeft.y, 0, "box still starts at top");
                chai_1.assert.strictEqual(boundsAfter.bottomRight.x, boundsBefore.bottomRight.x + dragDistance, "right edge moved");
                chai_1.assert.strictEqual(boundsAfter.bottomRight.y, DIV_HEIGHT, "box still ends at bottom");
                div.remove();
            });
            it("does not have resizable CSS class when disabled", function () {
                var xResizableClass = "x-resizable";
                var yResizableClass = "y-resizable";
                dbl.resizable(true);
                chai_1.assert.isTrue(dbl.hasClass(xResizableClass), "carries \"x-resizable\" class if resizable");
                chai_1.assert.isFalse(dbl.hasClass(yResizableClass), "does not carry \"y-resizable\" class even if resizable");
                dbl.enabled(false);
                chai_1.assert.isFalse(dbl.hasClass(xResizableClass), "does not carry \"x-resizable\" class if resizable, but not enabled");
                chai_1.assert.isFalse(dbl.hasClass(xResizableClass), "does not carry \"y-resizable\" class if resizable, but not enabled");
                dbl.resizable(false);
                dbl.enabled(true);
                chai_1.assert.isFalse(dbl.hasClass(xResizableClass), "does not carry \"x-resizable\" class if enabled, but not resizable");
                chai_1.assert.isFalse(dbl.hasClass(xResizableClass), "does not carry \"y-resizable\" class if enabled, but not resizable");
                div.remove();
            });
            it("does not error on destroy when scales are not inputted", function () {
                dbl.renderTo(div);
                chai_1.assert.doesNotThrow(function () { return dbl.destroy(); }, Error, "can destroy without scales");
                div.remove();
            });
        });
    });
});
