"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("SelectionBoxLayer", function () {
    describe("YDragBoxLayer", function () {
        describe("Basic usage", function () {
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 300;
            var quarterTopLeftPoint = {
                x: DIV_WIDTH / 4,
                y: DIV_HEIGHT / 4,
            };
            var middlePoint = {
                x: DIV_WIDTH / 2,
                y: DIV_HEIGHT / 2,
            };
            var quarterBottomRightPoint = {
                x: DIV_WIDTH * 3 / 4,
                y: DIV_HEIGHT * 3 / 4,
            };
            var div;
            var dbl;
            beforeEach(function () {
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                dbl = new Plottable.Components.YDragBoxLayer();
            });
            it("has bounds that extend across full SVG width", function () {
                dbl.boxVisible(true);
                dbl.renderTo(div);
                dbl.bounds({
                    topLeft: quarterTopLeftPoint,
                    bottomRight: middlePoint,
                });
                var actualBounds = dbl.bounds();
                chai_1.assert.strictEqual(actualBounds.topLeft.x, 0, "box starts at SVG left edge");
                chai_1.assert.strictEqual(actualBounds.topLeft.y, quarterTopLeftPoint.y, "top edge set correctly");
                chai_1.assert.strictEqual(actualBounds.bottomRight.x, DIV_WIDTH, "box ends at SVG right edge");
                chai_1.assert.strictEqual(actualBounds.bottomRight.y, middlePoint.y, "bottom edge set correctly");
                div.remove();
            });
            it("resizes only in y", function () {
                dbl.boxVisible(true);
                dbl.resizable(true);
                dbl.renderTo(div);
                var bounds = { topLeft: quarterTopLeftPoint, bottomRight: middlePoint };
                dbl.bounds(bounds);
                TestMethods.triggerFakeDragSequence(dbl.background(), bounds.bottomRight, quarterBottomRightPoint);
                var actualBounds = dbl.bounds();
                chai_1.assert.strictEqual(actualBounds.topLeft.x, 0, "box still starts at left");
                chai_1.assert.strictEqual(actualBounds.bottomRight.x, DIV_WIDTH, "box still ends at right");
                chai_1.assert.strictEqual(actualBounds.bottomRight.y, quarterBottomRightPoint.y, "resized in y");
                div.remove();
            });
            it("adapts to new SVG widths upon redraws", function () {
                dbl.boxVisible(true);
                dbl.resizable(true);
                dbl.renderTo(div);
                dbl.bounds({
                    topLeft: quarterTopLeftPoint,
                    bottomRight: middlePoint,
                });
                var boundsBefore = dbl.bounds();
                var newWidth = 2 * DIV_WIDTH;
                div.style("width", newWidth + "px");
                dbl.redraw();
                chai_1.assert.strictEqual(dbl.width(), newWidth, "component changed width to the new SVG width");
                var boundsAfter = dbl.bounds();
                chai_1.assert.strictEqual(boundsAfter.topLeft.x, 0, "box still starts at left");
                chai_1.assert.strictEqual(boundsAfter.topLeft.y, boundsBefore.topLeft.y, "box keeps same top edge");
                chai_1.assert.strictEqual(boundsAfter.bottomRight.x, DIV_WIDTH * 2, "box still ends at right");
                chai_1.assert.strictEqual(boundsAfter.bottomRight.y, boundsBefore.bottomRight.y, "box keeps same bottom edge");
                div.remove();
            });
            it("does not error on getting x scale", function () {
                chai_1.assert.doesNotThrow(function () { return dbl.xScale(); }, Error, "getting xScale should not error");
                div.remove();
            });
            it("throws error on setting x scale", function () {
                chai_1.assert.throws(function () { return dbl.xScale(new Plottable.Scales.Linear()); }, "xScales cannot be set on an YDragBoxLayer", "fails to set xScale");
                div.remove();
            });
            it("does not error on getting x extent", function () {
                chai_1.assert.doesNotThrow(function () { return dbl.xExtent(); }, Error, "getting xExtent should not error");
                div.remove();
            });
            it("throws error on setting x extent", function () {
                chai_1.assert.throws(function () { return dbl.xExtent([0]); }, Error, "YDragBoxLayer has no xExtent", "fails to set xExtent");
                div.remove();
            });
            it("moves only in y", function () {
                dbl.boxVisible(true);
                dbl.movable(true);
                dbl.renderTo(div);
                dbl.bounds({
                    topLeft: quarterTopLeftPoint,
                    bottomRight: quarterBottomRightPoint,
                });
                var boundsBefore = dbl.bounds();
                var dragDistance = 10;
                TestMethods.triggerFakeDragSequence(dbl.background(), { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 }, { x: DIV_WIDTH / 2 + dragDistance, y: DIV_HEIGHT / 2 + dragDistance });
                var boundsAfter = dbl.bounds();
                chai_1.assert.strictEqual(boundsAfter.topLeft.x, 0, "box still starts at left");
                chai_1.assert.strictEqual(boundsAfter.topLeft.y, boundsBefore.topLeft.y + dragDistance, "top edge moved");
                chai_1.assert.strictEqual(boundsAfter.bottomRight.x, DIV_WIDTH, "box still ends at right");
                chai_1.assert.strictEqual(boundsAfter.bottomRight.y, boundsBefore.bottomRight.y + dragDistance, "bottom edge moved");
                div.remove();
            });
            it("does not have resizable CSS class when disabled", function () {
                var xResizableClass = "x-resizable";
                var yResizableClass = "y-resizable";
                dbl.resizable(true);
                chai_1.assert.isTrue(dbl.hasClass(yResizableClass), "carries \"y-resizable\" class if resizable");
                chai_1.assert.isFalse(dbl.hasClass(xResizableClass), "does not carry \"x-resizable\" class if resizable");
                dbl.enabled(false);
                chai_1.assert.isFalse(dbl.hasClass(yResizableClass), "does not carry \"y-resizable\" class if resizable, but not enabled");
                chai_1.assert.isFalse(dbl.hasClass(xResizableClass), "does not carry \"x-resizable\" class if resizable, but not enabled");
                dbl.resizable(false);
                dbl.enabled(true);
                chai_1.assert.isFalse(dbl.hasClass(yResizableClass), "does not carry \"y-resizable\" class if enabled, but not resizable");
                chai_1.assert.isFalse(dbl.hasClass(xResizableClass), "does not carry \"x-resizable\" class if enabled, but not resizable");
                div.remove();
            });
            it("does not error on destroy when scales are not inputted", function () {
                dbl.renderTo(div);
                chai_1.assert.doesNotThrow(function () { return dbl.destroy(); }, Error, "can destroy without sacles");
                div.remove();
            });
        });
    });
});
