"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
var domUtils_1 = require("../../src/utils/domUtils");
var barPlot_1 = require("../../src/plots/barPlot");
describe("Metadata", function () {
    var xScale;
    var yScale;
    var data1 = [{ x: 0, y: 0 }, { x: 1, y: 1 }];
    var data2 = [{ x: 2, y: 2 }, { x: 3, y: 3 }];
    before(function () {
        xScale = new Plottable.Scales.Linear();
        yScale = new Plottable.Scales.Linear();
        xScale.domain([0, 400]);
        yScale.domain([400, 0]);
    });
    it("Dataset is passed in", function () {
        var div = TestMethods.generateDiv(400, 400);
        var metadata = { foo: 10, bar: 20 };
        var xAccessor = function (d, i, dataset) { return d.x + i * dataset.metadata().foo; };
        var yAccessor = function (d, i, dataset) { return dataset.metadata().bar; };
        var dataset = new Plottable.Dataset(data1, metadata);
        var plot = new Plottable.Plots.Scatter()
            .x(xAccessor, xScale)
            .y(yAccessor, yScale);
        plot.addDataset(dataset);
        plot.renderTo(div);
        var circles = plot.selections().nodes();
        var c1 = d3.select(circles[0]);
        var c2 = d3.select(circles[1]);
        var c1Position = domUtils_1.getTranslateValues(c1);
        var c2Position = domUtils_1.getTranslateValues(c2);
        chai_1.assert.closeTo(c1Position[0], 0, 0.01, "first circle cx is correct");
        chai_1.assert.closeTo(c1Position[1], 20, 0.01, "first circle cy is correct");
        chai_1.assert.closeTo(c2Position[0], 11, 0.01, "second circle cx is correct");
        chai_1.assert.closeTo(c2Position[1], 20, 0.01, "second circle cy is correct");
        var changedMetadata = { foo: 0, bar: 0 };
        dataset.metadata(changedMetadata);
        c1Position = domUtils_1.getTranslateValues(c1);
        c2Position = domUtils_1.getTranslateValues(c2);
        chai_1.assert.closeTo(c1Position[0], 0, 0.01, "first circle cx is correct after metadata change");
        chai_1.assert.closeTo(c1Position[1], 0, 0.01, "first circle cy is correct after metadata change");
        chai_1.assert.closeTo(c2Position[0], 1, 0.01, "second circle cx is correct after metadata change");
        chai_1.assert.closeTo(c2Position[1], 0, 0.01, "second circle cy is correct after metadata change");
        div.remove();
    });
    it("user metadata is applied to associated dataset", function () {
        var div = TestMethods.generateDiv(400, 400);
        var metadata1 = { foo: 10 };
        var metadata2 = { foo: 30 };
        var xAccessor = function (d, i, dataset) { return d.x + (i + 1) * dataset.metadata().foo; };
        var yAccessor = function () { return 0; };
        var dataset1 = new Plottable.Dataset(data1, metadata1);
        var dataset2 = new Plottable.Dataset(data2, metadata2);
        var plot = new Plottable.Plots.Scatter()
            .x(xAccessor, xScale)
            .y(yAccessor, yScale);
        plot.addDataset(dataset1);
        plot.addDataset(dataset2);
        plot.renderTo(div);
        var circles = plot.selections().nodes();
        var c1 = d3.select(circles[0]);
        var c2 = d3.select(circles[1]);
        var c3 = d3.select(circles[2]);
        var c4 = d3.select(circles[3]);
        var c1Position = domUtils_1.getTranslateValues(c1);
        var c2Position = domUtils_1.getTranslateValues(c2);
        var c3Position = domUtils_1.getTranslateValues(c3);
        var c4Position = domUtils_1.getTranslateValues(c4);
        chai_1.assert.closeTo(c1Position[0], 10, 0.01, "first circle is correct");
        chai_1.assert.closeTo(c2Position[0], 21, 0.01, "second circle is correct");
        chai_1.assert.closeTo(c3Position[0], 32, 0.01, "third circle is correct");
        chai_1.assert.closeTo(c4Position[0], 63, 0.01, "fourth circle is correct");
        div.remove();
    });
    it("each plot passes metadata to projectors", function () {
        var div = TestMethods.generateDiv(400, 400);
        var metadata = { foo: 11 };
        var dataset1 = new Plottable.Dataset(data1, metadata);
        var dataset2 = new Plottable.Dataset(data2, metadata);
        var checkXYPlot = function (plot) {
            var xAccessor = function (d, i, dataset) {
                return d.x + dataset.metadata().foo;
            };
            var yAccessor = function (d, i, dataset) {
                return d.y + dataset.metadata().foo;
            };
            plot.addDataset(dataset1)
                .addDataset(dataset2);
            plot.x(xAccessor, xScale)
                .y(yAccessor, yScale);
            // This should not crash. If some metadata is not passed, undefined property error will be raised during accessor call.
            plot.renderTo(div);
            plot.destroy();
        };
        var checkPiePlot = function (plot) {
            plot.sectorValue(function (d) { return d.x; }).addDataset(dataset1);
            // This should not crash. If some metadata is not passed, undefined property error will be raised during accessor call.
            plot.renderTo(div);
            plot.destroy();
        };
        checkXYPlot(new Plottable.Plots.Area());
        checkXYPlot(new Plottable.Plots.StackedArea());
        checkXYPlot(new Plottable.Plots.Bar());
        checkXYPlot(new Plottable.Plots.StackedBar());
        checkXYPlot(new Plottable.Plots.StackedBar(barPlot_1.BarOrientation.horizontal));
        checkXYPlot(new Plottable.Plots.ClusteredBar());
        checkXYPlot(new Plottable.Plots.Bar(barPlot_1.BarOrientation.horizontal));
        checkXYPlot(new Plottable.Plots.Scatter());
        checkPiePlot(new Plottable.Plots.Pie());
        div.remove();
    });
});
