"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("SymbolFactory", function () {
    describe("Generates correct path", function () {
        var svg;
        var symbolSize = 5;
        beforeEach(function () {
            svg = TestMethods.generateSVG();
        });
        it("returns a circle factory that generates circle based on symbolSize", function () {
            var circleFactory = Plottable.SymbolFactories.circle();
            var actualSize = Math.PI * Math.pow(symbolSize / 2, 2);
            var d = circleFactory(symbolSize);
            var expectedD = d3.symbol().type(d3.symbolCircle).size(actualSize)(null);
            chai_1.assert.strictEqual(d, expectedD, "a circle of set size is generated");
            var path = svg.append("path").attr("d", d);
            var bbox = Plottable.Utils.DOM.elementBBox(path);
            chai_1.assert.strictEqual(bbox.height, symbolSize, "height is as set");
            chai_1.assert.strictEqual(bbox.width, symbolSize, "width is as set");
            chai_1.assert.strictEqual(bbox.x, -bbox.width / 2, "x is centered in the middle");
            chai_1.assert.strictEqual(bbox.y, -bbox.height / 2, "y is centered in the middle");
            svg.remove();
        });
        it("returns a square factory that generates square based on symbolSize", function () {
            var squareFactory = Plottable.SymbolFactories.square();
            var actualSize = Math.pow(symbolSize, 2);
            var d = squareFactory(symbolSize);
            var expectedD = d3.symbol().type(d3.symbolSquare).size(actualSize)(null);
            chai_1.assert.strictEqual(d, expectedD, "a square of set size is generated");
            var path = svg.append("path").attr("d", d);
            var bbox = Plottable.Utils.DOM.elementBBox(path);
            chai_1.assert.strictEqual(bbox.height, symbolSize, " is as set");
            chai_1.assert.strictEqual(bbox.width, symbolSize, "width is as set");
            chai_1.assert.strictEqual(bbox.x, -bbox.width / 2, "x is centered in the middle");
            chai_1.assert.strictEqual(bbox.y, -bbox.height / 2, "y is centered in the middle");
            svg.remove();
        });
        it("returns a cross factory that generates cross based on symbolSize", function () {
            var crossFactory = Plottable.SymbolFactories.cross();
            var actualSize = (5 / 9) * Math.pow(symbolSize, 2);
            var d = crossFactory(symbolSize);
            var expectedD = d3.symbol().type(d3.symbolCross).size(actualSize)(null);
            chai_1.assert.strictEqual(d, expectedD, "a cross of set size is generated");
            var path = svg.append("path").attr("d", d);
            var bbox = Plottable.Utils.DOM.elementBBox(path);
            chai_1.assert.strictEqual(bbox.height, symbolSize, "height is as set");
            chai_1.assert.strictEqual(bbox.width, symbolSize, "width is as set");
            chai_1.assert.strictEqual(bbox.x, -bbox.width / 2, "x is centered in the middle");
            chai_1.assert.strictEqual(bbox.y, -bbox.height / 2, "y is centered in the middle");
            svg.remove();
        });
        it("returns a diamond factory that generates diamond based on symbolSize", function () {
            var diamondFactory = Plottable.SymbolFactories.diamond();
            var actualSize = Math.tan(Math.PI / 6) * Math.pow(symbolSize, 2) / 2;
            var d = diamondFactory(symbolSize);
            var expectedD = d3.symbol().type(d3.symbolDiamond).size(actualSize)(null);
            chai_1.assert.strictEqual(d, expectedD, "a diamond of set size is generated");
            var path = svg.append("path").attr("d", d);
            var bbox = Plottable.Utils.DOM.elementBBox(path);
            chai_1.assert.strictEqual(bbox.height, symbolSize, "height is as set");
            chai_1.assert.operator(bbox.width, "<", symbolSize, "width is as set");
            chai_1.assert.strictEqual(bbox.x, -bbox.width / 2, "x is centered in the middle");
            chai_1.assert.strictEqual(bbox.y, -bbox.height / 2, "y is centered in the middle");
            svg.remove();
        });
        it("returns a triangle factory that generates a triangle based on symbolSize", function () {
            var triangleFactory = Plottable.SymbolFactories.triangle();
            var actualSize = Math.sqrt(3) * Math.pow(symbolSize / 2, 2);
            var d = triangleFactory(symbolSize);
            var expectedD = d3.symbol().type(d3.symbolTriangle).size(actualSize)(null);
            chai_1.assert.strictEqual(d, expectedD, "a up triangle of set size is generated");
            var path = svg.append("path").attr("d", d);
            var bbox = Plottable.Utils.DOM.elementBBox(path);
            chai_1.assert.operator(bbox.height, "<", symbolSize, "height is as set");
            chai_1.assert.strictEqual(bbox.width, symbolSize, "width is as set");
            chai_1.assert.strictEqual(bbox.x, -bbox.width / 2, "x is centered in the middle");
            svg.remove();
        });
        it("returns a wye factory that generates wye based on symbolSize", function () {
            var wyeFactory = Plottable.SymbolFactories.wye();
            var a = ((1 / Math.sqrt(12)) / 2 + 1) * 3;
            var actualSize = a * Math.pow(symbolSize / 2.4, 2);
            var d = wyeFactory(symbolSize);
            var expectedD = d3.symbol().type(d3.symbolWye).size(actualSize)(null);
            chai_1.assert.strictEqual(d, expectedD, "a wye of set size is generated");
            var path = svg.append("path").attr("d", d);
            var bbox = Plottable.Utils.DOM.elementBBox(path);
            chai_1.assert.operator(bbox.height, "<", symbolSize, "height is as set");
            chai_1.assert.strictEqual(bbox.x, -bbox.width / 2, "x is centered in the middle");
            svg.remove();
        });
    });
});
