"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Dispatchers", function () {
    describe("Mouse Dispatcher", function () {
        describe("Basic usage", function () {
            var div;
            var component;
            beforeEach(function () {
                component = new Plottable.Component();
                div = TestMethods.generateDiv();
                component.anchor(div);
            });
            it("creates only one Dispatcher.Mouse per <div> using getDispatcher() ", function () {
                var dispatcher1 = Plottable.Dispatchers.Mouse.getDispatcher(component);
                chai_1.assert.isNotNull(dispatcher1, "created a new Dispatcher on an SVG");
                var dispatcher2 = Plottable.Dispatchers.Mouse.getDispatcher(component);
                chai_1.assert.strictEqual(dispatcher1, dispatcher2, "returned the existing Dispatcher if called again with same <div>");
                div.remove();
            });
            it("returns non-null value for default lastMousePosition()", function () {
                var mouseDispatcher = Plottable.Dispatchers.Mouse.getDispatcher(component);
                var point = mouseDispatcher.lastMousePosition();
                chai_1.assert.isNotNull(point, "returns a value after initialization");
                chai_1.assert.isNotNull(point.x, "x value is set");
                chai_1.assert.isNotNull(point.y, "y value is set");
                div.remove();
            });
        });
        describe("Callbacks", function () {
            var targetX = 17;
            var targetY = 76;
            var expectedPoint = {
                x: targetX,
                y: targetY,
            };
            var component;
            var div;
            var mouseDispatcher;
            beforeEach(function () {
                var DIV_WIDTH = 400;
                var DIV_HEIGHT = 400;
                component = new Plottable.Component();
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                component.anchor(div);
                mouseDispatcher = Plottable.Dispatchers.Mouse.getDispatcher(component);
            });
            it("calls the mouseDown callback", function () {
                var callbackWasCalled = false;
                var callback = function (point, event) {
                    callbackWasCalled = true;
                    TestMethods.assertPointsClose(point, expectedPoint, 0.5, "mouse position is correct");
                    chai_1.assert.isNotNull(event, "mouse event was passed to the callback");
                    chai_1.assert.instanceOf(event, MouseEvent, "the event passed is an instance of MouseEvent");
                };
                chai_1.assert.strictEqual(mouseDispatcher.onMouseDown(callback), mouseDispatcher, "setting the mouseDown callback returns the dispatcher");
                TestMethods.triggerFakeMouseEvent("mousedown", div, targetX, targetY);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on mouseDown");
                chai_1.assert.strictEqual(mouseDispatcher.offMouseDown(callback), mouseDispatcher, "unsetting the mouseDown callback returns the dispatcher");
                callbackWasCalled = false;
                TestMethods.triggerFakeMouseEvent("mousedown", div, targetX, targetY);
                chai_1.assert.isFalse(callbackWasCalled, "callback was disconnected from the dispatcher");
                div.remove();
            });
            it("calls the mouseUp callback", function () {
                var callbackWasCalled = false;
                var callback = function (point, event) {
                    callbackWasCalled = true;
                    TestMethods.assertPointsClose(point, expectedPoint, 0.5, "mouse position is correct");
                    chai_1.assert.isNotNull(event, "mouse event was passed to the callback");
                    chai_1.assert.instanceOf(event, MouseEvent, "the event passed is an instance of MouseEvent");
                };
                chai_1.assert.strictEqual(mouseDispatcher.onMouseUp(callback), mouseDispatcher, "setting the mouseUp callback returns the dispatcher");
                TestMethods.triggerFakeMouseEvent("mouseup", div, targetX, targetY);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on mouseUp");
                chai_1.assert.strictEqual(mouseDispatcher.offMouseUp(callback), mouseDispatcher, "unsetting the mouseUp callback returns the dispatcher");
                callbackWasCalled = false;
                TestMethods.triggerFakeMouseEvent("mouseup", div, targetX, targetY);
                chai_1.assert.isFalse(callbackWasCalled, "callback was disconnected from the dispatcher");
                div.remove();
            });
            it("calls the wheel callback", function () {
                // HACKHACK PhantomJS doesn't implement fake creation of WheelEvents
                // https://github.com/ariya/phantomjs/issues/11289
                if (window.PHANTOMJS) {
                    div.remove();
                    return;
                }
                var targetDeltaY = 10;
                var callbackWasCalled = false;
                var callback = function (point, event) {
                    callbackWasCalled = true;
                    chai_1.assert.strictEqual(event.deltaY, targetDeltaY, "deltaY value was passed to callback");
                    TestMethods.assertPointsClose(point, expectedPoint, 0.5, "mouse position is correct");
                    chai_1.assert.isNotNull(event, "mouse event was passed to the callback");
                    chai_1.assert.instanceOf(event, MouseEvent, "the event passed is an instance of MouseEvent");
                };
                chai_1.assert.strictEqual(mouseDispatcher.onWheel(callback), mouseDispatcher, "setting the wheel callback returns the dispatcher");
                TestMethods.triggerFakeWheelEvent("wheel", div, targetX, targetY, targetDeltaY);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on wheel");
                chai_1.assert.strictEqual(mouseDispatcher.offWheel(callback), mouseDispatcher, "unsetting the wheel callback returns the dispatcher");
                callbackWasCalled = false;
                TestMethods.triggerFakeWheelEvent("wheel", div, targetX, targetY, targetDeltaY);
                chai_1.assert.isFalse(callbackWasCalled, "callback was disconnected from the dispatcher");
                div.remove();
            });
            it("calls the dblClick callback", function () {
                var callbackWasCalled = false;
                var callback = function (point, event) {
                    callbackWasCalled = true;
                    TestMethods.assertPointsClose(point, expectedPoint, 0.5, "mouse position is correct");
                    chai_1.assert.isNotNull(event, "mouse event was passed to the callback");
                    chai_1.assert.instanceOf(event, MouseEvent, "the event passed is an instance of MouseEvent");
                };
                chai_1.assert.strictEqual(mouseDispatcher.onDblClick(callback), mouseDispatcher, "setting the dblClick callback returns the dispatcher");
                TestMethods.triggerFakeMouseEvent("dblclick", div, targetX, targetY);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on dblClick");
                chai_1.assert.strictEqual(mouseDispatcher.offDblClick(callback), mouseDispatcher, "unsetting the dblClick callback returns the dispatcher");
                callbackWasCalled = false;
                TestMethods.triggerFakeMouseEvent("dblclick", div, targetX, targetY);
                chai_1.assert.isFalse(callbackWasCalled, "callback was disconnected from the dispatcher");
                div.remove();
            });
            it("calls mouseMove callback on mouseover, mousemove, and mouseout", function () {
                var callbackWasCalled = false;
                var callback = function (point, event) {
                    callbackWasCalled = true;
                    TestMethods.assertPointsClose(point, expectedPoint, 0.5, "mouse position is correct");
                    chai_1.assert.isNotNull(event, "mouse event was passed to the callback");
                    chai_1.assert.instanceOf(event, MouseEvent, "the event passed is an instance of MouseEvent");
                };
                mouseDispatcher.onMouseMove(callback);
                TestMethods.triggerFakeMouseEvent("mouseover", div, targetX, targetY);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on mouseover");
                callbackWasCalled = false;
                TestMethods.triggerFakeMouseEvent("mousemove", div, targetX, targetY);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on mousemove");
                callbackWasCalled = false;
                TestMethods.triggerFakeMouseEvent("mouseout", div, targetX, targetY);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on mouseout");
                mouseDispatcher.offMouseMove(callback);
                callbackWasCalled = false;
                TestMethods.triggerFakeMouseEvent("mouseover", div, targetX, targetY);
                chai_1.assert.isFalse(callbackWasCalled, "disconnected dispatcher callback not called on mouseover");
                TestMethods.triggerFakeMouseEvent("mousemove", div, targetX, targetY);
                chai_1.assert.isFalse(callbackWasCalled, "disconnected dispatcher callback not called on mousemove");
                TestMethods.triggerFakeMouseEvent("mouseout", div, targetX, targetY);
                chai_1.assert.isFalse(callbackWasCalled, "disconnected dispatcher callback not called on mouseout");
                div.remove();
            });
            it("can register two callbacks for the same mouse dispatcher", function () {
                var cb1Called = false;
                var cb1 = function () { return cb1Called = true; };
                var cb2Called = false;
                var cb2 = function () { return cb2Called = true; };
                mouseDispatcher.onMouseMove(cb1);
                mouseDispatcher.onMouseMove(cb2);
                TestMethods.triggerFakeMouseEvent("mousemove", div, targetX, targetY);
                chai_1.assert.isTrue(cb1Called, "callback 1 was called on mousemove");
                chai_1.assert.isTrue(cb2Called, "callback 2 was called on mousemove");
                cb1Called = false;
                cb2Called = false;
                mouseDispatcher.offMouseMove(cb1);
                TestMethods.triggerFakeMouseEvent("mousemove", div, targetX, targetY);
                chai_1.assert.isFalse(cb1Called, "callback was not called after blanking");
                chai_1.assert.isTrue(cb2Called, "callback 2 was still called");
                mouseDispatcher.offMouseMove(cb2);
                div.remove();
            });
            it("doesn't call callbacks if not in the DOM", function () {
                var callbackWasCalled = false;
                var callback = function () { return callbackWasCalled = true; };
                mouseDispatcher.onMouseMove(callback);
                TestMethods.triggerFakeMouseEvent("mousemove", div, targetX, targetY);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on mousemove");
                div.remove();
                callbackWasCalled = false;
                TestMethods.triggerFakeMouseEvent("mousemove", div, targetX, targetY);
                chai_1.assert.isFalse(callbackWasCalled, "callback was not called after <div> was removed from DOM");
                mouseDispatcher.offMouseMove(callback);
            });
            it("doesn't call callbacks for clicks if obscured by overlay", function () {
                var callbackWasCalled = false;
                var callback = function () { return callbackWasCalled = true; };
                mouseDispatcher.onMouseDown(callback);
                TestMethods.triggerFakeMouseEvent("mousedown", div, targetX, targetY);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on mousedown");
                var element = div.node();
                // Getting the absolute coordinates of the SVG in order to place the overlay at the right location
                var topLeftCorner = { x: 0, y: 0 };
                while (element != null) {
                    topLeftCorner.x += (element.offsetLeft || element.clientLeft || 0);
                    topLeftCorner.y += (element.offsetTop || element.clientTop || 0);
                    element = (element.offsetParent || element.parentNode);
                }
                var overlay = TestMethods.getElementParent().append("div").styles({
                    height: "400px",
                    width: "400px",
                    topLeftCorner: "absolute",
                    top: topLeftCorner.y + "px",
                    left: topLeftCorner.x + "px",
                });
                callbackWasCalled = false;
                TestMethods.triggerFakeMouseEvent("mousedown", overlay, targetX, targetY);
                chai_1.assert.isFalse(callbackWasCalled, "callback was not called on mousedown on overlay");
                mouseDispatcher.offMouseDown(callback);
                div.remove();
                overlay.remove();
            });
        });
    });
});
