"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var d3 = require("d3");
var sinon = require("sinon");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Drawers", function () {
    describe("Drawer", function () {
        var MockDrawer = (function (_super) {
            __extends(MockDrawer, _super);
            function MockDrawer(dataset) {
                var _this = _super.call(this, dataset) || this;
                _this._svgElementName = MockDrawer.ELEMENT_NAME;
                _this._className = MockDrawer.CSS_CLASS_NAME;
                return _this;
            }
            return MockDrawer;
        }(Plottable.Drawer));
        MockDrawer.ELEMENT_NAME = "mock";
        MockDrawer.CSS_CLASS_NAME = "mock-css";
        var drawer;
        var dataset;
        beforeEach(function () {
            dataset = new Plottable.Dataset();
            drawer = new MockDrawer(dataset);
        });
        it("returns its element name as a selector", function () {
            chai_1.assert.strictEqual(drawer.selector(), MockDrawer.ELEMENT_NAME);
        });
        it("can set and get its renderArea", function () {
            var svg = TestMethods.generateSVG();
            chai_1.assert.strictEqual(drawer.renderArea(svg), drawer, "setter mode returns the calling Drawer");
            chai_1.assert.sameMembers(drawer.renderArea().nodes(), svg.nodes(), "getter mode returns the selection");
            svg.remove();
        });
        it("can remove its renderArea", function () {
            var svg = TestMethods.generateSVG();
            drawer.renderArea(svg);
            drawer.remove();
            chai_1.assert.isFalse(document.body.contains(svg.node()), "renderArea was removed from the DOM");
        });
        describe("drawing and retrieving elements", function () {
            var data = ["A", "B", "C"];
            var propertyName = "property";
            var svg;
            beforeEach(function () {
                svg = TestMethods.generateSVG();
                var invocationCount = 0;
                var validatingProjector = function (datum, index, passedDataset) {
                    chai_1.assert.strictEqual(datum, data[invocationCount], "projector was passed the correct datum");
                    chai_1.assert.strictEqual(index, invocationCount, "projector was passed the correct index");
                    chai_1.assert.strictEqual(passedDataset, dataset, "projector was passed the Drawer's dataset");
                    invocationCount++;
                    return datum;
                };
                var attrToProjector = {};
                attrToProjector[propertyName] = validatingProjector;
                var drawSteps = [
                    {
                        attrToProjector: attrToProjector,
                        animator: new Plottable.Animators.Null(),
                    },
                ];
                drawer.renderArea(svg);
                drawer.draw(data, drawSteps);
            });
            afterEach(function () {
                if (this.currentTest.state === "passed") {
                    svg.remove();
                }
            });
            it("draws elements accoding to a specified DrawStep", function () {
                var drawn = svg.selectAll(MockDrawer.ELEMENT_NAME);
                chai_1.assert.strictEqual(drawn.size(), data.length, "created one element per datum");
                drawn.each(function (datum, index) {
                    var element = d3.select(this);
                    chai_1.assert.strictEqual(element.attr(propertyName), data[index], "property was set correctly");
                    chai_1.assert.isTrue(element.classed(MockDrawer.CSS_CLASS_NAME), "element carries the correct CSS class");
                });
            });
            it("can retrieve a selection containing elements it drew", function () {
                var selection = drawer.selection();
                chai_1.assert.strictEqual(selection.size(), data.length, "retrieved one element per datum");
                var drawn = svg.selectAll(MockDrawer.ELEMENT_NAME);
                chai_1.assert.deepEqual(selection.node(), drawn.node(), "retrieved all elements it drew");
            });
            it("can retrieve the selection for a particular index", function () {
                var selection = drawer.selection();
                data.forEach(function (datum, index) {
                    var selectionForIndex = drawer.selectionForIndex(index);
                    chai_1.assert.strictEqual(selectionForIndex.node(), selection.nodes()[index], "retrieves the correct selection for index " + index);
                });
            });
        });
        describe("canvas drawing", function () {
            it("setting the renderArea/canvas clears the other one", function () {
                var svg = TestMethods.generateSVG();
                var canvas = d3.select(document.createElement("canvas"));
                drawer.renderArea(svg);
                drawer.canvas(canvas);
                chai_1.assert.isNull(drawer.renderArea(), "setting canvas clears renderArea");
                chai_1.assert.strictEqual(drawer.canvas(), canvas, "can get canvas after setting");
                drawer.renderArea(svg);
                chai_1.assert.isNull(drawer.canvas(), "setting renderArea clears canvas");
                chai_1.assert.strictEqual(drawer.renderArea(), svg, "can get renderArea after setting");
                svg.remove();
            });
            it("draw() calls _drawStepCanvas", function () {
                var canvas = d3.select(document.createElement("canvas"));
                drawer.canvas(canvas);
                var drawStep = {
                    animator: new Plottable.Animators.Null(),
                    attrToProjector: {}
                };
                var drawStepCanvasStub = sinon.stub(drawer, "_drawStepCanvas", function () { });
                var data = [];
                drawer.draw(data, [drawStep]);
                chai_1.assert.strictEqual(drawStepCanvasStub.args[0][0], data, "_drawStepCanvas is called");
                drawStepCanvasStub.reset();
                var svg = TestMethods.generateSVG();
                drawer.renderArea(svg);
                drawer.draw(data, [drawStep]);
                chai_1.assert.isFalse(drawStepCanvasStub.called, "_drawStepCanvas isn't called in svg land");
                svg.remove();
            });
        });
        describe("animation timings", function () {
            var MockAnimator = (function () {
                function MockAnimator(time, callback) {
                    this._time = time;
                    this._callback = callback;
                }
                MockAnimator.prototype.totalTime = function (numberOfIterations) {
                    return this._time;
                };
                MockAnimator.prototype.animate = function (selection, attrToProjector) {
                    if (this._callback) {
                        this._callback();
                    }
                    return selection;
                };
                return MockAnimator;
            }());
            var oldTimeout;
            var timings;
            var svg;
            before(function () {
                oldTimeout = Plottable.Utils.Window.setTimeout;
                Plottable.Utils.Window.setTimeout = function (f, time) {
                    var args = [];
                    for (var _i = 2; _i < arguments.length; _i++) {
                        args[_i - 2] = arguments[_i];
                    }
                    timings.push(time);
                    return oldTimeout(f, time, args);
                };
            });
            after(function () {
                Plottable.Utils.Window.setTimeout = oldTimeout;
            });
            beforeEach(function () {
                timings = [];
                svg = TestMethods.generateSVG();
                drawer = new Plottable.Drawer(null); // HACKHACK #2777: have to create a Plottable.Drawer to avoid "two Plottables" issue
                drawer._svgElementName = MockDrawer.ELEMENT_NAME;
                drawer.renderArea(svg);
            });
            afterEach(function () {
                svg.remove(); // no point keeping it around since we don't draw anything in it anyway
            });
            it("correctly computes the total draw time", function () {
                function makeFixedTimeAnimator(totalTime) {
                    return {
                        animate: function () { return null; },
                        totalTime: function () { return totalTime; },
                    };
                }
                var animationTimes = [10, 20];
                var drawSteps = animationTimes.map(function (time) {
                    return {
                        attrToProjector: {},
                        animator: makeFixedTimeAnimator(time),
                    };
                });
                var totalTime = drawer.totalDrawTime([], drawSteps);
                var expectedTotalTime = d3.sum(animationTimes);
                chai_1.assert.strictEqual(totalTime, expectedTotalTime, "returned the total time taken by all Animators");
            });
            it("computes the correct timings for Null Animators", function () {
                var a1 = new Plottable.Animators.Null();
                var a2 = new Plottable.Animators.Null();
                var ds1 = { attrToProjector: {}, animator: a1 };
                var ds2 = { attrToProjector: {}, animator: a2 };
                var steps = [ds1, ds2];
                drawer.draw([], steps);
                chai_1.assert.deepEqual(timings, [0, 0], "setTimeout called twice with 0 time both times");
            });
            it("computes the correct timings for non-Null Animators", function (done) {
                var callback1Called = false;
                var callback1 = function () {
                    callback1Called = true;
                };
                var animator1Time = 20;
                var a1 = new MockAnimator(animator1Time, callback1);
                var ds1 = { attrToProjector: {}, animator: a1 };
                var callback2Called = false;
                var callback2 = function () {
                    chai_1.assert.isTrue(callback1Called, "callback2 called after callback 1");
                    callback2Called = true;
                };
                var animator2Time = 10;
                var a2 = new MockAnimator(animator2Time, callback2);
                var ds2 = { attrToProjector: {}, animator: a2 };
                var callback3 = function () {
                    chai_1.assert.isTrue(callback2Called, "callback3 called after callback 2");
                    done();
                };
                var animator3Time = 0;
                var a3 = new MockAnimator(animator3Time, callback3);
                var ds3 = { attrToProjector: {}, animator: a3 };
                var steps = [ds1, ds2, ds3];
                drawer.draw([], steps);
                var expectedTimings = [0, animator1Time, animator1Time + animator2Time];
                chai_1.assert.deepEqual(timings, expectedTimings, "setTimeout called with appropriate times");
            });
        });
    });
});
