"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Interactions", function () {
    describe("Drag Interaction", function () {
        var DIV_WIDTH = 400;
        var DIV_HEIGHT = 400;
        var startPoint = {
            x: DIV_WIDTH / 4,
            y: DIV_HEIGHT / 4,
        };
        var endPoint = {
            x: DIV_WIDTH / 2,
            y: DIV_HEIGHT / 2,
        };
        var positiveOutsidePoint = {
            x: DIV_WIDTH * 1.5,
            y: DIV_HEIGHT * 1.5,
        };
        var negativeOutsidePoint = {
            x: -DIV_WIDTH / 2,
            y: -DIV_HEIGHT / 2,
        };
        var div;
        var component;
        var dragInteraction;
        function makeDragCallback() {
            var callback = function (start, end) {
                callback.lastStartPoint = start;
                callback.lastEndPoint = end;
                callback.called = true;
            };
            callback.called = false;
            callback.reset = function () {
                callback.lastStartPoint = undefined;
                callback.lastStartPoint = undefined;
                callback.called = false;
            };
            return callback;
        }
        function triggerFakeDragStart(point, mode) {
            if (mode === void 0) { mode = TestMethods.InteractionMode.Mouse; }
            TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.Start, component.background(), point.x, point.y);
        }
        function triggerFakeDragMove(start, end, mode) {
            if (mode === void 0) { mode = TestMethods.InteractionMode.Mouse; }
            TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.Start, component.background(), start.x, start.y);
            TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.Move, component.background(), end.x, end.y);
        }
        function triggerFakeDragEnd(start, end, mode) {
            if (mode === void 0) { mode = TestMethods.InteractionMode.Mouse; }
            TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.Start, component.background(), start.x, start.y);
            TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.End, component.background(), end.x, end.y);
        }
        beforeEach(function () {
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            component = new Plottable.Component();
            component.renderTo(div);
            dragInteraction = new Plottable.Interactions.Drag();
            dragInteraction.attachTo(component);
        });
        afterEach(function () {
            if (this.currentTest.state === "passed") {
                div.remove();
            }
        });
        describe("registering and dergistering callbacks", function () {
            ["DragStart", "Drag", "DragEnd"].forEach(function (eventName) {
                function register(callback) {
                    switch (eventName) {
                        case "DragStart":
                            return dragInteraction.onDragStart(callback);
                        case "Drag":
                            return dragInteraction.onDrag(callback);
                        case "DragEnd":
                            return dragInteraction.onDragEnd(callback);
                        default:
                            throw new Error("unrecognized event type \"" + eventName + "\"");
                    }
                }
                function deregister(callback) {
                    switch (eventName) {
                        case "DragStart":
                            return dragInteraction.offDragStart(callback);
                        case "Drag":
                            return dragInteraction.offDrag(callback);
                        case "DragEnd":
                            return dragInteraction.offDragEnd(callback);
                        default:
                            throw new Error("unrecognized event type \"" + eventName + "\"");
                    }
                }
                function triggerAppropriateFakeEvent(start, end) {
                    switch (eventName) {
                        case "DragStart":
                            triggerFakeDragStart(start);
                            break;
                        case "Drag":
                            triggerFakeDragMove(start, end);
                            break;
                        case "DragEnd":
                            triggerFakeDragEnd(start, end);
                            break;
                        default:
                            throw new Error("unrecognized event type \"" + eventName + "\"");
                    }
                }
                describe("for " + eventName, function () {
                    it("registers callback using on" + eventName, function () {
                        var callback = makeDragCallback();
                        chai_1.assert.strictEqual(register(callback), dragInteraction, "registration returns the calling Interaction");
                        triggerAppropriateFakeEvent(startPoint, endPoint);
                        chai_1.assert.isTrue(callback.called, "Interaction correctly triggers the callback");
                    });
                    it("deregisters callback using off" + eventName, function () {
                        var callback = makeDragCallback();
                        register(callback);
                        chai_1.assert.strictEqual(deregister(callback), dragInteraction, "deregistration returns the calling Interaction");
                        triggerAppropriateFakeEvent(startPoint, endPoint);
                        chai_1.assert.isFalse(callback.called, "callback should be disconnected from the Interaction");
                    });
                    it("can register multiple on" + eventName + " callbacks", function () {
                        var callback1 = makeDragCallback();
                        var callback2 = makeDragCallback();
                        register(callback1);
                        register(callback2);
                        triggerAppropriateFakeEvent(startPoint, endPoint);
                        chai_1.assert.isTrue(callback1.called, "Interaction should trigger the first callback");
                        chai_1.assert.isTrue(callback2.called, "Interaction should trigger the second callback");
                    });
                    it("can deregister a on" + eventName + " callback without affecting the other ones", function () {
                        var callback1 = makeDragCallback();
                        var callback2 = makeDragCallback();
                        register(callback1);
                        register(callback2);
                        deregister(callback1);
                        triggerAppropriateFakeEvent(startPoint, endPoint);
                        chai_1.assert.isFalse(callback1.called, "Callback1 should be disconnected from the Interaction");
                        chai_1.assert.isTrue(callback2.called, "Callback2 should still exist on the Interaction");
                    });
                });
            });
        });
        describe("invoking callbacks", function () {
            describe("for DragStart", function () {
                [TestMethods.InteractionMode.Mouse, TestMethods.InteractionMode.Touch].forEach(function (mode) {
                    describe("with " + TestMethods.InteractionMode[mode] + " events", function () {
                        var callback;
                        beforeEach(function () {
                            callback = makeDragCallback();
                            dragInteraction.onDragStart(callback);
                        });
                        it("invokes onDragStart callback on start event", function () {
                            triggerFakeDragStart(startPoint, mode);
                            chai_1.assert.isTrue(callback.called, "callback was called on beginning drag");
                            chai_1.assert.deepEqual(callback.lastStartPoint, startPoint, "was passed the correct point");
                        });
                        it("does not invoke callback if drag starts outside Component", function () {
                            triggerFakeDragStart(positiveOutsidePoint, mode);
                            chai_1.assert.isFalse(callback.called, "does not trigger callback if drag starts outside the Component (positive)");
                            triggerFakeDragStart(negativeOutsidePoint, mode);
                            chai_1.assert.isFalse(callback.called, "does not trigger callback if drag starts outside the Component (negative)");
                        });
                        if (mode === TestMethods.InteractionMode.Mouse) {
                            it("does not invoke onDragStart on right click", function () {
                                TestMethods.triggerFakeMouseEvent("mousedown", component.background(), startPoint.x, startPoint.y, 2);
                                chai_1.assert.isFalse(callback.called, "callback is not called on right-click");
                            });
                        }
                    });
                });
            });
            describe("for Drag", function () {
                var callback;
                beforeEach(function () {
                    callback = makeDragCallback();
                    dragInteraction.onDrag(callback);
                });
                [TestMethods.InteractionMode.Mouse, TestMethods.InteractionMode.Touch].forEach(function (mode) {
                    it("passes correct start and end point on drag for " + TestMethods.InteractionMode[mode], function () {
                        triggerFakeDragMove(startPoint, endPoint);
                        chai_1.assert.isTrue(callback.called, "callback was called on dragging");
                        chai_1.assert.deepEqual(callback.lastStartPoint, startPoint, "was passed the correct starting point");
                        chai_1.assert.deepEqual(callback.lastEndPoint, endPoint, "was passed the correct current point");
                    });
                });
                it("does not continue dragging once the touch is cancelled", function () {
                    var target = component.background();
                    var tenFromEndPoint = {
                        x: endPoint.x - 10,
                        y: endPoint.y - 10,
                    };
                    TestMethods.triggerFakeTouchEvent("touchstart", target, [startPoint]);
                    TestMethods.triggerFakeTouchEvent("touchmove", target, [tenFromEndPoint]);
                    TestMethods.triggerFakeTouchEvent("touchcancel", target, [tenFromEndPoint]);
                    TestMethods.triggerFakeTouchEvent("touchend", target, [endPoint]);
                    chai_1.assert.isTrue(callback.called, "the callback is called");
                    chai_1.assert.deepEqual(callback.lastStartPoint, startPoint, "start point is correct");
                    chai_1.assert.deepEqual(callback.lastEndPoint, tenFromEndPoint, "end point is last known location when cancelled");
                });
            });
            describe("for DragEnd", function () {
                var callback;
                beforeEach(function () {
                    callback = makeDragCallback();
                    dragInteraction.onDragEnd(callback);
                });
                [TestMethods.InteractionMode.Mouse, TestMethods.InteractionMode.Touch].forEach(function (mode) {
                    it("passes correct start and end point on drag ending for " + TestMethods.InteractionMode[mode], function () {
                        triggerFakeDragEnd(startPoint, endPoint);
                        chai_1.assert.isTrue(callback.called, "callback was called on drag ending");
                        chai_1.assert.deepEqual(callback.lastStartPoint, startPoint, "was passed the correct starting point");
                        chai_1.assert.deepEqual(callback.lastEndPoint, endPoint, "was passed the correct current point");
                    });
                    it("supports multiple start/move/end drag callbacks at the same time for " + TestMethods.InteractionMode[mode], function () {
                        var startCallback = makeDragCallback();
                        var moveCallback = makeDragCallback();
                        var endCallback = makeDragCallback();
                        dragInteraction.onDragStart(startCallback);
                        dragInteraction.onDrag(moveCallback);
                        dragInteraction.onDragEnd(endCallback);
                        TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.Start, component.background(), startPoint.x, startPoint.y);
                        chai_1.assert.isTrue(startCallback.called, "callback for drag start was called");
                        TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.Move, component.background(), endPoint.x, endPoint.y);
                        chai_1.assert.isTrue(moveCallback.called, "callback for drag was called");
                        TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.End, component.background(), endPoint.x, endPoint.y);
                        chai_1.assert.isTrue(endCallback.called, "callback for drag end was called");
                    });
                });
                it("does not call callback on mouseup from the right-click button", function () {
                    TestMethods.triggerFakeMouseEvent("mousedown", component.background(), startPoint.x, startPoint.y);
                    TestMethods.triggerFakeMouseEvent("mouseup", component.background(), endPoint.x, endPoint.y, 2);
                    chai_1.assert.isFalse(callback.called, "callback was not called on mouseup from the right-click button");
                    // end the drag
                    TestMethods.triggerFakeMouseEvent("mouseup", component.background(), endPoint.x, endPoint.y);
                });
            });
        });
        describe("constrainedToComponent", function () {
            it("is true by default", function () {
                chai_1.assert.isTrue(dragInteraction.constrainedToComponent(), "constrains by default");
            });
            it("can be set to false", function () {
                chai_1.assert.strictEqual(dragInteraction.constrainedToComponent(false), dragInteraction, "setter returns calling Drag Interaction");
                chai_1.assert.isFalse(dragInteraction.constrainedToComponent(), "constrains set to false");
            });
            [TestMethods.InteractionMode.Mouse, TestMethods.InteractionMode.Touch].forEach(function (mode) {
                describe("invoking callbacks with " + TestMethods.InteractionMode[mode] + " events when not constrained", function () {
                    var callback;
                    beforeEach(function () {
                        callback = makeDragCallback();
                        dragInteraction.constrainedToComponent(false);
                        dragInteraction.onDrag(callback);
                        dragInteraction.onDragEnd(callback);
                    });
                    it("does not constrain dragging for onDrag outside Component in positive direction", function () {
                        triggerFakeDragMove(startPoint, positiveOutsidePoint);
                        chai_1.assert.deepEqual(callback.lastEndPoint, positiveOutsidePoint, "was passed the correct end point");
                    });
                    it("does not constrain dragging for onDrag outside Component in negative direction", function () {
                        triggerFakeDragMove(startPoint, negativeOutsidePoint);
                        chai_1.assert.deepEqual(callback.lastEndPoint, negativeOutsidePoint, "was passed the correct end point");
                    });
                    it("does not constrain dragging for onDragEnd outside Component in positive direction", function () {
                        triggerFakeDragEnd(startPoint, positiveOutsidePoint);
                        chai_1.assert.deepEqual(callback.lastEndPoint, positiveOutsidePoint, "was passed the correct end point");
                    });
                    it("does not constrain dragging for onDragEnd outside Component in negative direction", function () {
                        triggerFakeDragEnd(startPoint, negativeOutsidePoint);
                        chai_1.assert.deepEqual(callback.lastEndPoint, negativeOutsidePoint, "was passed the correct end point");
                    });
                });
                describe("invoking callbacks with " + TestMethods.InteractionMode[mode] + " events when constrained", function () {
                    var constrainedPos = { x: DIV_WIDTH, y: DIV_HEIGHT };
                    var constrainedNeg = { x: 0, y: 0 };
                    var callback;
                    beforeEach(function () {
                        callback = makeDragCallback();
                        dragInteraction.onDrag(callback);
                        dragInteraction.onDragEnd(callback);
                    });
                    it("constrains dragging for onDrag outside Component in positive direction", function () {
                        triggerFakeDragMove(startPoint, positiveOutsidePoint);
                        chai_1.assert.deepEqual(callback.lastEndPoint, constrainedPos, "was passed the correct end point");
                    });
                    it("constrains dragging for onDrag outside Component in negative direction", function () {
                        triggerFakeDragMove(startPoint, negativeOutsidePoint);
                        chai_1.assert.deepEqual(callback.lastEndPoint, constrainedNeg, "was passed the correct end point");
                    });
                    it("constrains dragging for onDragEnd outside Component in positive direction", function () {
                        triggerFakeDragEnd(startPoint, positiveOutsidePoint);
                        chai_1.assert.deepEqual(callback.lastEndPoint, constrainedPos, "was passed the correct end point");
                    });
                    it("constrains dragging for onDragEnd outside Component in negative direction", function () {
                        triggerFakeDragEnd(startPoint, negativeOutsidePoint);
                        chai_1.assert.deepEqual(callback.lastEndPoint, constrainedNeg, "was passed the correct end point");
                    });
                });
            });
        });
    });
});
