"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
var barPlot_1 = require("../../src/plots/barPlot");
describe("Plots", function () {
    describe("Bar Plot", function () {
        describe("setting orientation", function () {
            it("rejects invalid orientations", function () {
                chai_1.assert.throws(function () { return new Plottable.Plots.Bar("diagonal"); }, Error);
            });
            it("defaults to vertical", function () {
                var defaultPlot = new Plottable.Plots.Bar();
                chai_1.assert.strictEqual(defaultPlot.orientation(), "vertical", "default Plots.Bar() are vertical");
            });
            it("sets orientation on construction", function () {
                var verticalPlot = new Plottable.Plots.Bar("vertical");
                chai_1.assert.strictEqual(verticalPlot.orientation(), "vertical", "vertical Plots.Bar()");
                var horizontalPlot = new Plottable.Plots.Bar("horizontal");
                chai_1.assert.strictEqual(horizontalPlot.orientation(), "horizontal", "horizontal Plots.Bar()");
            });
        });
        var orientations = [barPlot_1.BarOrientation.vertical, barPlot_1.BarOrientation.horizontal];
        orientations.forEach(function (orientation) {
            var isVertical = orientation === barPlot_1.BarOrientation.vertical;
            var basePositionAttr = isVertical ? "x" : "y";
            var baseSizeAttr = isVertical ? "width" : "height";
            var getDivBaseSizeDimension = function (div) {
                return isVertical ? Plottable.Utils.DOM.elementWidth(div) : Plottable.Utils.DOM.elementHeight(div);
            };
            var valuePositionAttr = isVertical ? "y" : "x";
            var valueSizeAttr = isVertical ? "height" : "width";
            var getValueBaseSizeDimension = function (div) {
                return isVertical ? Plottable.Utils.DOM.elementHeight(div) : Plottable.Utils.DOM.elementWidth(div);
            };
            describe("rendering when " + orientation, function () {
                var data = [
                    { base: "A", value: 1 },
                    { base: "B", value: 0 },
                    { base: "C", value: -1 },
                ];
                var div;
                var barPlot;
                var baseScale;
                var valueScale;
                var dataset;
                beforeEach(function () {
                    div = TestMethods.generateDiv();
                    barPlot = new Plottable.Plots.Bar(orientation);
                    baseScale = new Plottable.Scales.Category();
                    valueScale = new Plottable.Scales.Linear();
                    if (orientation === barPlot_1.BarOrientation.vertical) {
                        barPlot.x(function (d) { return d.base; }, baseScale);
                        barPlot.y(function (d) { return d.value; }, valueScale);
                    }
                    else {
                        barPlot.y(function (d) { return d.base; }, baseScale);
                        barPlot.x(function (d) { return d.value; }, valueScale);
                    }
                    dataset = new Plottable.Dataset(data);
                });
                afterEach(function () {
                    if (this.currentTest.state === "passed") {
                        barPlot.destroy();
                        div.remove();
                    }
                });
                function assertCorrectRendering() {
                    var baseline = barPlot.content().select(".baseline");
                    var scaledBaselineValue = valueScale.scale(barPlot.baselineValue());
                    chai_1.assert.strictEqual(TestMethods.numAttr(baseline, valuePositionAttr + "1"), scaledBaselineValue, "baseline " + valuePositionAttr + "1 is correct");
                    chai_1.assert.strictEqual(TestMethods.numAttr(baseline, valuePositionAttr + "2"), scaledBaselineValue, "baseline " + valuePositionAttr + "2 is correct");
                    chai_1.assert.strictEqual(TestMethods.numAttr(baseline, basePositionAttr + "1"), 0, "baseline " + basePositionAttr + "1 is correct");
                    chai_1.assert.strictEqual(TestMethods.numAttr(baseline, basePositionAttr + "2"), getDivBaseSizeDimension(div), "baseline " + basePositionAttr + "2 is correct");
                    var bars = barPlot.content().selectAll("rect");
                    chai_1.assert.strictEqual(bars.size(), data.length, "One bar was created per data point");
                    bars.each(function (datum, index) {
                        var bar = d3.select(this);
                        var baseSize = TestMethods.numAttr(bar, baseSizeAttr);
                        chai_1.assert.closeTo(baseSize, baseScale.rangeBand(), window.Pixel_CloseTo_Requirement, "bar " + baseSizeAttr + " is correct (index " + index + ")");
                        var valueSize = TestMethods.numAttr(bar, valueSizeAttr);
                        chai_1.assert.closeTo(valueSize, Math.abs(valueScale.scale(datum.value) - scaledBaselineValue), window.Pixel_CloseTo_Requirement, "bar " + valueSizeAttr + " is correct (index " + index + ")");
                        var basePosition = TestMethods.numAttr(bar, basePositionAttr);
                        chai_1.assert.closeTo(basePosition, baseScale.scale(datum.base) - 0.5 * baseSize, window.Pixel_CloseTo_Requirement, "bar " + basePositionAttr + " is correct (index " + index + ")");
                        var valuePosition = TestMethods.numAttr(bar, valuePositionAttr);
                        var isShifted = isVertical ? (datum.value > barPlot.baselineValue()) : (datum.value < barPlot.baselineValue());
                        var expectedValuePosition = isShifted ? scaledBaselineValue - valueSize : scaledBaselineValue;
                        chai_1.assert.closeTo(valuePosition, expectedValuePosition, window.Pixel_CloseTo_Requirement, "bar " + valuePositionAttr + " is correct (index " + index + ")");
                    });
                }
                it("renders with no data", function () {
                    chai_1.assert.doesNotThrow(function () { return barPlot.renderTo(div); }, Error);
                    chai_1.assert.strictEqual(barPlot.width(), Plottable.Utils.DOM.elementWidth(div), "was allocated width");
                    chai_1.assert.strictEqual(barPlot.height(), Plottable.Utils.DOM.elementHeight(div), "was allocated height");
                });
                it("draws bars and baseline in correct positions", function () {
                    barPlot.addDataset(dataset);
                    barPlot.renderTo(div);
                    assertCorrectRendering();
                });
                it("rerenders correctly when the baseline value is changed", function () {
                    barPlot.addDataset(dataset);
                    barPlot.renderTo(div);
                    barPlot.baselineValue(1);
                    assertCorrectRendering();
                });
                it("can autorange value scale based on visible points on base scale", function () {
                    var firstTwoBaseValues = [data[0].base, data[1].base];
                    valueScale.padProportion(0);
                    baseScale.domain(firstTwoBaseValues);
                    barPlot.addDataset(dataset);
                    barPlot.autorangeMode(valuePositionAttr);
                    barPlot.renderTo(div);
                    var valueScaleDomain = valueScale.domain();
                    var expectedValueDomainMin = Math.min(data[0].value, data[1].value);
                    var expectedValueDomainMax = Math.max(data[0].value, data[1].value);
                    chai_1.assert.strictEqual(valueScaleDomain[0], expectedValueDomainMin, "lower bound of domain set based on visible points");
                    chai_1.assert.strictEqual(valueScaleDomain[1], expectedValueDomainMax, "upper bound of domain set based on visible points");
                });
                it("doesn't show values from outside the base scale's domain", function () {
                    baseScale.domain(["-A"]);
                    barPlot.addDataset(dataset);
                    barPlot.renderTo(div);
                    chai_1.assert.strictEqual(barPlot.content().selectAll("rect").size(), 0, "draws no bars when the domain contains no data points");
                });
            });
            describe("autodomaining when " + orientation, function () {
                var div;
                var baseScale;
                var valueScale;
                var barPlot;
                var baseAccessor = function (d) { return d.base; };
                var valueAccessor = function (d) { return d.value; };
                beforeEach(function () {
                    div = TestMethods.generateDiv();
                    barPlot = new Plottable.Plots.Bar(orientation);
                    baseScale = new Plottable.Scales.Linear();
                    valueScale = new Plottable.Scales.Linear();
                    if (orientation === barPlot_1.BarOrientation.vertical) {
                        barPlot.x(baseAccessor, baseScale);
                        barPlot.y(valueAccessor, valueScale);
                    }
                    else {
                        barPlot.y(baseAccessor, baseScale);
                        barPlot.x(valueAccessor, valueScale);
                    }
                });
                afterEach(function () {
                    if (this.currentTest.state === "passed") {
                        barPlot.destroy();
                        div.remove();
                    }
                });
                it("computes the base scale domain correctly when there is only one data point", function () {
                    var singlePointData = [
                        { base: baseScale.domain()[1] + 10, value: 5 },
                    ];
                    barPlot.addDataset(new Plottable.Dataset(singlePointData));
                    barPlot.renderTo(div);
                    var baseScaleDomain = baseScale.domain();
                    chai_1.assert.operator(baseScaleDomain[0], "<=", singlePointData[0].base, "lower end of base domain is less than the value");
                    chai_1.assert.operator(baseScaleDomain[1], ">=", singlePointData[0].base, "upper end of base domain is greater than the value");
                });
                it("base scale domain does not change when autoDomain is called more than once", function () {
                    var data = [
                        { base: 0, value: 1 },
                        { base: 1, value: 2 },
                    ];
                    barPlot.addDataset(new Plottable.Dataset(data));
                    barPlot.renderTo(div);
                    var baseDomainAfterRendering = baseScale.domain();
                    baseScale.autoDomain();
                    var baseDomainAfterAutoDomaining = baseScale.domain();
                    chai_1.assert.deepEqual(baseDomainAfterRendering, baseDomainAfterAutoDomaining, "calling autoDomain() again does not change the domain");
                });
            });
            describe("auto bar width calculation when " + orientation, function () {
                var scaleTypes = ["Linear", "ModifiedLog", "Time"];
                scaleTypes.forEach(function (scaleType) {
                    describe("using a " + scaleType + " base Scale", function () {
                        var div;
                        var barPlot;
                        var baseScale;
                        var valueScale;
                        var dataset;
                        beforeEach(function () {
                            div = TestMethods.generateDiv();
                            barPlot = new Plottable.Plots.Bar(orientation);
                            switch (scaleType) {
                                case "Linear":
                                    baseScale = new Plottable.Scales.Linear();
                                    break;
                                case "ModifiedLog":
                                    baseScale = new Plottable.Scales.ModifiedLog();
                                    break;
                                case "Time":
                                    baseScale = new Plottable.Scales.Time();
                                    break;
                                default:
                                    throw new Error("unexpected base Scale type");
                            }
                            valueScale = new Plottable.Scales.Linear();
                            var baseAccessor = scaleType === "Time" ? function (d) { return new Date(d.base); } : function (d) { return d.base; };
                            var valueAccessor = function (d) { return d.value; };
                            if (orientation === barPlot_1.BarOrientation.vertical) {
                                barPlot.x(baseAccessor, baseScale);
                                barPlot.y(valueAccessor, valueScale);
                            }
                            else {
                                barPlot.y(baseAccessor, baseScale);
                                barPlot.x(valueAccessor, valueScale);
                            }
                            dataset = new Plottable.Dataset();
                            barPlot.addDataset(dataset);
                            barPlot.renderTo(div);
                        });
                        afterEach(function () {
                            if (this.currentTest.state === "passed") {
                                barPlot.destroy();
                                div.remove();
                            }
                        });
                        it("computes a sensible width", function () {
                            var data = [
                                { base: 1, value: 5 },
                                { base: 10, value: 2 },
                                { base: 100, value: 4 },
                            ];
                            dataset.data(data);
                            var closestSeparation = Math.abs(baseScale.scale(data[1].base) - baseScale.scale(data[0].base));
                            var bars = barPlot.content().selectAll("rect");
                            chai_1.assert.strictEqual(bars.size(), data.length, "one bar was drawn per datum");
                            bars.each(function () {
                                var bar = d3.select(this);
                                var barSize = TestMethods.numAttr(bar, baseSizeAttr);
                                chai_1.assert.operator(barSize, "<=", closestSeparation, "bar width is less than the closest distance between values");
                                chai_1.assert.operator(barSize, ">=", 0.5 * closestSeparation, "bar width is greater than half the closest distance between values");
                            });
                        });
                        it("accounts for the bar width when autoDomaining the base scale", function () {
                            var data = [
                                { base: 1, value: 5 },
                                { base: 10, value: 2 },
                                { base: 100, value: 4 },
                            ];
                            dataset.data(data);
                            var bars = barPlot.content().selectAll("rect");
                            chai_1.assert.strictEqual(bars.size(), data.length, "one bar was drawn per datum");
                            var divSize = getDivBaseSizeDimension(div);
                            bars.each(function () {
                                var bar = d3.select(this);
                                var barPosition = TestMethods.numAttr(bar, basePositionAttr);
                                var barSize = TestMethods.numAttr(bar, baseSizeAttr);
                                chai_1.assert.operator(barPosition, ">=", 0, "bar is within visible area (" + basePositionAttr + ")");
                                chai_1.assert.operator(barPosition, "<=", divSize, "bar is within visible area (" + basePositionAttr + ")");
                                chai_1.assert.operator(barPosition + barSize, ">=", 0, "bar is within visible area (" + baseSizeAttr + ")");
                                chai_1.assert.operator(barPosition + barSize, "<=", divSize, "bar is within visible area (" + baseSizeAttr + ")");
                            });
                        });
                        it("does not crash when given bad data", function () {
                            var badData = [
                                {},
                                { base: null, value: null },
                            ];
                            chai_1.assert.doesNotThrow(function () { return dataset.data(badData); }, Error);
                        });
                        it("computes a sensible width when given only one datum", function () {
                            var singleDatumData = [
                                { base: 1, value: 5 },
                            ];
                            dataset.data(singleDatumData);
                            var bar = barPlot.content().select("rect");
                            var barSize = TestMethods.numAttr(bar, baseSizeAttr);
                            var divSize = getDivBaseSizeDimension(div);
                            chai_1.assert.operator(barSize, ">=", divSize / 4, "bar is larger than 1/4 of the available space");
                            chai_1.assert.operator(barSize, "<=", divSize / 2, "bar is smaller than 1/2 of the available space");
                        });
                        it("computes a sensible width when given repeated base value", function () {
                            var repeatedBaseData = [
                                { base: 1, value: 5 },
                                { base: 1, value: -5 },
                            ];
                            dataset.data(repeatedBaseData);
                            var bars = barPlot.content().selectAll("rect");
                            chai_1.assert.strictEqual(bars.size(), repeatedBaseData.length, "one bar was drawn per datum");
                            var divSize = getDivBaseSizeDimension(div);
                            bars.each(function () {
                                var bar = d3.select(this);
                                var barSize = TestMethods.numAttr(bar, baseSizeAttr);
                                chai_1.assert.operator(barSize, ">=", divSize / 4, "bar is larger than 1/4 of the available space");
                                chai_1.assert.operator(barSize, "<=", divSize / 2, "bar is smaller than 1/2 of the available space");
                            });
                        });
                        it("computes a sensible width when given unsorted data", function () {
                            var unsortedData = [
                                { base: 10, value: 2 },
                                { base: 1, value: 5 },
                                { base: 100, value: 4 },
                            ];
                            dataset.data(unsortedData);
                            var closestSeparation = Math.abs(baseScale.scale(unsortedData[1].base) - baseScale.scale(unsortedData[0].base));
                            var bars = barPlot.content().selectAll("rect");
                            chai_1.assert.strictEqual(bars.size(), unsortedData.length, "one bar was drawn per datum");
                            bars.each(function () {
                                var bar = d3.select(this);
                                var barSize = TestMethods.numAttr(bar, baseSizeAttr);
                                chai_1.assert.operator(barSize, "<=", closestSeparation, "bar width is less than the closest distance between values");
                                chai_1.assert.operator(barSize, ">=", 0.5 * closestSeparation, "bar width is greater than half the closest distance between values");
                            });
                        });
                    });
                });
            });
            describe("labels when " + orientation, function () {
                var data = [
                    { base: -4, value: -4 },
                    { base: -2, value: -0.1 },
                    { base: 0, value: 0 },
                    { base: 2, value: 0.1 },
                    { base: 4, value: 4 },
                ];
                var DEFAULT_DOMAIN = [-5, 5];
                var div;
                var baseScale;
                var valueScale;
                var barPlot;
                var dataset;
                beforeEach(function () {
                    div = TestMethods.generateDiv();
                    barPlot = new Plottable.Plots.Bar(orientation);
                    baseScale = new Plottable.Scales.Linear();
                    baseScale.domain(DEFAULT_DOMAIN);
                    valueScale = new Plottable.Scales.Linear();
                    valueScale.domain(DEFAULT_DOMAIN);
                    if (orientation === barPlot_1.BarOrientation.vertical) {
                        barPlot.x(function (d) { return d.base; }, baseScale);
                        barPlot.y(function (d) { return d.value; }, valueScale);
                    }
                    else {
                        barPlot.y(function (d) { return d.base; }, baseScale);
                        barPlot.x(function (d) { return d.value; }, valueScale);
                    }
                    dataset = new Plottable.Dataset(data);
                    barPlot.addDataset(dataset);
                    barPlot.renderTo(div);
                });
                afterEach(function () {
                    if (this.currentTest.state === "passed") {
                        barPlot.destroy();
                        div.remove();
                    }
                });
                function getCenterOfText(textNode) {
                    var plotBoundingClientRect = barPlot.background().node().getBoundingClientRect();
                    var labelBoundingClientRect = textNode.getBoundingClientRect();
                    return {
                        x: (labelBoundingClientRect.left + labelBoundingClientRect.right) / 2 - plotBoundingClientRect.left,
                        y: (labelBoundingClientRect.top + labelBoundingClientRect.bottom) / 2 - plotBoundingClientRect.top,
                    };
                }
                ;
                it("does not show labels by default", function () {
                    var texts = barPlot.content().selectAll("text");
                    chai_1.assert.strictEqual(texts.size(), 0, "by default, no texts are drawn");
                });
                it("draws one label per datum", function () {
                    barPlot.labelsEnabled(true);
                    var texts = barPlot.content().selectAll("text");
                    chai_1.assert.strictEqual(texts.size(), data.length, "one label drawn per datum");
                    texts.each(function (d, i) {
                        chai_1.assert.strictEqual(d3.select(this).text(), data[i].value.toString(), "by default, label text is the bar's value (index " + i + ")");
                    });
                });
                it("hides the labels if bars are too thin to show them", function () {
                    div.style(baseSizeAttr, getDivBaseSizeDimension(div) / 10 + "px");
                    barPlot.redraw();
                    barPlot.labelsEnabled(true);
                    var texts = barPlot.content().selectAll("text");
                    chai_1.assert.strictEqual(texts.size(), 0, "no labels drawn");
                });
                it("can apply a formatter to the labels", function () {
                    barPlot.labelsEnabled(true);
                    var formatter = function (n) { return n + "%"; };
                    barPlot.labelFormatter(formatter);
                    var texts = barPlot.content().selectAll("text");
                    chai_1.assert.strictEqual(texts.size(), data.length, "one label drawn per datum");
                    var expectedTexts = data.map(function (d) { return formatter(d.value); });
                    texts.each(function (d, i) {
                        chai_1.assert.strictEqual(d3.select(this).text(), expectedTexts[i], "formatter is applied to the displayed value (index " + i + ")");
                    });
                });
                it("shows labels inside or outside the bar as appropriate", function () {
                    barPlot.labelsEnabled(true);
                    var labels = barPlot.content().selectAll(".on-bar-label, .off-bar-label");
                    chai_1.assert.strictEqual(labels.size(), data.length, "one label drawn per datum");
                    var bars = barPlot.content().select(".bar-area").selectAll("rect");
                    labels.each(function (d, i) {
                        var labelBoundingClientRect = labels.nodes()[i].getBoundingClientRect();
                        var barBoundingClientRect = bars.nodes()[i].getBoundingClientRect();
                        if (labelBoundingClientRect[valueSizeAttr] > barBoundingClientRect[valueSizeAttr]) {
                            chai_1.assert.isTrue(d3.select(labels.nodes()[i]).classed("off-bar-label"), "label with index " + i + " doesn't fit and carries the off-bar class");
                        }
                        else {
                            chai_1.assert.isTrue(d3.select(labels.nodes()[i]).classed("on-bar-label"), "label with index " + i + " fits and carries the on-bar class");
                        }
                    });
                });
                it("shows labels for bars with value = baseline on the \"positive\" side of the baseline", function () {
                    var zeroOnlyData = [{ base: 0, value: 0 }];
                    dataset.data(zeroOnlyData);
                    barPlot.labelsEnabled(true);
                    var label = barPlot.content().select("text");
                    var labelBoundingRect = label.node().getBoundingClientRect();
                    var lineBoundingRect = barPlot.content().select(".baseline").node().getBoundingClientRect();
                    if (isVertical) {
                        var labelPosition = labelBoundingRect.bottom - window.Pixel_CloseTo_Requirement;
                        var linePosition = lineBoundingRect.top;
                        chai_1.assert.operator(labelPosition, "<=", linePosition, "label with value = baseline is drawn above the baseline");
                    }
                    else {
                        var labelPosition = labelBoundingRect.left + window.Pixel_CloseTo_Requirement;
                        var linePosition = lineBoundingRect.right;
                        chai_1.assert.operator(labelPosition, ">=", linePosition, "label with value = baseline is drawn to the right of the baseline");
                    }
                });
                it("hides labels cut off by lower end of base scale", function () {
                    barPlot.labelsEnabled(true);
                    data.forEach(function (d, i) {
                        var textsBeforeRendering = barPlot.content().selectAll("text");
                        var centerOfText = getCenterOfText(textsBeforeRendering.nodes()[i]);
                        var centerValue = baseScale.invert(isVertical ? centerOfText.x : centerOfText.y);
                        baseScale.domain([centerValue, centerValue + (DEFAULT_DOMAIN[1] - DEFAULT_DOMAIN[0])]);
                        var textsAfterRendering = barPlot.content().selectAll("text");
                        chai_1.assert.strictEqual(d3.select(textsAfterRendering.nodes()[i]).style("visibility"), "hidden", "label for bar with index " + i + " is hidden");
                    });
                    div.remove();
                });
                it("hides labels cut off by upper end of base scale", function () {
                    barPlot.labelsEnabled(true);
                    data.forEach(function (d, i) {
                        var textsBeforeRendering = barPlot.content().selectAll("text");
                        var centerOfText = getCenterOfText(textsBeforeRendering.nodes()[i]);
                        var centerValue = baseScale.invert(isVertical ? centerOfText.x : centerOfText.y);
                        baseScale.domain([centerValue - (DEFAULT_DOMAIN[1] - DEFAULT_DOMAIN[0]), centerValue]);
                        var textsAfterRendering = barPlot.content().selectAll("text");
                        chai_1.assert.strictEqual(d3.select(textsAfterRendering.nodes()[i]).style("visibility"), "hidden", "label for bar with index " + i + " is hidden");
                    });
                });
                it("hides or shifts labels cut off by lower end of value scale", function () {
                    barPlot.labelsEnabled(true);
                    var labelsBeforeRendering = barPlot.content().selectAll(".on-bar-label, .off-bar-label");
                    var centerValues = labelsBeforeRendering.select("text").nodes().map(function (textNode) {
                        var centerOfText = getCenterOfText(textNode);
                        return valueScale.invert(isVertical ? centerOfText.y : centerOfText.x);
                    });
                    var wasOriginallyOnBar = labelsBeforeRendering.nodes().map(function (label) { return d3.select(label).classed("on-bar-label"); });
                    data.forEach(function (d, i) {
                        var centerValue = centerValues[i];
                        valueScale.domain([centerValue, centerValue + (DEFAULT_DOMAIN[1] - DEFAULT_DOMAIN[0])]);
                        var labels = barPlot.content().selectAll(".on-bar-label, .off-bar-label");
                        if (wasOriginallyOnBar[i] && d.value > 0) {
                            chai_1.assert.isTrue(d3.select(labels.nodes()[i]).classed("off-bar-label"), "cut off on-bar label was switched to off-bar (index " + i + ")");
                        }
                        else {
                            var textNode = labels.select("text").nodes()[i];
                            chai_1.assert.strictEqual(d3.select(textNode).style("visibility"), "hidden", "label for bar with index " + i + " is hidden");
                        }
                    });
                    div.remove();
                });
                it("hides or shifts labels cut off by upper end of value scale", function () {
                    barPlot.labelsEnabled(true);
                    var labelsBeforeRendering = barPlot.content().selectAll(".on-bar-label, .off-bar-label");
                    var centerValues = labelsBeforeRendering.select("text").nodes().map(function (textNode) {
                        var centerOfText = getCenterOfText(textNode);
                        return valueScale.invert(isVertical ? centerOfText.y : centerOfText.x);
                    });
                    var wasOriginallyOnBar = labelsBeforeRendering.nodes().map(function (label) { return d3.select(label).classed("on-bar-label"); });
                    data.forEach(function (d, i) {
                        var centerValue = centerValues[i];
                        valueScale.domain([centerValue - (DEFAULT_DOMAIN[1] - DEFAULT_DOMAIN[0]), centerValue]);
                        var labels = barPlot.content().selectAll(".on-bar-label, .off-bar-label");
                        if (wasOriginallyOnBar[i] && d.value < 0) {
                            chai_1.assert.isTrue(d3.select(labels.nodes()[i]).classed("off-bar-label"), "cut-off on-bar label was switched to off-bar (index " + i + ")");
                        }
                        else {
                            var textNode = labels.select("text").nodes()[i];
                            chai_1.assert.strictEqual(d3.select(textNode).style("visibility"), "hidden", "label for bar with index " + i + " is hidden");
                        }
                    });
                });
                // HACKHACK: This test is a bit hacky, but it seems to be testing for a bug fixed in
                // https://github.com/palantir/plottable/pull/1240 . Leaving it until we find a better way to test for it.
                it("removes labels instantly on dataset change", function (done) {
                    barPlot.labelsEnabled(true);
                    var texts = barPlot.content().selectAll("text");
                    chai_1.assert.strictEqual(texts.size(), dataset.data().length, "one label drawn per datum");
                    var originalDrawLabels = barPlot._drawLabels;
                    var called = false;
                    barPlot._drawLabels = function () {
                        if (!called) {
                            originalDrawLabels.apply(barPlot);
                            texts = barPlot.content().selectAll("text");
                            chai_1.assert.strictEqual(texts.size(), dataset.data().length, "texts were repopulated by drawLabels after the update");
                            called = true; // for some reason, in phantomJS, `done` was being called multiple times and this caused the test to fail.
                            done();
                        }
                    };
                    dataset.data(dataset.data());
                    texts = barPlot.content().selectAll("text");
                    chai_1.assert.strictEqual(texts.size(), 0, "texts were immediately removed");
                });
            });
            describe("retrieving Entities when " + orientation, function () {
                var data = [
                    { base: -1, value: -1 },
                    { base: 0, value: 0 },
                    { base: 1, value: 1 },
                ];
                var DEFAULT_DOMAIN = [-2, 2];
                var div;
                var barPlot;
                var baseScale;
                var valueScale;
                var dataset;
                beforeEach(function () {
                    div = TestMethods.generateDiv();
                    barPlot = new Plottable.Plots.Bar(orientation);
                    baseScale = new Plottable.Scales.Linear();
                    baseScale.domain(DEFAULT_DOMAIN);
                    valueScale = new Plottable.Scales.Linear();
                    valueScale.domain(DEFAULT_DOMAIN);
                    if (orientation === barPlot_1.BarOrientation.vertical) {
                        barPlot.x(function (d) { return d.base; }, baseScale);
                        barPlot.y(function (d) { return d.value; }, valueScale);
                    }
                    else {
                        barPlot.y(function (d) { return d.base; }, baseScale);
                        barPlot.x(function (d) { return d.value; }, valueScale);
                    }
                    dataset = new Plottable.Dataset(data);
                    barPlot.addDataset(dataset);
                    barPlot.renderTo(div);
                });
                afterEach(function () {
                    if (this.currentTest.state === "passed") {
                        barPlot.destroy();
                        div.remove();
                    }
                });
                it("returns the correct position for each Entity", function () {
                    var entities = barPlot.entities();
                    entities.forEach(function (entity, index) {
                        var xBinding = barPlot.x();
                        var yBinding = barPlot.y();
                        var scaledDataX = xBinding.scale.scale(xBinding.accessor(entity.datum, index, dataset));
                        var scaledDataY = yBinding.scale.scale(yBinding.accessor(entity.datum, index, dataset));
                        chai_1.assert.strictEqual(scaledDataX, entity.position.x, "entities().position.x is equal to scaled x value");
                        chai_1.assert.strictEqual(scaledDataY, entity.position.y, "entities().position.y is equal to scaled y value");
                    });
                });
                function getPointFromBaseAndValuePositions(basePosition, valuePosition) {
                    return {
                        x: isVertical ? basePosition : valuePosition,
                        y: isVertical ? valuePosition : basePosition,
                    };
                }
                function expectedEntityForIndex(index) {
                    var datum = data[index];
                    var basePosition = baseScale.scale(datum.base);
                    var valuePosition = valueScale.scale(datum.value);
                    return {
                        datum: datum,
                        index: index,
                        dataset: dataset,
                        datasetIndex: 0,
                        position: getPointFromBaseAndValuePositions(basePosition, valuePosition),
                        selection: d3.select(barPlot.content().selectAll("rect").nodes()[index]),
                        component: barPlot,
                    };
                }
                describe("retrieving the nearest Entity", function () {
                    function testEntityNearest() {
                        data.forEach(function (datum, index) {
                            var expectedEntity = expectedEntityForIndex(index);
                            var barBasePosition = baseScale.scale(datum.base);
                            var halfwayValuePosition = valueScale.scale((barPlot.baselineValue() + datum.value) / 2);
                            var pointInsideBar = getPointFromBaseAndValuePositions(barBasePosition, halfwayValuePosition);
                            var nearestInsideBar = barPlot.entityNearest(pointInsideBar);
                            TestMethods.assertPlotEntitiesEqual(nearestInsideBar, expectedEntity, "retrieves the Entity for a bar if inside the bar");
                            var abovePosition = valueScale.scale(2 * datum.value);
                            var pointAboveBar = getPointFromBaseAndValuePositions(barBasePosition, abovePosition);
                            var nearestAboveBar = barPlot.entityNearest(pointAboveBar);
                            TestMethods.assertPlotEntitiesEqual(nearestAboveBar, expectedEntity, "retrieves the Entity for a bar if beyond the end of the bar");
                            var belowPosition = valueScale.scale(-datum.value);
                            var pointBelowBar = getPointFromBaseAndValuePositions(barBasePosition, belowPosition);
                            var nearestBelowBar = barPlot.entityNearest(pointBelowBar);
                            TestMethods.assertPlotEntitiesEqual(nearestBelowBar, expectedEntity, "retrieves the Entity for a bar if on the other side of the baseline from the bar");
                        });
                    }
                    it("returns the closest by base, then by value", function () {
                        testEntityNearest();
                    });
                    it("returns the closest visible bar", function () {
                        var baseValuebetweenBars = (data[0].base + data[1].base) / 2;
                        baseScale.domain([baseValuebetweenBars, DEFAULT_DOMAIN[1]]); // cuts off bar 0
                        var bar0BasePosition = baseScale.scale(data[0].base);
                        var baselineValuePosition = valueScale.scale(barPlot.baselineValue());
                        var nearestEntity = barPlot.entityNearest(getPointFromBaseAndValuePositions(bar0BasePosition, baselineValuePosition));
                        var expectedEntity = expectedEntityForIndex(1); // nearest visible bar
                        TestMethods.assertPlotEntitiesEqual(nearestEntity, expectedEntity, "returned Entity for nearest in-view bar");
                    });
                    it("considers bars cut off by the value scale", function () {
                        valueScale.domain([-0.5, 0.5]);
                        testEntityNearest();
                    });
                    it("considers bars cut off by the base scale", function () {
                        baseScale.domain([-0.8, 0.8]);
                        testEntityNearest();
                    });
                    it("returns undefined if no bars are visible", function () {
                        baseScale.domain([100, 200]);
                        var centerOfPlot = {
                            x: barPlot.width() / 2,
                            y: barPlot.height() / 2,
                        };
                        var nearestEntity = barPlot.entityNearest(centerOfPlot);
                        chai_1.assert.isUndefined(nearestEntity, "returns undefined when no bars are in view");
                    });
                });
                describe("retrieving the Entities at a given point", function () {
                    it("returns the Entity at a given point, or an empty array if no bars are there", function () {
                        data.forEach(function (datum, index) {
                            if (datum.value === barPlot.baselineValue()) {
                                return; // bar has no height
                            }
                            var expectedEntity = expectedEntityForIndex(index);
                            var barBasePosition = baseScale.scale(datum.base);
                            var halfwayValuePosition = valueScale.scale((barPlot.baselineValue() + datum.value) / 2);
                            var pointInsideBar = getPointFromBaseAndValuePositions(barBasePosition, halfwayValuePosition);
                            var entitiesAtPointInside = barPlot.entitiesAt(pointInsideBar);
                            chai_1.assert.lengthOf(entitiesAtPointInside, 1, "exactly 1 Entity was returned (index " + index + ")");
                            TestMethods.assertPlotEntitiesEqual(entitiesAtPointInside[0], expectedEntity, "retrieves the Entity for a bar if inside the bar (index " + index + ")");
                        });
                        var pointOutsideBars = { x: -1, y: -1 };
                        var entitiesAtPointOutside = barPlot.entitiesAt(pointOutsideBars);
                        chai_1.assert.lengthOf(entitiesAtPointOutside, 0, "no Entities returned if no bars at query point");
                    });
                });
                describe("retrieving the Entities in a given range", function () {
                    it("returns the Entities for any bars that intersect the range", function () {
                        var bar0 = barPlot.content().select("rect");
                        var bar0Edge = TestMethods.numAttr(bar0, basePositionAttr);
                        var bar0FarEdge = TestMethods.numAttr(bar0, basePositionAttr) + TestMethods.numAttr(bar0, baseSizeAttr);
                        var baseRange = {
                            min: bar0Edge,
                            max: bar0FarEdge,
                        };
                        var fullSizeValueRange = {
                            min: -Infinity,
                            max: Infinity,
                        };
                        var entitiesInRange = isVertical ? barPlot.entitiesIn(baseRange, fullSizeValueRange)
                            : barPlot.entitiesIn(fullSizeValueRange, baseRange);
                        chai_1.assert.lengthOf(entitiesInRange, 1, "retrieved two Entities when range intersects one bar");
                        TestMethods.assertPlotEntitiesEqual(entitiesInRange[0], expectedEntityForIndex(0), "Entity corresponds to bar 0");
                    });
                    it("returns the Entity if the range includes any part of the bar", function () {
                        var quarterUpBar0 = valueScale.scale(data[0].value / 4);
                        var halfUpBar0 = valueScale.scale(data[0].value / 2);
                        var valueRange = {
                            min: Math.min(quarterUpBar0, halfUpBar0),
                            max: Math.max(quarterUpBar0, halfUpBar0),
                        };
                        var fullSizeBaseRange = {
                            min: -Infinity,
                            max: Infinity,
                        };
                        var entitiesInRange = isVertical ? barPlot.entitiesIn(fullSizeBaseRange, valueRange)
                            : barPlot.entitiesIn(valueRange, fullSizeBaseRange);
                        chai_1.assert.lengthOf(entitiesInRange, 1, "retrieved one entity when range intersects one bar");
                        TestMethods.assertPlotEntitiesEqual(entitiesInRange[0], expectedEntityForIndex(0), "Entity corresponds to bar 0");
                    });
                });
            });
        });
    });
});
