"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
var barPlot_1 = require("../../src/plots/barPlot");
describe("Plots", function () {
    [barPlot_1.BarOrientation.vertical, barPlot_1.BarOrientation.horizontal].forEach(function (orientation) {
        describe("Clustered Bar Plot in " + orientation + " orientation", function () {
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 400;
            var isVertical = orientation === barPlot_1.BarOrientation.vertical;
            var div;
            var categoryScale;
            var linearScale;
            var clusterBarPlot;
            beforeEach(function () {
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                categoryScale = new Plottable.Scales.Category();
                linearScale = new Plottable.Scales.Linear();
                clusterBarPlot = new Plottable.Plots.ClusteredBar(orientation);
                clusterBarPlot.x(function (d) { return isVertical ? d.category : d.num; }, isVertical ? categoryScale : linearScale);
                clusterBarPlot.y(function (d) { return isVertical ? d.num : d.category; }, isVertical ? linearScale : categoryScale);
                clusterBarPlot.baselineValue(0);
                clusterBarPlot.renderTo(div);
            });
            it("renders with correct width, height and position", function () {
                linearScale.domain([0, 2]);
                var dataset1 = new Plottable.Dataset([
                    { category: "A", num: 1 },
                    { category: "B", num: 2 },
                ]);
                var dataset2 = new Plottable.Dataset([
                    { category: "A", num: 2 },
                    { category: "B", num: 1 },
                ]);
                var originalData1 = dataset1.data().slice();
                var originalData2 = dataset2.data().slice();
                clusterBarPlot.addDataset(dataset1);
                clusterBarPlot.addDataset(dataset2);
                var barAreas = clusterBarPlot.content().selectAll(".bar-area");
                chai_1.assert.strictEqual(barAreas.selectAll("rect").size(), 4, "Number of bars should be equivalent to number of datum");
                var maxValue = Math.max.apply(null, originalData1.map(function (d) { return d.num; }).concat(originalData2.map(function (d) { return d.num; })));
                var innerScale = clusterBarPlot._makeInnerScale();
                var rangeBand = innerScale.rangeBand();
                var off = innerScale.scale("0");
                var width = categoryScale.rangeBand() / 2;
                barAreas.each(function (_, outerIndex) {
                    d3.select(this).selectAll("rect").each(function (datum, index) {
                        var bar = d3.select(this);
                        var mainAttr = isVertical ? "height" : "width";
                        var secondaryAttr = isVertical ? "width" : "height";
                        var position = isVertical ? "x" : "y";
                        chai_1.assert.closeTo(TestMethods.numAttr(bar, secondaryAttr), rangeBand, 2, secondaryAttr + " is correct for bar " + index);
                        chai_1.assert.closeTo(TestMethods.numAttr(bar, mainAttr), (isVertical ? DIV_HEIGHT : DIV_WIDTH) / maxValue * datum.num, window.Pixel_CloseTo_Requirement, mainAttr + " is correct for bar " + index);
                        // check that clustering is correct
                        var offset = outerIndex === 0 ? off - width : width - off;
                        chai_1.assert.closeTo(TestMethods.numAttr(bar, position) + TestMethods.numAttr(bar, secondaryAttr) / 2, categoryScale.scale(datum.category) + offset, window.Pixel_CloseTo_Requirement, position + " pos correct for bar " + index);
                    });
                });
                chai_1.assert.deepEqual(dataset1.data(), originalData1, "underlying data is not modified for dataset1");
                chai_1.assert.deepEqual(dataset2.data(), originalData2, "underlying data is not modified for dataset2");
                div.remove();
            });
            it("renders correctly with missing values", function () {
                clusterBarPlot.addDataset(new Plottable.Dataset([
                    { category: "A", num: 1 },
                    { category: "B", num: 2 },
                    { category: "C", num: 1 },
                ]));
                clusterBarPlot.addDataset(new Plottable.Dataset([
                    { category: "A", num: 2 },
                    { category: "B", num: 4 },
                ]));
                clusterBarPlot.addDataset(new Plottable.Dataset([
                    { category: "B", num: 15 },
                    { category: "C", num: 15 },
                ]));
                var barAreas = clusterBarPlot.content().selectAll(".bar-area");
                chai_1.assert.strictEqual(barAreas.selectAll("rect").size(), 7, "Number of bars should be equivalent to number of datum");
                var bars = barAreas.nodes().map(function (node) {
                    return d3.select(node).selectAll("rect").nodes();
                });
                var aBar0 = d3.select(bars[0][0]);
                var aBar1 = d3.select(bars[1][0]);
                var bBar0 = d3.select(bars[0][1]);
                var bBar1 = d3.select(bars[1][1]);
                var bBar2 = d3.select(bars[2][0]);
                var cBar0 = d3.select(bars[0][2]);
                var cBar1 = d3.select(bars[2][1]);
                var attr = isVertical ? "x" : "y";
                // check bars are in domain order
                chai_1.assert.operator(TestMethods.numAttr(aBar0, attr), "<", TestMethods.numAttr(bBar0, attr), "first dataset bars ordered correctly");
                chai_1.assert.operator(TestMethods.numAttr(bBar0, attr), "<", TestMethods.numAttr(cBar0, attr), "first dataset bars ordered correctly");
                chai_1.assert.operator(TestMethods.numAttr(aBar1, attr), "<", TestMethods.numAttr(bBar1, attr), "second dataset bars ordered correctly");
                chai_1.assert.operator(TestMethods.numAttr(bBar2, attr), "<", TestMethods.numAttr(cBar1, attr), "third dataset bars ordered correctly");
                // check that clustering is correct
                chai_1.assert.operator(TestMethods.numAttr(aBar0, attr), "<", TestMethods.numAttr(aBar1, attr), "A bars clustered in dataset order");
                chai_1.assert.operator(TestMethods.numAttr(bBar0, attr), "<", TestMethods.numAttr(bBar1, attr), "B bars clustered in dataset order");
                chai_1.assert.operator(TestMethods.numAttr(bBar1, attr), "<", TestMethods.numAttr(bBar2, attr), "B bars clustered in dataset order");
                chai_1.assert.operator(TestMethods.numAttr(cBar0, attr), "<", TestMethods.numAttr(cBar1, attr), "C bars clustered in dataset order");
                div.remove();
            });
        });
    });
});
