"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Plots", function () {
    describe("LinePlot", function () {
        describe("Basic Usage", function () {
            var div;
            var xScale;
            var yScale;
            var linePlot;
            var data = [{ x: 0, y: 0 }, { x: 1, y: 1 }];
            var dataset;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                linePlot = new Plottable.Plots.Line();
                linePlot.x(function (d) { return d.x; }, xScale);
                linePlot.y(function (d) { return d.y; }, yScale);
                dataset = new Plottable.Dataset(data);
            });
            it("does not throw error when given NaN values", function () {
                var dataWithNaN = [
                    { x: 0.0, y: 0.0 },
                    { x: 0.2, y: 0.2 },
                    { x: 0.4, y: NaN },
                    { x: 0.6, y: 0.6 },
                    { x: 0.8, y: 0.8 },
                ];
                linePlot.addDataset(new Plottable.Dataset(dataWithNaN));
                chai_1.assert.doesNotThrow(function () { return linePlot.renderTo(div); }, Error, "does not throw error with NaN data");
                var entities = linePlot.entities();
                var expectedLength = dataWithNaN.length - 1;
                chai_1.assert.lengthOf(entities, expectedLength, "NaN data was not returned");
                div.remove();
            });
            it("does not throw error when rendering without data", function () {
                chai_1.assert.doesNotThrow(function () { return linePlot.renderTo(div); }, Error, "does not throw error rendering without data");
                div.remove();
            });
            it("retains original classes when setting class with attr", function () {
                var cssClass = "pink";
                linePlot.attr("class", cssClass);
                linePlot.renderTo(div);
                linePlot.addDataset(dataset);
                var linePath = linePlot.content().select(".line");
                chai_1.assert.isTrue(linePath.classed(cssClass), "custom class is applied");
                chai_1.assert.isTrue(linePath.classed("line"), "default class is retained");
                div.remove();
            });
            it("draws a line with correct data points and fill and stroke settings", function () {
                linePlot.addDataset(dataset);
                linePlot.attr("stroke", function (d, i, m) { return d3.rgb(d.x, d.y, i).toString(); });
                linePlot.renderTo(div);
                var linePath = linePlot.content().select(".line");
                TestMethods.assertPathEqualToDataPoints(linePath.attr("d"), data, xScale, yScale);
                chai_1.assert.strictEqual(linePath.style("fill"), "none", "line fill renders as \"none\"");
                chai_1.assert.strictEqual(linePath.attr("stroke"), "rgb(0, 0, 0)", "stroke set correctly");
                div.remove();
            });
            it("can set attributes and render accordingly", function () {
                linePlot.addDataset(dataset);
                linePlot.renderTo(div);
                var newColor = "pink";
                linePlot.attr("stroke", newColor);
                linePlot.render();
                var linePath = linePlot.content().select(".line");
                chai_1.assert.strictEqual(linePath.attr("stroke"), newColor, "stroke changed correctly");
                div.remove();
            });
            it("skips NaN and undefined x and y values", function () {
                var lineData = [
                    { x: 0.0, y: 0.0 },
                    { x: 0.2, y: 0.2 },
                    { x: 0.4, y: 0.4 },
                    { x: 0.6, y: 0.6 },
                    { x: 0.8, y: 0.8 },
                ];
                var mutableDataset = new Plottable.Dataset(lineData);
                linePlot.addDataset(mutableDataset);
                linePlot.renderTo(div);
                var linePath = linePlot.content().select(".line");
                var validPoints = lineData.slice(0, 2).concat(lineData.slice(3, 5));
                var dataWithNaN = lineData.slice();
                dataWithNaN[2] = { x: 0.4, y: NaN };
                mutableDataset.data(dataWithNaN);
                linePlot.render();
                TestMethods.assertPathEqualToDataPoints(linePath.attr("d"), validPoints, xScale, yScale);
                dataWithNaN[2] = { x: NaN, y: 0.4 };
                mutableDataset.data(dataWithNaN);
                TestMethods.assertPathEqualToDataPoints(linePath.attr("d"), validPoints, xScale, yScale);
                var dataWithUndefined = lineData.slice();
                dataWithUndefined[2] = { x: 0.4, y: undefined };
                mutableDataset.data(dataWithUndefined);
                TestMethods.assertPathEqualToDataPoints(linePath.attr("d"), validPoints, xScale, yScale);
                dataWithUndefined[2] = { x: undefined, y: 0.4 };
                mutableDataset.data(dataWithUndefined);
                TestMethods.assertPathEqualToDataPoints(linePath.attr("d"), validPoints, xScale, yScale);
                div.remove();
            });
        });
        describe("interpolation", function () {
            var div;
            var xScale;
            var yScale;
            var linePlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                linePlot = new Plottable.Plots.Line();
                linePlot.x(function (d) { return d.x; }, xScale);
                linePlot.y(function (d) { return d.y; }, yScale);
            });
            it("sets interploation correctly", function () {
                chai_1.assert.strictEqual(linePlot.curve(), "linear", "the default interpolation mode is linear");
                chai_1.assert.strictEqual(linePlot.curve("step"), linePlot, "setting an interpolation mode returns the plot");
                chai_1.assert.strictEqual(linePlot.curve(), "step", "setting an interpolation mode works");
                div.remove();
            });
            it("draws step function when interpolator is set to step", function () {
                var data = [
                    { x: 0.0, y: 0 },
                    { x: 0.8, y: 0.717 },
                    { x: 1.6, y: 0.999 },
                    { x: 2.4, y: 0.675 },
                    { x: 3.2, y: -0.058 },
                    { x: 4.0, y: -0.756 },
                    { x: 4.8, y: -0.996 },
                    { x: 5.6, y: -0.631 },
                ];
                linePlot.addDataset(new Plottable.Dataset(data));
                linePlot.renderTo(div);
                var svgPath = linePlot.content().select("path").attr("d");
                TestMethods.assertPathEqualToDataPoints(svgPath, data, xScale, yScale);
                linePlot.curve("step");
                svgPath = linePlot.content().select("path").attr("d");
                var stepPoints = [data[0]];
                for (var i = 1; i < data.length; i++) {
                    var p1 = data[i - 1];
                    var p2 = data[i];
                    stepPoints.push({ x: (p1.x + p2.x) / 2, y: p1.y });
                    stepPoints.push({ x: (p1.x + p2.x) / 2, y: p2.y });
                }
                stepPoints.push({ x: data[data.length - 1].x, y: data[data.length - 1].y });
                TestMethods.assertPathEqualToDataPoints(svgPath, stepPoints, xScale, yScale);
                div.remove();
            });
        });
        describe("selections", function () {
            var div;
            var xScale;
            var yScale;
            var data = [{ x: 0, y: 0.75 }, { x: 1, y: 0.25 }];
            var data2 = [{ x: 0, y: 1 }, { x: 1, y: 0.95 }];
            var dataset;
            var dataset2;
            var linePlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                dataset = new Plottable.Dataset(data);
                dataset2 = new Plottable.Dataset(data2);
                linePlot = new Plottable.Plots.Line();
                linePlot.x(function (d) { return d.x; }, xScale);
                linePlot.y(function (d) { return d.y; }, yScale);
                linePlot.addDataset(dataset);
                linePlot.renderTo(div);
            });
            afterEach(function () {
                div.remove();
            });
            it("can retrieve entities in a certain range", function () {
                linePlot.addDataset(dataset2);
                var entities = linePlot.entitiesIn({
                    min: xScale.scale(0),
                    max: xScale.scale(0),
                }, {
                    min: yScale.scale(1),
                    max: yScale.scale(0),
                });
                chai_1.assert.lengthOf(entities, 2, "only two Entities have been retrieved");
                chai_1.assert.deepEqual(entities[0].datum, { x: 0, y: 0.75 }, "correct datum has been retrieved");
                chai_1.assert.deepEqual(entities[1].datum, { x: 0, y: 1 }, "correct datum has been retrieved");
            });
            it("can retrieve Entities in a certain bounds", function () {
                linePlot.addDataset(dataset2);
                var entities = linePlot.entitiesIn({
                    topLeft: {
                        x: xScale.scale(0),
                        y: yScale.scale(1),
                    },
                    bottomRight: {
                        x: xScale.scale(0),
                        y: yScale.scale(0),
                    },
                });
                chai_1.assert.lengthOf(entities, 2, "only two Entities have been retrieved");
                chai_1.assert.deepEqual(entities[0].datum, { x: 0, y: 0.75 }, "correct datum has been retrieved");
                chai_1.assert.deepEqual(entities[1].datum, { x: 0, y: 1 }, "correct datum has been retrieved");
            });
            it("doesn't return entities outside of the bounds", function () {
                var entities = linePlot.entitiesIn({
                    topLeft: {
                        x: xScale.scale(0.01),
                        y: yScale.scale(1),
                    },
                    bottomRight: {
                        x: xScale.scale(0.01),
                        y: yScale.scale(0),
                    },
                });
                chai_1.assert.lengthOf(entities, 0, "no entities have been retrieved");
            });
            it("retrieves all dataset selections with no args", function () {
                linePlot.addDataset(dataset2);
                var allLines = linePlot.selections();
                chai_1.assert.strictEqual(allLines.size(), 2, "all lines retrieved");
                div.remove();
            });
            it("retrieves selections for selected dataset", function () {
                linePlot.addDataset(dataset2);
                var allLines = linePlot.selections([dataset]);
                chai_1.assert.strictEqual(allLines.size(), 1, "all lines retrieved");
                var selectionData = allLines.data()[0];
                chai_1.assert.deepEqual(selectionData, dataset.data(), "third dataset data in selection data");
            });
            it("skips invalid Dataset", function () {
                var dummyDataset = new Plottable.Dataset([]);
                var allLines = linePlot.selections([dataset, dummyDataset]);
                chai_1.assert.strictEqual(allLines.size(), 1, "all lines retrieved");
                var selectionData = allLines.data()[0];
                chai_1.assert.deepEqual(selectionData, dataset.data(), "third dataset data in selection data");
            });
            it("can retrieve the nearest Entity", function () {
                var px = xScale.scale(data[0].x);
                var py = yScale.scale(data[0].y);
                var closest = linePlot.entityNearest({ x: px, y: py + 1 });
                chai_1.assert.strictEqual(closest.datum, data[0], "it retrieves the closest point from above");
                closest = linePlot.entityNearest({ x: px, y: py - 1 });
                chai_1.assert.strictEqual(closest.datum, data[0], "it retrieves the closest point from below");
                px = xScale.scale(data[1].x);
                py = yScale.scale(data[1].y);
                closest = linePlot.entityNearest({ x: px + 1, y: py });
                chai_1.assert.strictEqual(closest.datum, data[1], "it retrieves the closest point from the right");
                closest = linePlot.entityNearest({ x: px - 1, y: py });
                chai_1.assert.strictEqual(closest.datum, data[1], "it retrieves the closest point from the left");
            });
            it("considers only in-view points for the nearest Entity", function () {
                xScale.domain([0.25, 1]);
                var closest = linePlot.entityNearest({ x: xScale.scale(data[0].x), y: yScale.scale(data[0].y) });
                chai_1.assert.strictEqual(closest.datum, data[1], "it retrieves the closest point in-view");
            });
            it("returns undefined if no Entities are visible", function () {
                dataset.data([]);
                var closest = linePlot.entityNearest({ x: 0, y: 0 });
                chai_1.assert.isUndefined(closest, "returns undefined if no Entity can be found");
            });
        });
        describe("smooth autoranging", function () {
            var div;
            var xScale;
            var yScale;
            var data = [
                { x: 0.0, y: -1 },
                { x: 1.8, y: -2 },
            ];
            var dataset;
            var line;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                xScale.domain([0.1, 1.1]);
                line = new Plottable.Plots.Line();
                line.x(function (d) { return d.x; }, xScale);
                line.y(function (d) { return d.y; }, yScale);
                dataset = new Plottable.Dataset(data);
            });
            it("handles autoranging smoothly ", function () {
                line.addDataset(dataset);
                line.autorangeMode("y");
                xScale.padProportion(0);
                yScale.padProportion(0);
                line.renderTo(div);
                chai_1.assert.deepEqual(yScale.domain(), [0, 1], "when there are no visible points in the view, the y-scale domain defaults to [0, 1]");
                line.autorangeSmooth(true);
                var base = data[0].y;
                var x1 = xScale.domain()[1] - data[0].x;
                var x2 = data[1].x - data[0].x;
                var y2 = data[1].y - data[0].y;
                var expectedBottom = base + y2 * x1 / x2;
                x1 = xScale.domain()[0] - data[0].x;
                var expectedTop = base + y2 * x1 / x2;
                chai_1.assert.closeTo(yScale.domain()[0], expectedBottom, 0.001, "smooth autoranging forces the domain to include the line (left)");
                chai_1.assert.closeTo(yScale.domain()[1], expectedTop, 0.001, "smooth autoranging forces the domain to include the line (right)");
                line.autorangeSmooth(false);
                chai_1.assert.deepEqual(yScale.domain(), [0, 1], "resetting the smooth autorange works");
                xScale.domain([data[0].x, data[1].x]);
                chai_1.assert.deepEqual(yScale.domain(), [-2, -1], "no changes for autoranging smooth with same edge poitns (no smooth)");
                line.autorangeSmooth(true);
                chai_1.assert.deepEqual(yScale.domain(), [-2, -1], "no changes for autoranging smooth with same edge points (smooth)");
                div.remove();
            });
            it("handles autoranging smoothly when autorangeSmooth set before accessors", function () {
                line = new Plottable.Plots.Line();
                line.autorangeSmooth(true);
                line.x(function (d) { return d.x; }, xScale);
                line.y(function (d) { return d.y; }, yScale);
                line.addDataset(dataset);
                line.autorangeMode("y");
                line.renderTo(div);
                var base = data[0].y;
                var x1 = (xScale.domain()[1] - data[0].x) - (xScale.domain()[1] - xScale.domain()[0]) * (1 + yScale.padProportion() / 2);
                var x2 = data[1].x - data[0].x;
                var y2 = data[1].y - data[0].y;
                var expectedTop = base + y2 * x1 / x2;
                x1 = (xScale.domain()[0] - data[0].x) + (xScale.domain()[1] - xScale.domain()[0]) * (1 + yScale.padProportion() / 2);
                var expectedBottom = base + y2 * x1 / x2;
                chai_1.assert.closeTo(yScale.domain()[0], expectedBottom, 0.001, "smooth autoranging forces the domain to include the line (left)");
                chai_1.assert.closeTo(yScale.domain()[1], expectedTop, 0.001, "smooth autoranging forces the domain to include the line (right)");
                div.remove();
            });
            it("handles autoranging smoothly when autorangeSmooth set before autorangeMode", function () {
                line.addDataset(dataset);
                line.autorangeSmooth(true);
                line.autorangeMode("y");
                line.renderTo(div);
                var base = data[0].y;
                var x1 = (xScale.domain()[1] - data[0].x) - (xScale.domain()[1] - xScale.domain()[0]) * (1 + yScale.padProportion() / 2);
                var x2 = data[1].x - data[0].x;
                var y2 = data[1].y - data[0].y;
                var expectedTop = base + y2 * x1 / x2;
                x1 = (xScale.domain()[0] - data[0].x) + (xScale.domain()[1] - xScale.domain()[0]) * (1 + yScale.padProportion() / 2);
                var expectedBottom = base + y2 * x1 / x2;
                chai_1.assert.closeTo(yScale.domain()[0], expectedBottom, 0.001, "smooth autoranging forces the domain to include the line (left)");
                chai_1.assert.closeTo(yScale.domain()[1], expectedTop, 0.001, "smooth autoranging forces the domain to include the line (right)");
                div.remove();
            });
            it("handles autoranging smoothly when autorangeSmooth set before rendering", function () {
                line.addDataset(dataset);
                line.autorangeMode("y");
                line.autorangeSmooth(true);
                line.renderTo(div);
                var base = data[0].y;
                var x1 = (xScale.domain()[1] - data[0].x) - (xScale.domain()[1] - xScale.domain()[0]) * (1 + yScale.padProportion() / 2);
                var x2 = data[1].x - data[0].x;
                var y2 = data[1].y - data[0].y;
                var expectedTop = base + y2 * x1 / x2;
                x1 = (xScale.domain()[0] - data[0].x) + (xScale.domain()[1] - xScale.domain()[0]) * (1 + yScale.padProportion() / 2);
                var expectedBottom = base + y2 * x1 / x2;
                chai_1.assert.closeTo(yScale.domain()[0], expectedBottom, 0.001, "smooth autoranging forces the domain to include the line (left)");
                chai_1.assert.closeTo(yScale.domain()[1], expectedTop, 0.001, "smooth autoranging forces the domain to include the line (right)");
                div.remove();
            });
            it("handles autoranging smoothly when autorangeSmooth set after rendering", function () {
                line.addDataset(dataset);
                line.autorangeMode("y");
                line.renderTo(div);
                line.autorangeSmooth(true);
                var base = data[0].y;
                var x1 = (xScale.domain()[1] - data[0].x) - (xScale.domain()[1] - xScale.domain()[0]) * (1 + yScale.padProportion() / 2);
                var x2 = data[1].x - data[0].x;
                var y2 = data[1].y - data[0].y;
                var expectedTop = base + y2 * x1 / x2;
                x1 = (xScale.domain()[0] - data[0].x) + (xScale.domain()[1] - xScale.domain()[0]) * (1 + yScale.padProportion() / 2);
                var expectedBottom = base + y2 * x1 / x2;
                chai_1.assert.closeTo(yScale.domain()[0], expectedBottom, 0.001, "smooth autoranging forces the domain to include the line (left)");
                chai_1.assert.closeTo(yScale.domain()[1], expectedTop, 0.001, "smooth autoranging forces the domain to include the line (right)");
                div.remove();
            });
            it("handles autoranging smoothly when autorangeSmooth set after rendering, before autorangeMode", function () {
                line.addDataset(dataset);
                line.renderTo(div);
                line.autorangeSmooth(true);
                line.autorangeMode("y");
                var base = data[0].y;
                var x1 = (xScale.domain()[1] - data[0].x) - (xScale.domain()[1] - xScale.domain()[0]) * (1 + yScale.padProportion() / 2);
                var x2 = data[1].x - data[0].x;
                var y2 = data[1].y - data[0].y;
                var expectedTop = base + y2 * x1 / x2;
                x1 = (xScale.domain()[0] - data[0].x) + (xScale.domain()[1] - xScale.domain()[0]) * (1 + yScale.padProportion() / 2);
                var expectedBottom = base + y2 * x1 / x2;
                chai_1.assert.closeTo(yScale.domain()[0], expectedBottom, 0.001, "smooth autoranging forces the domain to include the line (left)");
                chai_1.assert.closeTo(yScale.domain()[1], expectedTop, 0.001, "smooth autoranging forces the domain to include the line (right)");
                div.remove();
            });
            it("handles autoDomain correcly with and without autorangeSmooth", function () {
                var expectedDomain = [-0.2, 2];
                xScale.domain([-0.2, 2]);
                line.addDataset(dataset);
                line.autorangeMode("y");
                line.autorangeSmooth(true);
                xScale.autoDomain();
                line.renderTo(div);
                chai_1.assert.deepEqual(xScale.domain(), expectedDomain, "autoDomain works even when autoranging is done smoothly");
                line.autorangeSmooth(false);
                chai_1.assert.deepEqual(xScale.domain(), expectedDomain, "autoDomain works when smooth autoranging is disabled back");
                div.remove();
            });
            it("handles autoDomain correcly with smooth autoranging set after rendering", function () {
                var expectedDomain = [-0.2, 2];
                xScale.domain([-0.2, 2]);
                line.addDataset(dataset);
                line.renderTo(div);
                line.autorangeSmooth(true);
                xScale.autoDomain();
                chai_1.assert.deepEqual(xScale.domain(), expectedDomain, "autoDomain works even when autoranging is done smoothly");
                line.autorangeSmooth(false);
                chai_1.assert.deepEqual(xScale.domain(), expectedDomain, "autoDomain works when smooth autoranging is disabled back");
                div.remove();
            });
            it("handles autorange smoothly for vertical lines", function () {
                yScale.domain([0.1, 1.1]);
                var verticalLinesData = [
                    { x: -2, y: 1.8 },
                    { x: -1, y: 0.0 },
                ];
                line.addDataset(new Plottable.Dataset(verticalLinesData));
                line.autorangeMode("x");
                xScale.padProportion(0);
                yScale.padProportion(0);
                line.renderTo(div);
                chai_1.assert.deepEqual(xScale.domain(), [0, 1], "when there are no visible points in the view, the x-scale domain defaults to [0, 1]");
                line.autorangeSmooth(true);
                var base = verticalLinesData[0].x;
                var x1 = (yScale.domain()[1] - verticalLinesData[0].y);
                var x2 = verticalLinesData[1].y - verticalLinesData[0].y;
                var y2 = verticalLinesData[1].x - verticalLinesData[0].x;
                var expectedTop = base + y2 * x1 / x2;
                x1 = (yScale.domain()[0] - verticalLinesData[0].y);
                var expectedBottom = base + y2 * x1 / x2;
                chai_1.assert.closeTo(xScale.domain()[0], expectedTop, 0.001, "smooth autoranging forces the domain to include the line (left)");
                chai_1.assert.closeTo(xScale.domain()[1], expectedBottom, 0.001, "smooth autoranging forces the domain to include the line (right)");
                line.autorangeSmooth(false);
                chai_1.assert.deepEqual(xScale.domain(), [0, 1], "resetting the smooth autorange works");
                yScale.domain([verticalLinesData[0].y, verticalLinesData[1].y]);
                chai_1.assert.deepEqual(xScale.domain(), [-2, -1], "no changes for autoranging smooth with same edge poitns (no smooth)");
                line.autorangeSmooth(true);
                chai_1.assert.deepEqual(xScale.domain(), [-2, -1], "no changes for autoranging smooth with same edge points (smooth)");
                div.remove();
            });
        });
        describe("Cropped Rendering Performance", function () {
            var div;
            var plot;
            var xScale;
            var yScale;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                plot = new Plottable.Plots.Line();
                plot.x(function (d) { return d.x; }, xScale).y(function (d) { return d.y; }, yScale);
            });
            it("can set the croppedRendering option", function () {
                plot.renderTo(div);
                chai_1.assert.isTrue(plot.croppedRenderingEnabled(), "croppedRendering is enabled by default");
                chai_1.assert.strictEqual(plot.croppedRenderingEnabled(false), plot, "setting the croppedRendering option returns the plot");
                chai_1.assert.isFalse(plot.croppedRenderingEnabled(), "can disable the croppedRendering option");
                chai_1.assert.strictEqual(plot.croppedRenderingEnabled(true), plot, "setting the croppedRendering option returns the plot");
                chai_1.assert.isTrue(plot.croppedRenderingEnabled(), "can enable the croppedRendering option");
                div.remove();
            });
            it("does not render lines that are outside the viewport", function () {
                var data = [
                    { x: 1, y: 1 },
                    { x: 2, y: 2 },
                    { x: 3, y: 1 },
                    { x: 4, y: 2 },
                    { x: 5, y: 1 },
                ];
                plot.addDataset(new Plottable.Dataset(data));
                // Only middle point is in viewport
                xScale.domain([2.5, 3.5]);
                plot.croppedRenderingEnabled(true);
                plot.renderTo(div);
                var path = plot.content().select("path.line").attr("d");
                var expectedRenderedData = [1, 2, 3].map(function (d) { return data[d]; });
                TestMethods.assertPathEqualToDataPoints(path, expectedRenderedData, xScale, yScale);
                div.remove();
            });
            it("works when the performance option is set after rendering to svg", function () {
                var data = [
                    { x: 1, y: 1 },
                    { x: 2, y: 2 },
                    { x: 3, y: 1 },
                    { x: 4, y: 2 },
                    { x: 5, y: 1 },
                ];
                plot.addDataset(new Plottable.Dataset(data));
                // Only middle point is in viewport
                xScale.domain([2.5, 3.5]);
                plot.renderTo(div);
                plot.croppedRenderingEnabled(true);
                var path = plot.content().select("path.line").attr("d");
                var expectedRenderedData = [1, 2, 3].map(function (d) { return data[d]; });
                TestMethods.assertPathEqualToDataPoints(path, expectedRenderedData, xScale, yScale);
                div.remove();
            });
            it("works for vertical line plots", function () {
                var data = [
                    { x: 1, y: 1 },
                    { x: 2, y: 2 },
                    { x: 1, y: 3 },
                    { x: 2, y: 4 },
                    { x: 1, y: 5 },
                ];
                plot.addDataset(new Plottable.Dataset(data));
                xScale.padProportion(0);
                // Only middle point is in viewport
                yScale.domain([2.5, 3.5]);
                plot.croppedRenderingEnabled(true);
                plot.renderTo(div);
                var path = plot.content().select("path.line").attr("d");
                var expectedRenderedData = [1, 2, 3].map(function (d) { return data[d]; });
                TestMethods.assertPathEqualToDataPoints(path, expectedRenderedData, xScale, yScale);
                div.remove();
            });
            it("adapts to scale changes", function () {
                var data = [
                    { x: 1, y: 1 },
                    { x: 2, y: 2 },
                    { x: 3, y: 1 },
                    { x: 4, y: 2 },
                    { x: 5, y: 1 },
                ];
                plot.addDataset(new Plottable.Dataset(data));
                plot.croppedRenderingEnabled(true);
                plot.renderTo(div);
                var path = plot.content().select("path.line").attr("d");
                TestMethods.assertPathEqualToDataPoints(path, [0, 1, 2, 3, 4].map(function (d) { return data[d]; }), xScale, yScale);
                // Only middle point is in viewport
                xScale.domain([2.5, 3.5]);
                path = plot.content().select("path.line").attr("d");
                TestMethods.assertPathEqualToDataPoints(path, [1, 2, 3].map(function (d) { return data[d]; }), xScale, yScale);
                // Only first point is in viewport
                xScale.domain([-0.5, 1.5]);
                path = plot.content().select("path.line").attr("d");
                TestMethods.assertPathEqualToDataPoints(path, [0, 1].map(function (d) { return data[d]; }), xScale, yScale);
                div.remove();
            });
        });
        describe("Downsampling Performance", function () {
            var div;
            var plot;
            var xScale;
            var yScale;
            beforeEach(function () {
                div = TestMethods.generateDiv(50, 50);
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                plot = new Plottable.Plots.Line();
                plot.x(function (d) { return d.x; }, xScale).y(function (d) { return d.y; }, yScale);
            });
            it("can set the downsampling option", function () {
                plot.renderTo(div);
                chai_1.assert.isFalse(plot.downsamplingEnabled(), "downsampling is not enabled by default");
                chai_1.assert.strictEqual(plot.downsamplingEnabled(true), plot, "enabling the downsampling option returns the plot");
                chai_1.assert.isTrue(plot.downsamplingEnabled(), "can enable the downsampling option");
                plot.downsamplingEnabled(false);
                chai_1.assert.isFalse(plot.downsamplingEnabled(), "can disable the downsampling option");
                div.remove();
            });
            it("does not render points that should be removed in downsampling in horizontal line plots", function () {
                var data = [
                    { x: -100, y: -1 },
                    { x: 0, y: 2 },
                    { x: 0.5, y: 1.5 },
                    { x: 1, y: 1 },
                    { x: 2, y: 4 },
                    { x: 3, y: 3 },
                    { x: 100, y: 2 },
                ];
                plot.addDataset(new Plottable.Dataset(data));
                xScale.domain([-100, 100]);
                plot.downsamplingEnabled(true);
                plot.renderTo(div);
                var lineScaledXValue = Math.floor(xScale.scale(data[1].x));
                chai_1.assert.notStrictEqual(Math.floor(xScale.scale(data[0].x)), lineScaledXValue, "point(" + data[0].x + "," + data[0].y + ") should not have the same scaled x value as the horizontal line");
                data.slice(1, 6).forEach(function (d, i) {
                    chai_1.assert.strictEqual(Math.floor(xScale.scale(d.x)), lineScaledXValue, "point(" + d.x + "," + d.y + " should have the same scaled x value as the horizontal line");
                });
                chai_1.assert.notStrictEqual(Math.floor(xScale.scale(data[6].x)), lineScaledXValue, "point(" + data[6].x + "," + data[6].y + ") should not have the same scaled x value as the horizontal line");
                var path = plot.content().select("path.line").attr("d");
                var expectedRenderedData = [0, 1, 4, 3, 5, 6].map(function (d) { return data[d]; });
                TestMethods.assertPathEqualToDataPoints(path, expectedRenderedData, xScale, yScale);
                div.remove();
            });
            it("does not render points that should be removed in downsampling in vertical line plots", function () {
                var data = [
                    { x: -1, y: -50 },
                    { x: 2, y: 1 },
                    { x: 1.5, y: 1.5 },
                    { x: 1, y: 2 },
                    { x: 4, y: 3 },
                    { x: 3, y: 4 },
                    { x: 2, y: 100 },
                ];
                plot.addDataset(new Plottable.Dataset(data));
                yScale.domain([-200, 200]);
                plot.downsamplingEnabled(true);
                plot.renderTo(div);
                var lineScaledYValue = Math.floor(yScale.scale(data[1].y));
                chai_1.assert.notStrictEqual(Math.floor(yScale.scale(data[0].y)), lineScaledYValue, "point(" + data[0].x + "," + data[0].y + ") should not have the same scaled y value as the vertical line");
                data.slice(1, 6).forEach(function (d, i) {
                    chai_1.assert.strictEqual(Math.floor(yScale.scale(d.y)), lineScaledYValue, "point(" + d.x + "," + d.y + ") should have the same scaled y value as the vertical line");
                });
                chai_1.assert.notStrictEqual(Math.floor(yScale.scale(data[6].y)), lineScaledYValue, "point(" + data[6].x + "," + data[6].y + ") should not have the same scaled y value as the vertical line");
                var path = plot.content().select("path.line").attr("d");
                var expectedRenderedData = [0, 1, 3, 4, 5, 6].map(function (d) { return data[d]; });
                TestMethods.assertPathEqualToDataPoints(path, expectedRenderedData, xScale, yScale);
                div.remove();
            });
            it("does not render points that are on the same line except for the first, the last, the largest and the smallest points", function () {
                var data = [
                    { x: 3, y: 1 },
                    { x: 2, y: 2 },
                    { x: 1, y: 1 },
                    { x: 10, y: 10 },
                    { x: 2.5, y: 2.5 },
                    { x: 3, y: 3 },
                    { x: 3, y: 1 },
                ];
                plot.addDataset(new Plottable.Dataset(data));
                var expectedYValue = function (p1, p2, slope) {
                    return p1.y + (p2.x - p1.x) * slope;
                };
                var lineCurrentSlope = (data[2].y - data[1].y) / (data[2].x - data[1].x);
                chai_1.assert.notStrictEqual(Math.floor(expectedYValue(data[1], data[0], lineCurrentSlope)), Math.floor(data[0].y), "point(" + data[0].x + "," + data[0].y + ") is not on the line with slope " + lineCurrentSlope);
                data.slice(1, 6).forEach(function (d, i) {
                    chai_1.assert.strictEqual(Math.floor(expectedYValue(data[1], d, lineCurrentSlope)), Math.floor(d.y), "point(" + d.x + "," + d.y + ") is on the line with slope " + lineCurrentSlope);
                });
                chai_1.assert.notStrictEqual(Math.floor(expectedYValue(data[1], data[6], lineCurrentSlope)), Math.floor(data[6].y), "point(" + data[6].x + "," + data[6].y + ") is not on the line with slope " + lineCurrentSlope);
                plot.downsamplingEnabled(true);
                plot.renderTo(div);
                var path = plot.content().select("path.line").attr("d");
                var expectedRenderedData = [0, 1, 2, 3, 5, 6].map(function (d) { return data[d]; });
                TestMethods.assertPathEqualToDataPoints(path, expectedRenderedData, xScale, yScale);
                div.remove();
            });
        });
    });
});
