"use strict";
var chai_1 = require("chai");
var sinon = require("sinon");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Plots", function () {
    describe("Plot", function () {
        it("adds a \"plot\" css class by default", function () {
            var plot = new Plottable.Plot();
            chai_1.assert.isTrue(plot.hasClass("plot"), "plot class added by default");
        });
        it("turns _overflowHidden on", function () {
            var plot = new Plottable.Plot();
            chai_1.assert.isTrue(plot._overflowHidden, "overflowHidden is enabled");
        });
        describe("managing entities", function () {
            var plot;
            var div;
            beforeEach(function () {
                plot = new Plottable.Plot();
                div = TestMethods.generateDiv();
                plot.renderTo(div);
            });
            afterEach(function () {
                div.remove();
            });
            it("first call to entities builds a new store", function () {
                var data = [5, -5, 10];
                var dataset = new Plottable.Dataset(data);
                plot.addDataset(dataset);
                var lightweightPlotEntitySpy = sinon.spy(plot, "_buildLightweightPlotEntities");
                var entities = plot.entities();
                var entities1 = plot.entities();
                chai_1.assert.deepEqual(entities, entities1);
                chai_1.assert.isTrue(lightweightPlotEntitySpy.calledOnce);
            });
            it("new store is built each time entities is called with data", function () {
                var data = [5, -5, 10];
                var dataset = new Plottable.Dataset(data);
                plot.addDataset(dataset);
                var lightweightPlotEntitySpy = sinon.spy(plot, "_buildLightweightPlotEntities");
                var entities = plot.entities([dataset]);
                var entities1 = plot.entities([dataset]);
                chai_1.assert.deepEqual(entities, entities1);
                chai_1.assert.isTrue(lightweightPlotEntitySpy.calledTwice);
            });
            it("rebuilds the store when the datasets change", function () {
                var dataset = new Plottable.Dataset([5, -5, 10]);
                var dataset1 = new Plottable.Dataset([1, -2, 3]);
                plot.addDataset(dataset);
                var lightweightPlotEntitySpy = sinon.spy(plot, "_buildLightweightPlotEntities");
                var entities = plot.entities();
                chai_1.assert.strictEqual(entities.length, 3);
                plot.datasets([dataset1]);
                var entities1 = plot.entities();
                chai_1.assert.isTrue(lightweightPlotEntitySpy.calledTwice);
                chai_1.assert.strictEqual(entities1.length, 3);
                plot.addDataset(dataset);
                var entities2 = plot.entities();
                chai_1.assert.isTrue(lightweightPlotEntitySpy.calledThrice);
                chai_1.assert.strictEqual(entities2.length, 6);
                plot.removeDataset(dataset);
                var entities3 = plot.entities();
                chai_1.assert.deepEqual(entities3, entities1);
                chai_1.assert.strictEqual(lightweightPlotEntitySpy.callCount, 4);
            });
        });
        describe("entityNearest", function () {
            var plot;
            var div;
            beforeEach(function () {
                plot = new Plottable.Plot();
                div = TestMethods.generateDiv();
                plot.renderTo(div);
            });
            afterEach(function () {
                div.remove();
            });
            it("builds entityStore if entity store is undefined", function () {
                var dataset = new Plottable.Dataset([5, -5, 10]);
                plot.addDataset(dataset);
                var lightweightPlotEntitySpy = sinon.spy(plot, "_buildLightweightPlotEntities");
                plot.entityNearest({ x: 5, y: 0 });
                chai_1.assert.isTrue(lightweightPlotEntitySpy.calledOnce);
            });
        });
        describe("managing datasets", function () {
            var plot;
            beforeEach(function () {
                plot = new Plottable.Plot();
            });
            it("can add a dataset", function () {
                var dataset = new Plottable.Dataset();
                chai_1.assert.strictEqual(plot.addDataset(dataset), plot, "adding a dataset returns the plot");
                chai_1.assert.deepEqual(plot.datasets(), [dataset], "dataset has been added");
            });
            it("can remove a dataset", function () {
                var dataset = new Plottable.Dataset();
                plot.addDataset(dataset);
                chai_1.assert.strictEqual(plot.removeDataset(dataset), plot, "removing a dataset returns the plot");
                chai_1.assert.deepEqual(plot.datasets(), [], "dataset has been removed");
            });
            it("can set the datasets", function () {
                var datasetCount = 5;
                var datasets = Plottable.Utils.Math.range(0, datasetCount).map(function () { return new Plottable.Dataset(); });
                chai_1.assert.strictEqual(plot.datasets(datasets), plot, "setting the datasets returns the plot");
                chai_1.assert.deepEqual(plot.datasets(), datasets, "datasets have been set");
            });
            it("adds a g element for each dataset to the render area", function () {
                var datasetCount = 5;
                var datasets = Plottable.Utils.Math.range(0, datasetCount).map(function () { return new Plottable.Dataset(); });
                plot.datasets(datasets);
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                chai_1.assert.strictEqual(plot.content().select(".render-area").selectAll("g").size(), datasetCount, "g for each dataset");
                div.remove();
            });
            it("updates the scales extents when the datasets get updated", function () {
                var scale = new Plottable.Scales.Linear();
                var data = [5, -5, 10];
                var dataset = new Plottable.Dataset(data);
                plot.attr("foo", function (d) { return d; }, scale);
                plot.addDataset(dataset);
                var oldDomain = scale.domain();
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                chai_1.assert.operator(scale.domainMin(), "<=", Math.min.apply(null, data), "domainMin extended to at least minimum");
                chai_1.assert.operator(scale.domainMax(), ">=", Math.max.apply(null, data), "domainMax extended to at least maximum");
                plot.removeDataset(dataset);
                chai_1.assert.deepEqual(scale.domain(), oldDomain, "domain does not include dataset if removed");
                var data2 = [7, -7, 8];
                var dataset2 = new Plottable.Dataset(data2);
                plot.datasets([dataset, dataset2]);
                chai_1.assert.operator(scale.domainMin(), "<=", Math.min.apply(null, data.concat(data2)), "domainMin includes new dataset");
                chai_1.assert.operator(scale.domainMax(), ">=", Math.max.apply(null, data.concat(data2)), "domainMax includes new dataset");
                div.remove();
            });
            it("updates the scale extents in dataset order", function () {
                var categoryScale = new Plottable.Scales.Category();
                var data = ["A"];
                var dataset = new Plottable.Dataset(data);
                var data2 = ["B"];
                var dataset2 = new Plottable.Dataset(data2);
                plot.addDataset(dataset2);
                plot.addDataset(dataset);
                plot.attr("key", function (d) { return d; }, categoryScale);
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                chai_1.assert.deepEqual(categoryScale.domain(), data2.concat(data), "extent in the right order");
                div.remove();
            });
        });
        it("can set if the plot will animate", function () {
            var plot = new Plottable.Plot();
            chai_1.assert.strictEqual(plot.animated(), false, "by default the plot is not animated");
            chai_1.assert.strictEqual(plot.animated(true), plot, "toggling animation returns the plot");
            chai_1.assert.strictEqual(plot.animated(), true, "can set if plot does animate");
            plot.animated(false);
            chai_1.assert.strictEqual(plot.animated(), false, "can set if plot does not animate");
        });
        describe("managing animators", function () {
            var plot;
            beforeEach(function () {
                plot = new Plottable.Plot();
            });
            it("uses a null animator for the reset animator by default", function () {
                chai_1.assert.isTrue(plot.animator(Plottable.Plots.Animator.RESET) instanceof Plottable.Animators.Null, "null by default");
            });
            it("uses an easing animator for the main animator by default", function () {
                chai_1.assert.isTrue(plot.animator(Plottable.Plots.Animator.MAIN) instanceof Plottable.Animators.Easing, "easing by default");
            });
            it("can set the animator for a key", function () {
                var animator = new Plottable.Animators.Easing();
                var animatorKey = "foo";
                chai_1.assert.strictEqual(plot.animator(animatorKey, animator), plot, "setting an animator returns the plot");
                chai_1.assert.strictEqual(plot.animator(animatorKey), animator, "can set the animator for a given key");
            });
        });
        it("disconnects the data extents from the scales when destroyed", function () {
            var plot = new Plottable.Plot();
            var scale = new Plottable.Scales.Linear();
            plot.attr("attr", function (d) { return d; }, scale);
            var data = [5, -5, 10];
            var dataset = new Plottable.Dataset(data);
            plot.addDataset(dataset);
            var oldDomain = scale.domain();
            var div = TestMethods.generateDiv();
            plot.anchor(div);
            plot.destroy();
            chai_1.assert.deepEqual(scale.domain(), oldDomain, "destroying the plot removes its extents from the scale");
            div.remove();
        });
        it("disconnects the data extents from the scales when detached", function () {
            var plot = new Plottable.Plot();
            var scale = new Plottable.Scales.Linear();
            plot.attr("attr", function (d) { return d; }, scale);
            var data = [5, -5, 10];
            var dataset = new Plottable.Dataset(data);
            plot.addDataset(dataset);
            var oldDomain = scale.domain();
            var div = TestMethods.generateDiv();
            plot.anchor(div);
            plot.detach();
            chai_1.assert.deepEqual(scale.domain(), oldDomain, "detaching the plot removes its extents from the scale");
            div.remove();
        });
        describe("setting attributes with attr()", function () {
            var plot;
            beforeEach(function () {
                plot = new Plottable.Plot();
            });
            it("can set the attribute to a constant value", function () {
                var constantNumber = 10;
                chai_1.assert.strictEqual(plot.attr("foo", constantNumber), plot, "setting the attribute returns the calling plot");
                chai_1.assert.strictEqual(plot.attr("foo").accessor(null, 0, null), constantNumber, "can set the attribute to a constant number");
                var constantString = "one";
                plot.attr("foo", constantString);
                chai_1.assert.strictEqual(plot.attr("foo").accessor(null, 0, null), constantString, "can set the attribute to a constant string");
            });
            it("can set the attribute based on the input data", function () {
                var data = [1, 2, 3, 4, 5];
                var dataset = new Plottable.Dataset(data);
                plot.addDataset(dataset);
                var numberAccessor = function (d, i) { return d + i * 10; };
                chai_1.assert.strictEqual(plot.attr("foo", numberAccessor), plot, "setting the attribute returns the calling plot");
                var attrAccessor = plot.attr("foo").accessor;
                /* tslint:disable no-shadowed-variable */
                plot.datasets().forEach(function (dataset, datasetIndex) {
                    dataset.data().forEach(function (datum, index) {
                        chai_1.assert.strictEqual(attrAccessor(datum, index, dataset), numberAccessor(datum, index), "can set attribute for number datum " + index + " in dataset " + datasetIndex);
                    });
                });
                /* tslint:enable no-shadowed-variable */
                var stringAccessor = function (d, i) { return d + i * 10 + " foo"; };
                chai_1.assert.strictEqual(plot.attr("foo", stringAccessor), plot, "setting the attribute returns the calling plot");
                attrAccessor = plot.attr("foo").accessor;
                /* tslint:disable no-shadowed-variable */
                plot.datasets().forEach(function (dataset, datasetIndex) {
                    dataset.data().forEach(function (datum, index) {
                        chai_1.assert.strictEqual(attrAccessor(datum, index, dataset), stringAccessor(datum, index), "can set attribute for string datum " + index + " in dataset " + datasetIndex);
                    });
                });
                /* tslint:enable no-shadowed-variable */
            });
            it("passes the correct index to the Accessor", function () {
                var data = ["A", "B", "C"];
                var dataset = new Plottable.Dataset(data);
                plot.addDataset(dataset);
                var indexCheckAccessor = function (datum, index) {
                    chai_1.assert.strictEqual(index, data.indexOf(datum), "was passed correct index");
                    return datum;
                };
                var scale = new Plottable.Scales.Category();
                plot.attr("foo", indexCheckAccessor, scale);
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                div.remove();
            });
            it("can apply a scale to the returned values", function () {
                var data = [1, 2, 3, 4, 5];
                var dataset = new Plottable.Dataset(data);
                plot.addDataset(dataset);
                var numberAccessor = function (d, i) { return d; };
                var linearScale = new Plottable.Scales.Linear();
                chai_1.assert.strictEqual(plot.attr("foo", numberAccessor, linearScale), plot, "setting the attribute returns the calling plot");
                var attrAccessor = plot.attr("foo").accessor;
                var attrScale = plot.attr("foo").scale;
                /* tslint:disable no-shadowed-variable */
                plot.datasets().forEach(function (dataset, datasetIndex) {
                    dataset.data().forEach(function (datum, index) {
                        chai_1.assert.strictEqual(attrScale.scale(attrAccessor(datum, index, dataset)), linearScale.scale(numberAccessor(datum, index)), "can set based on scaled version of number datum " + index + " in dataset " + datasetIndex);
                    });
                });
                /* tslint:enable no-shadowed-variable */
                var stringAccessor = function (d, i) { return d + " foo"; };
                var categoryScale = new Plottable.Scales.Category();
                categoryScale.domain(data.map(stringAccessor));
                chai_1.assert.strictEqual(plot.attr("foo", stringAccessor, categoryScale), plot, "setting the attribute returns the calling plot");
                attrAccessor = plot.attr("foo").accessor;
                attrScale = plot.attr("foo").scale;
                /* tslint:disable no-shadowed-variable */
                plot.datasets().forEach(function (dataset, datasetIndex) {
                    dataset.data().forEach(function (datum, index) {
                        chai_1.assert.strictEqual(attrScale.scale(attrAccessor(datum, index, dataset)), categoryScale.scale(stringAccessor(datum, index)), "can set based on scaled version of string datum " + index + " in dataset " + datasetIndex);
                    });
                });
                /* tslint:enable no-shadowed-variable */
            });
            it("updates the scales extents associated with an attribute when that attribute is set", function () {
                var scale = new Plottable.Scales.Linear();
                var data = [5, -5, 10];
                var dataset = new Plottable.Dataset(data);
                plot.addDataset(dataset);
                plot.attr("foo", function (d) { return d; }, scale);
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                chai_1.assert.operator(scale.domainMin(), "<=", Math.min.apply(null, data), "domainMin extended to at least minimum");
                chai_1.assert.operator(scale.domainMax(), ">=", Math.max.apply(null, data), "domainMax extended to at least maximum");
                div.remove();
            });
        });
        describe("canvas renderer", function () {
            var plot;
            beforeEach(function () {
                plot = new Plottable.Plot();
            });
            it("can get/set the .renderer", function () {
                chai_1.assert.strictEqual(plot.renderer(), "svg", "defaults to svg");
                plot.renderer("canvas");
                chai_1.assert.strictEqual(plot.renderer(), "canvas", "can change to canvas");
            });
            it("adds a canvas element to the DOM when rendered", function () {
                plot.renderer("canvas");
                var div = TestMethods.generateDiv();
                plot.renderTo(div);
                chai_1.assert.isDefined(div.select("canvas").node(), "canvas is in DOM");
                div.remove();
            });
            it("removes renderArea from the DOM after anchoring", function () {
                plot.addDataset(new Plottable.Dataset([]));
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                // set to canvas after anchor
                plot.renderer("canvas");
                chai_1.assert.strictEqual(plot.content().select(".render-area").selectAll("g").size(), 0, "no g's in renderArea anymore");
                plot.renderer("svg");
                chai_1.assert.strictEqual(plot.content().select(".render-area").selectAll("g").size(), 1, "g's come back");
                div.remove();
            });
            it("sets Drawers' canvas when renderer is set", function () {
                plot.addDataset(new Plottable.Dataset([]));
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                plot.renderer("canvas");
                plot._datasetToDrawer.forEach(function (drawer) {
                    chai_1.assert.strictEqual(drawer.canvas().node(), div.select("canvas").node(), "drawer's canvas is set");
                });
                div.remove();
            });
            it("returns null on .selections()", function () {
                plot.addDataset(new Plottable.Dataset([]));
                plot.renderer("canvas");
                var div = TestMethods.generateDiv();
                plot.renderTo(div);
                chai_1.assert.isNull(plot.selections(), "no selections on canvas");
                div.remove();
            });
        });
    });
});
