"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
var domUtils_1 = require("../../src/utils/domUtils");
describe("Plots", function () {
    describe("XY Plot", function () {
        describe("autoranging on the x and y scales", function () {
            var div;
            var xScale;
            var yScale;
            var plot;
            var xAccessor = function (d) { return d.x; };
            var yAccessor = function (d) { return d.y; };
            var yTransform = function (d) { return Math.pow(d, 2); };
            var xTransform = function (d) { return Math.pow(d, 0.5); };
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                plot = new Plottable.XYPlot();
                var simpleDataset = new Plottable.Dataset(generateYTransformData(5));
                plot.addDataset(simpleDataset);
                plot.x(xAccessor, xScale)
                    .y(yAccessor, yScale)
                    .anchor(div);
                xScale.padProportion(0);
                yScale.padProportion(0);
            });
            function generateYTransformData(count) {
                return Plottable.Utils.Math.range(0, count).map(function (datumNumber) {
                    return {
                        x: datumNumber,
                        y: yTransform(datumNumber),
                    };
                });
            }
            ;
            it("can set the autorange mode", function () {
                chai_1.assert.strictEqual(plot.autorangeMode(), "none", "defaults to no autoranging");
                chai_1.assert.strictEqual(plot.autorangeMode("x"), plot, "setting autorange mode returns plot");
                chai_1.assert.strictEqual(plot.autorangeMode(), "x", "autorange mode set to x");
                plot.autorangeMode("y");
                chai_1.assert.strictEqual(plot.autorangeMode(), "y", "autorange mode set to y");
                plot.autorangeMode("none");
                chai_1.assert.strictEqual(plot.autorangeMode(), "none", "autorange mode set to none");
                div.remove();
            });
            it("throws an error on setting an invalid autorange mode", function () {
                chai_1.assert.throws(function () { return plot.autorangeMode("foobar"); }, "Invalid scale", "cannot set an invalid mode");
                div.remove();
            });
            it("automatically adjusts Y domain over visible points", function () {
                var oldYDomain = yScale.domain();
                var newXDomain = [2, 3];
                xScale.domain(newXDomain);
                chai_1.assert.deepEqual(yScale.domain(), oldYDomain, "domain not adjusted to visible points");
                plot.autorangeMode("y");
                chai_1.assert.deepEqual(yScale.domain(), newXDomain.map(yTransform), "domain has been adjusted to visible points");
                div.remove();
            });
            it("automatically adjusts Y domain to the default when no points are visible", function () {
                plot.autorangeMode("y");
                var newXDomain = [-2, -1];
                xScale.domain(newXDomain);
                chai_1.assert.deepEqual(yScale.domain(), [0, 1], "scale uses default domain");
                div.remove();
            });
            it("automatically adjusts Y domain when X scale is replaced", function () {
                plot.autorangeMode("y");
                var newXScaleDomain = [0, 3];
                var newXScale = new Plottable.Scales.Linear().domain(newXScaleDomain);
                plot.x(xAccessor, newXScale);
                chai_1.assert.deepEqual(yScale.domain(), newXScaleDomain.map(yTransform), "domain adjusted to visible points on new X scale domain");
                xScale.domain([-2, 2]);
                chai_1.assert.deepEqual(yScale.domain(), newXScaleDomain.map(yTransform), "y scale domain does not change");
                div.remove();
            });
            it("automatically adjusts X domain over visible points", function () {
                var oldXDomain = xScale.domain();
                yScale.domain([0, 1]);
                chai_1.assert.deepEqual(xScale.domain(), oldXDomain, "domain has not been adjusted to visible points");
                plot.autorangeMode("x");
                chai_1.assert.deepEqual(xScale.domain(), yScale.domain().map(xTransform), "domain has been adjusted to visible points");
                div.remove();
            });
            it("automatically adjusts X domain to the default when no points are visible", function () {
                plot.autorangeMode("x");
                yScale.domain([-2, -1]);
                chai_1.assert.deepEqual(xScale.domain(), [0, 1], "scale uses default domain");
                div.remove();
            });
            it("automatically adjusts X domain when Y scale is replaced", function () {
                plot.autorangeMode("x");
                var newYScaleDomain = [0, 1];
                var newYScale = new Plottable.Scales.Linear().domain(newYScaleDomain);
                plot.y(yAccessor, newYScale);
                chai_1.assert.deepEqual(xScale.domain(), newYScaleDomain.map(xTransform), "domain adjusted to visible points on new Y scale domain");
                yScale.domain([-2, 2]);
                chai_1.assert.deepEqual(xScale.domain(), newYScaleDomain.map(xTransform), "x scale domain does not change");
                div.remove();
            });
            it("can show all of the data regardless of autoranging", function () {
                plot.autorangeMode("y");
                xScale.domain([-0.5, 0.5]);
                plot.showAllData();
                chai_1.assert.deepEqual(yScale.domain(), [0, 16], "y domain adjusted to show all data");
                chai_1.assert.deepEqual(xScale.domain(), [0, 4], "x domain adjusted to show all data");
                div.remove();
            });
            it("stops autoranging after the plot is destroyed", function () {
                plot.autorangeMode("y");
                plot.destroy();
                xScale.domain([0, 2]);
                chai_1.assert.deepEqual(yScale.domain(), [0, 1], "autoranging no longer occurs");
                div.remove();
            });
        });
        describe("deferred rendering", function () {
            var plot;
            beforeEach(function () {
                plot = new Plottable.XYPlot();
            });
            it("can set if rendering is deferred", function () {
                chai_1.assert.strictEqual(plot.deferredRendering(), false, "deferred rendering is false by default");
                chai_1.assert.strictEqual(plot.deferredRendering(true), plot, "setting the deferred rendering option returns the plot");
                chai_1.assert.strictEqual(plot.deferredRendering(), true, "deferred rendering can be turned on");
                plot.deferredRendering(false);
                chai_1.assert.strictEqual(plot.deferredRendering(), false, "deferred rendering can be turned off");
            });
            it("immediately translates the render area when the scale domain is translated", function () {
                plot.deferredRendering(true);
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                var xScale = new Plottable.Scales.Linear();
                plot.x(0, xScale);
                var translateAmount = 1;
                xScale.domain(xScale.domain().map(function (d) { return d + translateAmount; }));
                var renderAreaTranslate = domUtils_1.getTranslateValues(plot.content().select(".render-area"));
                chai_1.assert.deepEqual(renderAreaTranslate[0], -translateAmount, "translates with the same amount as domain shift");
                div.remove();
            });
            it("immediately scales the render area when the scale domain is scaled", function () {
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                var xScale = new Plottable.Scales.Linear();
                plot.x(0, xScale);
                plot.deferredRendering(true);
                plot.computeLayout();
                var magnifyAmount = 2;
                xScale.domain(xScale.domain().map(function (d) { return d * magnifyAmount; }));
                var renderAreaScale = domUtils_1.getScaleValues(plot.content().select(".render-area"));
                chai_1.assert.deepEqual(renderAreaScale[0], 1 / magnifyAmount, "translates with the same amount as domain shift");
                div.remove();
            });
        });
        describe("computing the layout", function () {
            var div;
            beforeEach(function () {
                div = TestMethods.generateDiv();
            });
            it("extends the x scale range to the component width", function () {
                var xScale = new Plottable.Scales.Linear();
                var plot = new Plottable.XYPlot();
                plot.x(0, xScale);
                plot.anchor(div);
                plot.computeLayout();
                chai_1.assert.deepEqual(xScale.range(), [0, plot.width()], "range extends to the width");
                div.remove();
            });
            it("sets the y scale range to start from the bottom for non-Category scales", function () {
                var yScale = new Plottable.Scales.Linear();
                var plot = new Plottable.XYPlot();
                plot.y(0, yScale);
                plot.anchor(div);
                plot.computeLayout();
                chai_1.assert.deepEqual(yScale.range(), [plot.height(), 0], "range extends from the height");
                div.remove();
            });
            it("sets the y scale range to start from the top if it is a Category scale", function () {
                var yScale = new Plottable.Scales.Category();
                var plot = new Plottable.XYPlot();
                plot.y("one", yScale);
                plot.anchor(div);
                plot.computeLayout();
                chai_1.assert.deepEqual(yScale.range(), [0, plot.height()], "range extends to the height");
                div.remove();
            });
        });
        describe("managing the x property", function () {
            var plot;
            beforeEach(function () {
                plot = new Plottable.XYPlot();
                // HACKHACK: Must install y scale https://github.com/palantir/plottable/issues/2934
                plot.y(0, new Plottable.Scales.Linear());
            });
            afterEach(function () {
                plot.destroy();
            });
            it("can set the x property to a constant value", function () {
                var constantX = 10;
                chai_1.assert.strictEqual(plot.x(constantX), plot, "setting the x property returns the plot");
                chai_1.assert.strictEqual(plot.x().accessor(null, 0, null), constantX, "returns an accessor returning the constant value");
            });
            it("can set the x property be based on data", function () {
                var accessor = function (d) { return d * 10; };
                chai_1.assert.strictEqual(plot.x(accessor), plot, "setting the x property returns the plot");
                var testData = [4, 2, 3, 5, 6];
                var dataset = new Plottable.Dataset(testData);
                dataset.data().forEach(function (d, i) {
                    chai_1.assert.strictEqual(plot.x().accessor(d, i, dataset), accessor(d), "returns the input accessor");
                });
            });
            it("can set the x property be based on scaled data", function () {
                var accessor = function (d) { return d * 10; };
                var scale = new Plottable.Scales.Linear();
                chai_1.assert.strictEqual(plot.x(accessor, scale), plot, "setting the x property returns the plot");
                var testData = [4, 2, 3, 5, 6];
                var dataset = new Plottable.Dataset(testData);
                dataset.data().forEach(function (d, i) {
                    var bindingAccessor = plot.x().accessor;
                    var bindingScale = plot.x().scale;
                    chai_1.assert.strictEqual(bindingScale.scale(bindingAccessor(d, i, dataset)), scale.scale(accessor(d)), "returns the input accessor and scale");
                });
            });
            it("sets the range on the input scale if the plot has a width", function () {
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                plot.computeLayout();
                var scale = new Plottable.Scales.Linear();
                chai_1.assert.strictEqual(plot.x(0, scale), plot, "setting the x property returns the plot");
                chai_1.assert.deepEqual(scale.range(), [0, plot.width()], "range goes to width on scale");
                div.remove();
            });
            it("can install its extent onto the input scale", function () {
                var div = TestMethods.generateDiv();
                var data = [0, 1, 2, 3, 4];
                plot.addDataset(new Plottable.Dataset(data));
                plot.anchor(div);
                var scale = new Plottable.Scales.Linear();
                scale.padProportion(0);
                chai_1.assert.strictEqual(plot.x(function (d) { return d; }, scale), plot, "setting the x property returns the plot");
                var mathUtils = Plottable.Utils.Math;
                chai_1.assert.deepEqual(scale.domain(), [mathUtils.min(data, 0), mathUtils.max(data, 0)], "range goes to width on scale");
                div.remove();
            });
        });
        describe("managing the y property", function () {
            var plot;
            beforeEach(function () {
                plot = new Plottable.XYPlot();
                // HACKHACK: Must install x scale https://github.com/palantir/plottable/issues/2934
                plot.x(0, new Plottable.Scales.Linear());
            });
            afterEach(function () {
                plot.destroy();
            });
            it("can set the y property to a constant value", function () {
                var constantY = 10;
                chai_1.assert.strictEqual(plot.y(constantY), plot, "setting the y property returns the plot");
                chai_1.assert.strictEqual(plot.y().accessor(null, 0, null), constantY, "returns an accessor returning the constant value");
            });
            it("can set the y property be based on data", function () {
                var accessor = function (d) { return d * 10; };
                chai_1.assert.strictEqual(plot.y(accessor), plot, "setting the y property returns the plot");
                var testData = [4, 2, 3, 5, 6];
                var dataset = new Plottable.Dataset(testData);
                dataset.data().forEach(function (d, i) {
                    chai_1.assert.strictEqual(plot.y().accessor(d, i, dataset), accessor(d), "returns the input accessor");
                });
            });
            it("can set the y property be based on scaled data", function () {
                var accessor = function (d) { return d * 10; };
                var scale = new Plottable.Scales.Linear();
                chai_1.assert.strictEqual(plot.y(accessor, scale), plot, "setting the y property returns the plot");
                var testData = [4, 2, 3, 5, 6];
                var dataset = new Plottable.Dataset(testData);
                dataset.data().forEach(function (d, i) {
                    var bindingAccessor = plot.y().accessor;
                    var bindingScale = plot.y().scale;
                    chai_1.assert.strictEqual(bindingScale.scale(bindingAccessor(d, i, dataset)), scale.scale(accessor(d)), "returns the input accessor and scale");
                });
            });
            it("sets the range on the input scale if the plot has a height", function () {
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                plot.computeLayout();
                var scale = new Plottable.Scales.Linear();
                chai_1.assert.strictEqual(plot.y(0, scale), plot, "setting the y property returns the plot");
                chai_1.assert.deepEqual(scale.range(), [plot.height(), 0], "range goes from height on scale");
                div.remove();
            });
            it("sets the range to be reversed on the input Category scale if the plot has a height", function () {
                var div = TestMethods.generateDiv();
                plot.anchor(div);
                plot.computeLayout();
                var scale = new Plottable.Scales.Category();
                chai_1.assert.strictEqual(plot.y(0, scale), plot, "setting the y property returns the plot");
                chai_1.assert.deepEqual(scale.range(), [0, plot.height()], "range goes to height on scale");
                div.remove();
            });
            it("can install its extent onto the input scale", function () {
                var div = TestMethods.generateDiv();
                var data = [0, 1, 2, 3, 4];
                plot.addDataset(new Plottable.Dataset(data));
                plot.anchor(div);
                var scale = new Plottable.Scales.Linear();
                scale.padProportion(0);
                chai_1.assert.strictEqual(plot.y(function (d) { return d; }, scale), plot, "setting the y property returns the plot");
                var mathUtils = Plottable.Utils.Math;
                chai_1.assert.deepEqual(scale.domain(), [mathUtils.min(data, 0), mathUtils.max(data, 0)], "range goes to width on scale");
                div.remove();
            });
        });
    });
});
