"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
describe("Scales", function () {
    describe("Modified Log Scale", function () {
        describe("Basic Usage", function () {
            var scale;
            var base = 10;
            var delta = 0.00001;
            var epsilon = 0.00001;
            beforeEach(function () {
                scale = new Plottable.Scales.ModifiedLog(base);
            });
            it("is an increasing, continuous function that can go negative", function () {
                d3.range(-base * 2, base * 2, base / 20).forEach(function (x) {
                    // increasing
                    chai_1.assert.operator(scale.scale(x - delta), "<", scale.scale(x));
                    chai_1.assert.operator(scale.scale(x), "<", scale.scale(x + delta));
                    // continuous
                    chai_1.assert.closeTo(scale.scale(x - delta), scale.scale(x), epsilon);
                    chai_1.assert.closeTo(scale.scale(x), scale.scale(x + delta), epsilon);
                });
                chai_1.assert.closeTo(scale.scale(0), 0, epsilon);
            });
            it("has log() behavior at values > base", function () {
                [10, 100, 23103.4, 1e+45].forEach(function (x) {
                    chai_1.assert.closeTo(scale.scale(x), Math.log(x) / Math.log(10), epsilon);
                });
            });
            it("ensures x = invert(scale(x))", function () {
                [0, 1, base, 100, 0.001, -1, -0.3, -base, base - 0.001].forEach(function (x) {
                    chai_1.assert.closeTo(x, scale.invert(scale.scale(x)), epsilon);
                    chai_1.assert.closeTo(x, scale.scale(scale.invert(x)), epsilon);
                });
            });
            it("defaults to the [0, base] domain", function () {
                chai_1.assert.deepEqual(scale.domain(), [0, base], "default domain is [0, base]");
            });
            it("can be padded", function () {
                scale.addIncludedValuesProvider(function () { return [0, base]; });
                scale.padProportion(0);
                var unpaddedDomain = scale.domain();
                scale.padProportion(0.1);
                chai_1.assert.operator(scale.domain()[0], "<", unpaddedDomain[0], "left side of domain has been padded");
                chai_1.assert.operator(unpaddedDomain[1], "<", scale.domain()[1], "right side of domain has been padded");
            });
            it("works on inverted domain", function () {
                scale.domain([200, -100]);
                scale.range([10, 20]);
                var range = scale.range();
                chai_1.assert.strictEqual(scale.scale(-100), range[1], "minimum value in domain maps to maximum value in range");
                chai_1.assert.strictEqual(scale.scale(200), range[0], "maximum value in domain maps to minimum value in range");
                var a = [-100, -10, -3, 0, 1, 3.64, 50, 60, 200];
                var b = a.map(function (x) { return scale.scale(x); });
                chai_1.assert.deepEqual(b, b.slice().sort().reverse(), "should be decreasing function; reverse is sorted");
            });
        });
        describe("Scale bases", function () {
            it("uses 10 as the default base", function () {
                var scale = new Plottable.Scales.ModifiedLog();
                scale.range([0, 1]);
                chai_1.assert.strictEqual(scale.scale(10), 1, "10 is base");
                chai_1.assert.strictEqual(scale.scale(100), 2, "10^2 will result in a double value compared to the base");
            });
            it("can scale values using base 2", function () {
                var scale = new Plottable.Scales.ModifiedLog(2);
                scale.domain([0, 16]);
                scale.range([0, 1]);
                chai_1.assert.strictEqual(scale.scale(-2), -0.25, "scales negative values");
                chai_1.assert.strictEqual(scale.scale(0), 0, "scales 0");
                chai_1.assert.strictEqual(scale.scale(2), 0.25, "scales base");
                chai_1.assert.strictEqual(scale.scale(4), 0.5, "scales other values");
                chai_1.assert.strictEqual(scale.scale(16), 1, "scales maximum value");
                chai_1.assert.strictEqual(scale.scale(256), 2, "scales values outside the domain");
                chai_1.assert.strictEqual(scale.invert(1), 16, "inverts maximum value");
                chai_1.assert.strictEqual(scale.invert(0), 0, "inverts zero");
            });
        });
        describe("Auto Domaining", function () {
            var scale;
            var base = 10;
            beforeEach(function () {
                scale = new Plottable.Scales.ModifiedLog(base);
                scale.padProportion(0);
            });
            it("expands single value domains to [value / base, value * base].sort()", function () {
                var singleValue = 15;
                scale.addIncludedValuesProvider(function () { return [singleValue]; });
                chai_1.assert.deepEqual(scale.domain(), [singleValue / base, singleValue * base], "positive single-value extent was expanded to [value / base, value * base]");
                singleValue = -15;
                scale.autoDomain();
                chai_1.assert.deepEqual(scale.domain(), [singleValue * base, singleValue / base], "negative single-value extent was expanded to [value * base, value / base]");
                singleValue = 0;
                scale.autoDomain();
                chai_1.assert.deepEqual(scale.domain(), [-base, base], "zero single-value extent was expanded to [base, -base]");
            });
            it("doesn't lock up if a zero-width domain is set while there are value providers", function () {
                scale.padProportion(0.1);
                var provider = function () { return [0, 1]; };
                scale.addIncludedValuesProvider(provider);
                scale.autoDomain();
                var originalAutoDomain = scale.domain();
                scale.domain([0, 0]);
                scale.autoDomain();
                chai_1.assert.deepEqual(scale.domain(), originalAutoDomain, "autodomained as expected");
            });
            it("can force the minimum of the domain with domainMin()", function () {
                var requestedDomain = [-5, 5];
                scale.addIncludedValuesProvider(function () { return requestedDomain; });
                var minBelowBottom = -10;
                chai_1.assert.strictEqual(scale.domainMin(minBelowBottom), scale, "the scale is returned by the setter");
                chai_1.assert.strictEqual(scale.domainMin(), minBelowBottom, "can get the domainMin()");
                chai_1.assert.deepEqual(scale.domain(), [minBelowBottom, requestedDomain[1]], "lower end of domain was set by domainMin()");
                var minInMiddle = 0;
                scale.domainMin(minInMiddle);
                chai_1.assert.deepEqual(scale.domain(), [minInMiddle, requestedDomain[1]], "lower end was set even if requested value cuts off some data");
                scale.autoDomain();
                chai_1.assert.deepEqual(scale.domain(), requestedDomain, "calling autoDomain() overrides domainMin()");
                chai_1.assert.strictEqual(scale.domainMin(), scale.domain()[0], "returns autoDomain()-ed min value after autoDomain()-ing");
                var minEqualTop = scale.domain()[1];
                scale.domainMin(minEqualTop);
                chai_1.assert.deepEqual(scale.domain(), [minEqualTop, minEqualTop * base], "domain is set to [min, min * base] if the requested value is >= autoDomain()-ed max value");
                scale.domainMin(minInMiddle);
                var requestedDomain2 = [-10, 10];
                scale.addIncludedValuesProvider(function () { return requestedDomain2; });
                chai_1.assert.deepEqual(scale.domain(), [minInMiddle, requestedDomain2[1]], "adding another ExtentsProvider doesn't change domainMin()");
            });
            it("can force the maximum of the domain with domainMax()", function () {
                var requestedDomain = [-5, 5];
                scale.addIncludedValuesProvider(function () { return requestedDomain; });
                var maxAboveTop = 10;
                chai_1.assert.strictEqual(scale.domainMax(maxAboveTop), scale, "the scale is returned by the setter");
                chai_1.assert.strictEqual(scale.domainMax(), maxAboveTop, "can get the domainMax()");
                chai_1.assert.deepEqual(scale.domain(), [requestedDomain[0], maxAboveTop], "upper end of domain was set by domainMax()");
                var maxInMiddle = 0;
                scale.domainMax(maxInMiddle);
                chai_1.assert.deepEqual(scale.domain(), [requestedDomain[0], maxInMiddle], "upper end was set even if requested value cuts off some data");
                scale.autoDomain();
                chai_1.assert.deepEqual(scale.domain(), requestedDomain, "calling autoDomain() overrides domainMax()");
                chai_1.assert.strictEqual(scale.domainMax(), scale.domain()[1], "returns autoDomain()-ed max value after autoDomain()-ing");
                var maxEqualBottom = scale.domain()[0];
                scale.domainMax(maxEqualBottom);
                chai_1.assert.deepEqual(scale.domain(), [maxEqualBottom * base, maxEqualBottom], "domain is set to [max * base, max] if the requested value is <= autoDomain()-ed min value and negative");
                scale.domainMax(maxInMiddle);
                var requestedDomain2 = [-10, 10];
                scale.addIncludedValuesProvider(function () { return requestedDomain2; });
                chai_1.assert.deepEqual(scale.domain(), [requestedDomain2[0], maxInMiddle], "adding another ExtentsProvider doesn't change domainMax()");
            });
            it("can force the domain by using domainMin() and domainMax() together", function () {
                var requestedDomain = [-5, 5];
                scale.addIncludedValuesProvider(function () { return requestedDomain; });
                var desiredMin = -10;
                var desiredMax = 10;
                scale.domainMin(desiredMin);
                scale.domainMax(desiredMax);
                chai_1.assert.deepEqual(scale.domain(), [desiredMin, desiredMax], "setting domainMin() and domainMax() sets the domain");
                scale.autoDomain();
                var bigMin = 10;
                var smallMax = -10;
                scale.domainMin(bigMin);
                scale.domainMax(smallMax);
                chai_1.assert.deepEqual(scale.domain(), [bigMin, smallMax], "setting both is allowed even if it reverse the domain");
            });
        });
        describe("Ticks", function () {
            var scale;
            var base = 10;
            beforeEach(function () {
                scale = new Plottable.Scales.ModifiedLog(base);
            });
            it("gives reasonable values for ticks()", function () {
                var includedValuesProvider = function () { return [0, base / 2]; };
                scale.addIncludedValuesProvider(includedValuesProvider);
                var ticks = scale.ticks();
                chai_1.assert.operator(ticks.length, ">", 0, "there should be some ticks generated");
                scale.removeIncludedValuesProvider(includedValuesProvider);
                includedValuesProvider = function () { return [-base * 2, base * 2]; };
                scale.addIncludedValuesProvider(includedValuesProvider);
                ticks = scale.ticks();
                var beforePivot = ticks.filter(function (x) { return x <= -base; });
                var afterPivot = ticks.filter(function (x) { return base <= x; });
                var betweenPivots = ticks.filter(function (x) { return -base < x && x < base; });
                chai_1.assert.operator(beforePivot.length, ">", 0, "there should be ticks before -base");
                chai_1.assert.operator(afterPivot.length, ">", 0, "there should be ticks after base");
                chai_1.assert.operator(betweenPivots.length, ">", 0, "there should be ticks between -base and base");
            });
            it("works on inverted domain", function () {
                scale.domain([200, -100]);
                var ticks = scale.ticks();
                chai_1.assert.deepEqual(ticks, ticks.slice().sort(function (x, y) { return x - y; }), "ticks should be sorted");
                chai_1.assert.deepEqual(ticks, Plottable.Utils.Array.uniq(ticks), "ticks should not be repeated");
                var beforePivot = ticks.filter(function (x) { return x <= -base; });
                var afterPivot = ticks.filter(function (x) { return base <= x; });
                var betweenPivots = ticks.filter(function (x) { return -base < x && x < base; });
                chai_1.assert.operator(beforePivot.length, ">", 0, "should be ticks before -base");
                chai_1.assert.operator(afterPivot.length, ">", 0, "should be ticks after base");
                chai_1.assert.operator(betweenPivots.length, ">", 0, "should be ticks between -base and base");
            });
            it("always has more than 2 ticks", function () {
                [null, [2, 9], [0, 1], [1, 2], [0.001, 0.01], [-0.1, 0.1], [-3, -2]].forEach(function (domain) {
                    scale.domain(domain);
                    var ticks = scale.ticks();
                    chai_1.assert.operator(ticks.length, ">", 2, "there should be at least 2 ticks in domain " + domain);
                });
            });
        });
    });
});
