"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var chai_1 = require("chai");
var Plottable = require("../../src");
describe("Scales", function () {
    describe("Scale", function () {
        var MockScale = (function (_super) {
            __extends(MockScale, _super);
            function MockScale(autodomainCallback) {
                if (autodomainCallback === void 0) { autodomainCallback = function () { return []; }; }
                var _this = _super.call(this) || this;
                _this._domain = [];
                _this._range = [];
                _this._autodomainCallback = autodomainCallback;
                return _this;
            }
            MockScale.prototype._getExtent = function () {
                return this._autodomainCallback();
            };
            MockScale.prototype._getDomain = function () {
                return this._backingScaleDomain();
            };
            MockScale.prototype._backingScaleDomain = function (values) {
                if (values == null) {
                    return this._domain;
                }
                else {
                    this._domain = values;
                    return this;
                }
            };
            MockScale.prototype._getRange = function () {
                return this._range;
            };
            MockScale.prototype._setRange = function (values) {
                this._range = values;
            };
            MockScale.prototype.includedValues = function () {
                return this._getAllIncludedValues();
            };
            return MockScale;
        }(Plottable.Scale));
        describe("domain and range", function () {
            it("can set and get the domain", function () {
                var scale = new MockScale();
                var expectedDomain = [0, 10];
                chai_1.assert.strictEqual(scale.domain(expectedDomain), scale, "setter mode returns the calling Scale");
                chai_1.assert.deepEqual(scale.domain(), expectedDomain, "returns the set domain");
            });
            it("can set and get the range", function () {
                var scale = new MockScale();
                var expectedRange = [0, 10];
                chai_1.assert.strictEqual(scale.range(expectedRange), scale, "setter mode returns the calling Scale");
                chai_1.assert.deepEqual(scale.range(), expectedRange, "returns the set range");
            });
        });
        describe("onUpdate", function () {
            it("alerts listeners when its domain is updated", function () {
                var scale = new MockScale();
                var expectedDomain = [0, 10];
                var callbackWasCalled = false;
                var testCallback = function (listenable) {
                    chai_1.assert.strictEqual(listenable, scale, "Callback received the calling scale as the first argument");
                    chai_1.assert.deepEqual(listenable.domain(), expectedDomain, "domain was updated bofore the callback was called");
                    callbackWasCalled = true;
                };
                scale.onUpdate(testCallback);
                scale.domain(expectedDomain);
                chai_1.assert.isTrue(callbackWasCalled, "The registered callback was called");
            });
            it("can disconnect listeners", function () {
                var scale = new MockScale();
                var callbackWasCalled = false;
                var testCallback = function (listenable) {
                    chai_1.assert.strictEqual(listenable, scale, "Callback received the calling scale as the first argument");
                    callbackWasCalled = true;
                };
                scale.onUpdate(testCallback);
                scale.domain([0, 10]);
                chai_1.assert.isTrue(callbackWasCalled, "The registered callback was called");
                callbackWasCalled = false;
                scale.offUpdate(testCallback);
                scale.domain([11, 19]);
                chai_1.assert.isFalse(callbackWasCalled, "The registered callback was not called because the callback was removed");
            });
        });
        describe("included values", function () {
            it("adding or removing an IncludedValuesProvider returns the Scale", function () {
                var scale = new MockScale();
                var valueProvider = function () { return []; };
                chai_1.assert.strictEqual(scale.addIncludedValuesProvider(valueProvider), scale, "adding the provider returns the Scale");
                chai_1.assert.strictEqual(scale.removeIncludedValuesProvider(valueProvider), scale, "removing the provider returns the Scale");
            });
            it("combines all values from its IncludedValuesProviders", function () {
                var scale = new MockScale();
                var provider1Value = { value: 1 };
                var provider1 = function (passedScale) {
                    chai_1.assert.strictEqual(passedScale, scale, "was passed the scale it was attached to");
                    return [provider1Value];
                };
                var provider2Value = { value: 2 };
                var provider2 = function () { return [provider2Value]; };
                scale.addIncludedValuesProvider(provider1);
                scale.addIncludedValuesProvider(provider2);
                var expectedIncludedValues = [provider1Value, provider2Value];
                chai_1.assert.deepEqual(scale.includedValues(), expectedIncludedValues, "combines all included values into a single array");
            });
            it("does not include values from removed IncludedValuesProviders", function () {
                var scale = new MockScale();
                var provider1Value = { value: 1 };
                var provider1 = function () { return [provider1Value]; };
                var provider2Value = { value: 2 };
                var provider2 = function () { return [provider2Value]; };
                scale.addIncludedValuesProvider(provider1);
                scale.addIncludedValuesProvider(provider2);
                scale.removeIncludedValuesProvider(provider2);
                var expectedIncludedValues = [provider1Value];
                chai_1.assert.deepEqual(scale.includedValues(), expectedIncludedValues, "includes only values from remaining providers");
            });
            it("won't add the same IncludedValuesProvider twice", function () {
                var scale = new MockScale();
                var providerValue = { value: 1 };
                var provider = function () { return [providerValue]; };
                scale.addIncludedValuesProvider(provider);
                scale.addIncludedValuesProvider(provider);
                var expectedIncludedValues = [providerValue];
                chai_1.assert.deepEqual(scale.includedValues(), expectedIncludedValues, "only includes the value once");
            });
        });
        describe("autodomaining", function () {
            it("overrides a set domain if explicitly autodomained", function () {
                var expectedDomain = [0, 99];
                var autodomainCallback = function () { return expectedDomain; };
                var scale = new MockScale(autodomainCallback);
                var explicitlySetDomain = [0, 1];
                scale.domain(explicitlySetDomain);
                scale.autoDomain();
                chai_1.assert.deepEqual(scale.domain(), expectedDomain, "returns the autodomain value by default");
            });
            it("autodomains when adding or removing an IncludedValuesProvider if in auto mode", function () {
                var autodomainCalls = 0;
                var autodomainCallback = function () {
                    autodomainCalls++;
                    return [];
                };
                var scale = new MockScale(autodomainCallback);
                var provider = function () { return []; };
                scale.addIncludedValuesProvider(provider);
                chai_1.assert.strictEqual(autodomainCalls, 1, "scale autodomained when a provider was added");
                scale.removeIncludedValuesProvider(provider);
                chai_1.assert.strictEqual(autodomainCalls, 2, "scale autodomained when a provider was removed");
            });
            it("does not autodomain when adding or removing an IncludedValuesProvider if NOT in auto mode", function () {
                var autodomainCalls = 0;
                var autodomainCallback = function () {
                    autodomainCalls++;
                    return [];
                };
                var scale = new MockScale(autodomainCallback);
                scale.domain([0, 1]);
                var provider = function () { return []; };
                scale.addIncludedValuesProvider(provider);
                chai_1.assert.strictEqual(autodomainCalls, 0, "scale did not autodomain when a provider was added");
                scale.removeIncludedValuesProvider(provider);
                chai_1.assert.strictEqual(autodomainCalls, 0, "scale dit not autodomain when a provider was removed");
            });
        });
    });
});
