"use strict";
var chai_1 = require("chai");
var d3 = require("d3");
var Plottable = require("../src");
var domUtils_1 = require("../src/utils/domUtils");
function generateDiv(width, height) {
    if (width === void 0) { width = 400; }
    if (height === void 0) { height = 400; }
    var parent = getElementParent();
    return parent.append("div").style("width", width + "px").style("height", height + "px").attr("class", "div");
}
exports.generateDiv = generateDiv;
function generateSVG(width, height) {
    if (width === void 0) { width = 400; }
    if (height === void 0) { height = 400; }
    var parent = getElementParent();
    return parent.append("svg").attr("width", width).attr("height", height).attr("class", "svg");
}
exports.generateSVG = generateSVG;
function getElementParent() {
    var mocha = d3.select("#mocha-report");
    if (mocha.node() != null) {
        var suites = mocha.selectAll(".suite");
        var lastSuite = d3.select(suites.nodes()[suites.size() - 1]);
        return lastSuite.selectAll("ul");
    }
    else {
        return d3.select("body");
    }
}
exports.getElementParent = getElementParent;
function isInDOM(component) {
    var contentNode = component.content().node();
    return contentNode != null && Plottable.Utils.DOM.contains(document.body, contentNode);
}
exports.isInDOM = isInDOM;
function verifySpaceRequest(sr, expectedMinWidth, expectedMinHeight, message) {
    chai_1.assert.strictEqual(sr.minWidth, expectedMinWidth, message + " (space request: minWidth)");
    chai_1.assert.strictEqual(sr.minHeight, expectedMinHeight, message + " (space request: minHeight)");
}
exports.verifySpaceRequest = verifySpaceRequest;
function fixComponentSize(c, fixedWidth, fixedHeight) {
    c.requestedSpace = function (w, h) {
        return {
            minWidth: fixedWidth == null ? 0 : fixedWidth,
            minHeight: fixedHeight == null ? 0 : fixedHeight,
        };
    };
    c.fixedWidth = function () { return fixedWidth == null ? false : true; };
    c.fixedHeight = function () { return fixedHeight == null ? false : true; };
    return c;
}
exports.fixComponentSize = fixComponentSize;
function makeFixedSizeComponent(fixedWidth, fixedHeight) {
    return fixComponentSize(new Plottable.Component(), fixedWidth, fixedHeight);
}
exports.makeFixedSizeComponent = makeFixedSizeComponent;
// TODO deprecate
function getTranslate(element) {
    return domUtils_1.getTranslateValues(element);
}
exports.getTranslate = getTranslate;
function assertBBoxEquivalence(bbox, widthAndHeightPair, message) {
    var width = widthAndHeightPair[0];
    var height = widthAndHeightPair[1];
    chai_1.assert.strictEqual(bbox.width, width, "width: " + message);
    chai_1.assert.strictEqual(bbox.height, height, "height: " + message);
}
exports.assertBBoxEquivalence = assertBBoxEquivalence;
function assertBBoxInclusion(outerEl, innerEl) {
    var outerBox = outerEl.node().getBoundingClientRect();
    var innerBox = innerEl.node().getBoundingClientRect();
    chai_1.assert.operator(Math.floor(outerBox.left), "<=", Math.ceil(innerBox.left) + window.Pixel_CloseTo_Requirement, "bounding rect left included");
    chai_1.assert.operator(Math.floor(outerBox.top), "<=", Math.ceil(innerBox.top) + window.Pixel_CloseTo_Requirement, "bounding rect top included");
    chai_1.assert.operator(Math.ceil(outerBox.right) + window.Pixel_CloseTo_Requirement, ">=", Math.floor(innerBox.right), "bounding rect right included");
    chai_1.assert.operator(Math.ceil(outerBox.bottom) + window.Pixel_CloseTo_Requirement, ">=", Math.floor(innerBox.bottom), "bounding rect bottom included");
}
exports.assertBBoxInclusion = assertBBoxInclusion;
function assertBBoxNonIntersection(firstEl, secondEl) {
    var firstBox = firstEl.node().getBoundingClientRect();
    var secondBox = secondEl.node().getBoundingClientRect();
    var intersectionBox = {
        left: Math.max(firstBox.left, secondBox.left),
        right: Math.min(firstBox.right, secondBox.right),
        bottom: Math.min(firstBox.bottom, secondBox.bottom),
        top: Math.max(firstBox.top, secondBox.top),
    };
    // +1 for inaccuracy in IE
    chai_1.assert.isTrue(intersectionBox.left + 1 >= intersectionBox.right || intersectionBox.bottom + 1 >= intersectionBox.top, "bounding rects are not intersecting");
}
exports.assertBBoxNonIntersection = assertBBoxNonIntersection;
function assertPointsClose(actual, expected, epsilon, message) {
    chai_1.assert.closeTo(actual.x, expected.x, epsilon, message + " (x)");
    chai_1.assert.closeTo(actual.y, expected.y, epsilon, message + " (y)");
}
exports.assertPointsClose = assertPointsClose;
;
function assertWidthHeight(el, widthExpected, heightExpected, message) {
    var width = el.attr("width");
    var height = el.attr("height");
    chai_1.assert.strictEqual(width, String(widthExpected), "width: " + message);
    chai_1.assert.strictEqual(height, String(heightExpected), "height: " + message);
}
exports.assertWidthHeight = assertWidthHeight;
function assertLineAttrs(line, expectedAttrs, message) {
    var floatingPointError = 0.000000001;
    chai_1.assert.closeTo(numAttr(line, "x1"), expectedAttrs.x1, floatingPointError, message + " (x1)");
    chai_1.assert.closeTo(numAttr(line, "y1"), expectedAttrs.y1, floatingPointError, message + " (y1)");
    chai_1.assert.closeTo(numAttr(line, "x2"), expectedAttrs.x2, floatingPointError, message + " (x2)");
    chai_1.assert.closeTo(numAttr(line, "y2"), expectedAttrs.y2, floatingPointError, message + " (y2)");
}
exports.assertLineAttrs = assertLineAttrs;
function assertEntitiesEqual(actual, expected, msg) {
    chai_1.assert.deepEqual(actual.datum, expected.datum, msg + " (datum)");
    assertPointsClose(actual.position, expected.position, 0.01, msg);
    chai_1.assert.strictEqual(actual.selection.size(), expected.selection.size(), msg + " (selection length)");
    actual.selection.nodes().forEach(function (element, index) {
        chai_1.assert.strictEqual(element, expected.selection.nodes()[index], msg + " (selection contents)");
    });
    chai_1.assert.strictEqual(actual.component, expected.component, msg + " (component)");
}
exports.assertEntitiesEqual = assertEntitiesEqual;
function assertPlotEntitiesEqual(actual, expected, msg) {
    assertEntitiesEqual(actual, expected, msg);
    chai_1.assert.strictEqual(actual.dataset, expected.dataset, msg + " (dataset)");
    chai_1.assert.strictEqual(actual.index, expected.index, msg + " (index)");
}
exports.assertPlotEntitiesEqual = assertPlotEntitiesEqual;
function makeLinearSeries(n) {
    function makePoint(x) {
        return { x: x, y: x };
    }
    return d3.range(n).map(makePoint);
}
exports.makeLinearSeries = makeLinearSeries;
function makeQuadraticSeries(n) {
    function makeQuadraticPoint(x) {
        return { x: x, y: x * x };
    }
    return d3.range(n).map(makeQuadraticPoint);
}
exports.makeQuadraticSeries = makeQuadraticSeries;
// for IE, whose paths look like "M 0 500 L" instead of "M0,500L"
function normalizePath(pathString) {
    return pathString.replace(/ *([A-Z]) */g, "$1").replace(/ /g, ",");
}
exports.normalizePath = normalizePath;
/**
 * Decomposes a normalized path-string ("d" attribute from a <path>)
 * to an array of command-argument pairs:
 * {
   *   command: string;
   *   arguments: number[];
   * }[]
 */
function decomposePath(normalizedPathString) {
    var commands = normalizedPathString.split(/[^A-Z]/).filter(function (s) { return s !== ""; });
    var argumentStrings = normalizedPathString.split(/[A-Z]/).slice(1);
    return commands.map(function (command, index) {
        return {
            command: command,
            arguments: argumentStrings[index].split(",").filter(function (s) { return s !== ""; }).map(function (s) { return parseFloat(s); }),
        };
    });
}
exports.decomposePath = decomposePath;
function numAttr(s, a) {
    return parseFloat(s.attr(a));
}
exports.numAttr = numAttr;
function triggerFakeUIEvent(type, target) {
    var e = document.createEvent("UIEvents");
    e.initUIEvent(type, true, true, window, 1);
    target.node().dispatchEvent(e);
}
exports.triggerFakeUIEvent = triggerFakeUIEvent;
function triggerFakeMouseEvent(type, target, relativeX, relativeY, button) {
    if (button === void 0) { button = 0; }
    var clientRect = target.node().getBoundingClientRect();
    var xPos = clientRect.left + relativeX;
    var yPos = clientRect.top + relativeY;
    var e = document.createEvent("MouseEvents");
    e.initMouseEvent(type, true, true, window, 1, xPos, yPos, xPos, yPos, false, false, false, false, button, null);
    target.node().dispatchEvent(e);
}
exports.triggerFakeMouseEvent = triggerFakeMouseEvent;
function triggerFakeDragSequence(target, start, end, numSteps) {
    if (numSteps === void 0) { numSteps = 2; }
    triggerFakeMouseEvent("mousedown", target, start.x, start.y);
    for (var i = 1; i < numSteps; i++) {
        triggerFakeMouseEvent("mousemove", target, start.x + (end.x - start.x) * i / numSteps, start.y + (end.y - start.y) * i / numSteps);
    }
    triggerFakeMouseEvent("mousemove", target, end.x, end.y);
    triggerFakeMouseEvent("mouseup", target, end.x, end.y);
}
exports.triggerFakeDragSequence = triggerFakeDragSequence;
function isIE() {
    var userAgent = window.navigator.userAgent;
    return userAgent.indexOf("MSIE ") > -1 || userAgent.indexOf("Trident/") > -1;
}
exports.isIE = isIE;
function triggerFakeWheelEvent(type, target, relativeX, relativeY, deltaY) {
    var clientRect = target.node().getBoundingClientRect();
    var xPos = clientRect.left + relativeX;
    var yPos = clientRect.top + relativeY;
    var event;
    if (isIE()) {
        event = document.createEvent("WheelEvent");
        event.initWheelEvent("wheel", true, true, window, 1, xPos, yPos, xPos, yPos, 0, null, null, 0, deltaY, 0, 0);
    }
    else {
        // HACKHACK anycasting constructor to allow for the dictionary argument
        // https://github.com/Microsoft/TypeScript/issues/2416
        event = new WheelEvent("wheel", { bubbles: true, clientX: xPos, clientY: yPos, deltaY: deltaY });
    }
    target.node().dispatchEvent(event);
}
exports.triggerFakeWheelEvent = triggerFakeWheelEvent;
function triggerFakeTouchEvent(type, target, touchPoints, ids) {
    if (ids === void 0) { ids = []; }
    var targetNode = target.node();
    var clientRect = targetNode.getBoundingClientRect();
    var e = document.createEvent("UIEvent");
    e.initUIEvent(type, true, true, window, 1);
    var fakeTouchList = [];
    touchPoints.forEach(function (touchPoint, i) {
        var xPos = clientRect.left + touchPoint.x;
        var yPos = clientRect.top + touchPoint.y;
        var identifier = ids[i] == null ? 0 : ids[i];
        fakeTouchList.push({
            identifier: identifier,
            target: targetNode,
            screenX: xPos,
            screenY: yPos,
            clientX: xPos,
            clientY: yPos,
            pageX: xPos,
            pageY: yPos,
        });
    });
    fakeTouchList.item = function (index) { return fakeTouchList[index]; };
    e.touches = fakeTouchList;
    e.targetTouches = fakeTouchList;
    e.changedTouches = fakeTouchList;
    e.altKey = false;
    e.metaKey = false;
    e.ctrlKey = false;
    e.shiftKey = false;
    target.node().dispatchEvent(e);
}
exports.triggerFakeTouchEvent = triggerFakeTouchEvent;
var InteractionMode;
(function (InteractionMode) {
    InteractionMode[InteractionMode["Mouse"] = 0] = "Mouse";
    InteractionMode[InteractionMode["Touch"] = 1] = "Touch";
})(InteractionMode = exports.InteractionMode || (exports.InteractionMode = {}));
;
var InteractionType;
(function (InteractionType) {
    InteractionType[InteractionType["Start"] = 0] = "Start";
    InteractionType[InteractionType["Move"] = 1] = "Move";
    InteractionType[InteractionType["End"] = 2] = "End";
})(InteractionType = exports.InteractionType || (exports.InteractionType = {}));
function triggerFakeInteractionEvent(mode, type, target, relativeX, relativeY) {
    var typeString = getInteractionTypeString(mode, type);
    switch (mode) {
        case InteractionMode.Mouse:
            triggerFakeMouseEvent(typeString, target, relativeX, relativeY);
            break;
        case InteractionMode.Touch:
            triggerFakeTouchEvent(typeString, target, [{ x: relativeX, y: relativeY }]);
            break;
        default:
            throw new Error("Unrecognized enum value: " + mode);
    }
}
exports.triggerFakeInteractionEvent = triggerFakeInteractionEvent;
/* tslint:disable:no-switch-case-fall-through */
function getInteractionTypeString(mode, type) {
    switch (mode) {
        case InteractionMode.Mouse:
            switch (type) {
                case InteractionType.Start:
                    return "mousedown";
                case InteractionType.Move:
                    return "mousemove";
                case InteractionType.End:
                    return "mouseup";
                default:
                    throw new Error("Unrecognized enum value: " + type);
            }
        case InteractionMode.Touch:
            switch (type) {
                case InteractionType.Start:
                    return "touchstart";
                case InteractionType.Move:
                    return "touchmove";
                case InteractionType.End:
                    return "touchend";
                default:
                    throw new Error("Unrecognized enum value: " + type);
            }
        default:
            throw new Error("Unrecognized enum value: " + mode);
    }
}
/* tslint:enable:no-switch-case-fall-through */
function triggerFakeKeyboardEvent(type, target, keyCode, options) {
    var event = document.createEvent("Events");
    event.initEvent(type, true, true);
    event.keyCode = keyCode;
    if (options != null) {
        Object.keys(options).forEach(function (key) { return event[key] = options[key]; });
    }
    target.node().dispatchEvent(event);
}
exports.triggerFakeKeyboardEvent = triggerFakeKeyboardEvent;
function assertAreaPathCloseTo(actualPath, expectedPath, precision, msg) {
    var actualAreaPathNumbers = tokenizePathString(actualPath);
    var expectedAreaPathNumbers = tokenizePathString(expectedPath);
    chai_1.assert.lengthOf(actualAreaPathNumbers, expectedAreaPathNumbers.length, msg + ": number of numbers in each path should be equal");
    actualAreaPathNumbers.forEach(function (actualAreaNumber, i) {
        var expectedAreaNumber = expectedAreaPathNumbers[i];
        chai_1.assert.closeTo(+actualAreaNumber, +expectedAreaNumber, 0.1, msg);
    });
}
exports.assertAreaPathCloseTo = assertAreaPathCloseTo;
function tokenizePathString(pathString) {
    var numbers = [];
    pathString.split("Z").forEach(function (path) {
        return path.split(/[A-Z]/).forEach(function (token) {
            return token.split(",").forEach(function (numberString) {
                return numberString.split(" ").forEach(function (num) {
                    if (num !== "") {
                        numbers.push(num);
                    }
                });
            });
        });
    });
    return numbers;
}
function colorHexToRGB(hex) {
    return {
        red: parseInt(hex.substr(1, 2), 16),
        green: parseInt(hex.substr(3, 2), 16),
        blue: parseInt(hex.substr(5, 2), 16),
    };
}
exports.colorHexToRGB = colorHexToRGB;
function colorRGBToHex(rgb) {
    var redHex = rgb.red.toString(16);
    if (redHex.length === 1) {
        redHex = "0" + redHex;
    }
    var greenHex = rgb.blue.toString(16);
    if (greenHex.length === 1) {
        greenHex = "0" + greenHex;
    }
    var blueHex = rgb.green.toString(16);
    if (blueHex.length === 1) {
        blueHex = "0" + blueHex;
    }
    return "#" + redHex + greenHex + blueHex;
}
exports.colorRGBToHex = colorRGBToHex;
function assertWarns(fn, warningMessage, assertMessage) {
    var receivedWarning = "";
    var oldWarn = Plottable.Utils.Window.warn;
    Plottable.Utils.Window.warn = function (msg) { return receivedWarning = msg; };
    try {
        fn.call(this);
    }
    finally {
        Plottable.Utils.Window.warn = oldWarn;
    }
    chai_1.assert.include(receivedWarning, warningMessage, assertMessage);
}
exports.assertWarns = assertWarns;
function areaVertices(areaSelection) {
    var areaPathString = normalizePath(areaSelection.attr("d")).slice(1, -1);
    return areaPathString.split("L")
        .map(function (d) {
        var pointArray = d.trim().split(",");
        return {
            x: +pointArray[0],
            y: +pointArray[1],
        };
    });
}
exports.areaVertices = areaVertices;
function assertPathEqualToDataPoints(path, data, xScale, yScale) {
    var EPSILON = 0.0001;
    var lineEdges = normalizePath(path).match(/([MmLl](\-?\d+\.?\d*)(,|\s)(-?\d+\.?\d*)|([HhVv](\-?\d+\.?\d*)))/g);
    chai_1.assert.strictEqual(lineEdges.length, data.length, "correct number of edges drawn");
    lineEdges.forEach(function (edge, i) {
        var command = edge[0];
        var coordinates = edge.slice(1).split(/,|\s/);
        switch (command) {
            case "M":
            case "m":
            case "L":
            case "l":
                chai_1.assert.strictEqual(coordinates.length, 2, "There is an x coordinate and a y coordinate");
                assertPointsClose({ x: xScale.invert(+coordinates[0]), y: yScale.invert(+coordinates[1]) }, data[i], EPSILON, "Point " + i + " drawn, has correct coordinates");
                break;
            case "H":
            case "h":
                chai_1.assert.strictEqual(coordinates.length, 1, "There is an x coordinate");
                chai_1.assert.closeTo(xScale.invert(+coordinates[0]), data[i].x, EPSILON, "Point " + i + " drawn, has correct x coordinate");
                break;
            case "V":
            case "v":
                chai_1.assert.strictEqual(coordinates.length, 1, "There is a y coordinate");
                chai_1.assert.closeTo(yScale.invert(+coordinates[0]), data[i].y, EPSILON, "Point " + i + " drawn, has correct y coordinate");
                break;
        }
    });
}
exports.assertPathEqualToDataPoints = assertPathEqualToDataPoints;
